/// @file acumon/acquisition_info.h

// Copyright (c) 2007-2010 Acumem AB.  All rights reserved.

#ifndef ACQUISITION_INFO_HH
#define ACQUISITION_INFO_HH

#include "library/acquisition/acquisition.h"

#include "os_dependent_types.hh"

/* Opens the acquisition info file, must be called before any of the
   other functions below. If not called, the functions will be no-ops. */
acq_handle_t *open_acquisition_info(const char *acquisition_info_path);

/* Closes the acquisition info file. */
void close_acquisition_info(acq_handle_t *acq_handle);

void append_acquisition_mappings(acq_handle_t *acq_handle,
				 am_process_t *process);

/* Creates the acquisition info file and writes an acquisition start entry to
   it. Must be called while the sampled process is still alive.

   The tid argument must be the tid of a live thread on Linux, since
   /proc/<pid>/exe does not work if the initial thread of the process has
   terminated. It is unused on Solaris. */
void write_acquisition_start(acq_handle_t *acq_handle,
			     am_process_t *process,
			     tid_t tid);

/* Append an acquisition end entry to the acquisition info file. May be called
   after the sampled process has terminated. */
void append_acquisition_end(acq_handle_t *acq_handle);

/* Append a burst start entry. */
void append_burst_start(acq_handle_t *acq_handle,
			unsigned int burst_generation,
			uint64_t burst_sampling_length,
			uint64_t burst_cooldown_length);

/* Append a burst end entry. */
void append_burst_end(acq_handle_t *acq_handle,
		      unsigned int burst_generation);

#endif /* ACQUISITION_INFO_HH */
