/// @file acumon/am_assert.c

// Copyright (c) 2008 Acumem AB.  All rights reserved.

#include <errno.h>

#include "library/version/version.h"

#include "acumon.hh"
#include "am_assert.hh"

static void
print_additional_diagnostics()
{
    char build[20];
    char tag[20];
    version_get_svnversion(build, sizeof(build));
    version_get_svnpath_last_component(tag, sizeof(tag));
    am_printf(AM_Ctrl_Msg_Error,
	      "Additional information: \n"
	      "  Build: %s\n"
	      "  Compilation time: " __DATE__ " " __TIME__ "\n"
	      "  Tag: %s\n",
	      build, tag);

}

void
am_assert_fail(const char *assertion, const char *file,
	       int line, const char *function)
{
    am_printf(AM_Ctrl_Msg_Error,
	      "%s:%i: %s: Assertion `%s' failed.\n",
	      file, line, function, assertion);

    print_additional_diagnostics();
    abort();
}

void
am_assert_perror_fail(const char *assertion, const char *file,
		      int line, const char *function)
{
    char err_buf[256];
    int err = errno;

    if (strerror_r(err, err_buf, sizeof(err_buf)) == 0)
	am_printf(AM_Ctrl_Msg_Error,
		  "%s:%i: %s: Assertion `%s' failed: %s\n",
		  file, line, function, assertion, err_buf);
    else
	am_printf(AM_Ctrl_Msg_Error,
		  "%s:%i: %s: Assertion `%s' failed: Unknown error %i.\n",
		  file, line, function, assertion, err);

    print_additional_diagnostics();
    abort();
}

void
am_illegal_state(const char *file, int line, const char *function)
{
    am_printf(AM_Ctrl_Msg_Error,
	      "%s:%i: %s: Entered an illegal state, aborting.\n",
	      file, line, function);

    print_additional_diagnostics();
    abort();
}
