/// @file acumon/am_assert.h

// Copyright (c) 2008, 2010 Acumem AB.  All rights reserved.

#ifndef AM_ASSERT_H
#define AM_ASSERT_H

#include "library/lang/common_macros.h"

/* Like assert(3), but outputs the error message and some additional
   diagnostics using am_printf. */
#define AM_ASSERT(c)							\
    do {								\
	if (!(c))							\
	    am_assert_fail(# c, __FILE__, __LINE__, __func__);		\
    } while(0)

/* Like AM_ASSERT, but outputs includes errno. */
#define AM_ASSERT_WITH_ERRNO(c)						\
    do {								\
	if (!(c))							\
	    am_assert_perror_fail(# c, __FILE__, __LINE__, __func__);	\
    } while(0)

/* Behaves like using AM_ASSERT(0), but outputs a different
   error message. */
#define AM_ILLEGAL_STATE()						\
    do {								\
	am_illegal_state(__FILE__, __LINE__, __func__);			\
    } while(0)

/* Called if an assertion fails, prints an error message using
   am_printf and calls abort(3). */
void am_assert_fail(const char *assertion, const char *file,
		    int line, const char *function)
    __attribute__((__noreturn__));

/* Called if an assertion fails, prints an error message including
   errno using am_printf and calls abort(3). */
void am_assert_perror_fail(const char *assertion, const char *file,
			   int line, const char *function)
    __attribute__((__noreturn__));

/* Called from the AM_ILLEGAL_STATE macro to print an error message
   using am_printf and abort using abort(3). */
void am_illegal_state(const char *file, int line, const char *function)
    __attribute__((__noreturn__));

#endif /* AM_ASSERT_H */
