/// @file acumon/event.h

// Copyright (c) 2007-2008, 2010 Acumem AB.  All rights reserved.

#ifndef EVENT_HH
#define EVENT_HH

/* Time in nanoseconds. */
typedef int64_t am_time_t;

/* Add an event handler that is called when any of the specified poll events
   occur on the file descriptor. The event handler is passed the triggered
   events and the specified data pointer. */
void add_fd_event(int fd, short events,
                  void (*handler)(short events, void *data), void *data);

/* Remove the event for the specified arguments. */
void remove_fd_event(int fd, short events,
                     void (*handler)(short events, void *data), void *data);

/* Add an event handler that is called after the specified time. The event
   handler is passed the specified data pointer. */
void add_time_event(am_time_t timeout,
                    void (*handler)(void *data), void *data);

/* Remove the time event with the specified handler and data. */
void remove_time_event(void (*handler)(void *data), void *data);

/* Remove all timer events associated with specified data, typically the
   process object */
void remove_time_events_with_data(void *data);

/* Wait until an event has occurred or timeout ms have passed. Give a timeout
   of -1 for an infinite timeout. Returns 1 if an event has occurred, 0 if
   timeout ms passed without any event. */
void wait_for_event(void);

/* Initialize the event handling code. */
void event_init(void);

/* Free all resource allocated by the event handling code. All events must
   aalready be removed. */
void event_fini(void);

#endif /* EVENT_HH */
