/// @file acumon/instrumenter_loader.h

// Copyright (c) 2008, 2010 Acumem AB.  All rights reserved.

#ifndef INSTRUMENTER_LOADER_H
#define INSTRUMENTER_LOADER_H

#include "protocol.h"

BEGIN_EXTERN_C;

/* Struct written to the hijacked page when loading the instrumenter.

   Must be identical for 32-bit and 64-bit target. */
typedef struct {
    /* Code buffer, must be first on the page. The start of this buffer is the
       code entry point. */
    am_code_buf_t code;

    /* Amount of memory to preallocate, setting this to 0 disables
       preallocation (should only be done in 32-bit mode!). */
    uint64_t preallocation_size;

    /* The size of the memory region needed for the instrumenter. */
    uint64_t instr_size;

    /* The path to the the unix domain socket for communication with the
       monitor. */
    am_path_buf_t socket_path;

    /* 32-bit Solaris syscalls take arguments in memory. Since we don't have
       any writeable memory before the initial stack mmap, we allocate memory
       for the mmap arguments here. The monitor fills in these arguments so
       that we can use them to do the mmap.

       We also use these arguments in the 32-bit Linux instrumenter loader for
       convenience. */
    syscall_args_t mmap_syscall_args;

    /* A "load ready" message to send once the instrumenter load is
       finished. The syscall arguments needed to send it are stored in
       sendto_syscall_args. */
    am_message_t load_ready_msg;

    /* 32-bit Solaris syscalls and 32-bit Linux socketcalls take arguments in
       memory. Since we don't have any writeable memory after unmapping the
       stack, we allocate memory for the arguments to the sendto syscall to
       send load_ready_msg here and fill it in from the monitor.

       We also use these arguments in the 64-bit instrumenter loaders for
       convenience. */
    syscall_args_t sendto_syscall_args;
} instrumenter_loader_page_t;

END_EXTERN_C;

#endif /* INSTRUMENTER_LOADER_H */
