/// @file acumon/instrumenter_unloader.h

// Copyright (c) 2008, 2010 Acumem AB.  All rights reserved.

#ifndef INSTRUMENTER_UNLOADER_H
#define INSTRUMENTER_UNLOADER_H

#include "protocol.h"

#include "library/lang/common_macros.h"

BEGIN_EXTERN_C

/* Struct written to the hijacked page when unloading the instrumenter.

   Must be identical for 32-bit and 64-bit target. */
typedef struct {
    /* Code buffer, must be first on the page. The start of this buffer is the
       code entry point. */
    am_code_buf_t code;

    /* The unload entry point of the instrumenter image. */
    uint64_t unload_entry;

    /* The address of the instrumenter image. */
    uint64_t instr_addr;

    /* The size of the instrumenter image, or the preallocation if it was
       active during bootstrapping. */
    uint64_t instr_size;

    /* 32-bit Solaris syscalls take arguments in memory. Since we don't have
       any writeable memory before the initial stack mmap, we allocate memory
       for the mmap arguments here. The monitor fills in these arguments so
       that we can use them to do the mmap.

       We also use these arguments in the 32-bit Linux instrumenter unloader for
       convenience. */
    syscall_args_t mmap_syscall_args;

    /* 32-bit Solaris syscalls take arguments in memory. Since we don't have
       any writeable memory after unmapping the stack, we allocate memory for
       the arguments to the close syscall here and fill them in from the
       monitor.

       We also use these arguments in the other instrumenter unloaders for
       convenience. */
    syscall_args_t close_syscall_args;
} instrumenter_unloader_page_t;

END_EXTERN_C

#endif /* INSTRUMENTER_UNLOADER_H */
