/// @file acumon/pages.c

// Copyright (c) 2008, 2012 Rogue Wave Software, Inc.  All rights reserved.

#include <string.h>

#include "library/am_mem/am_mem.h"

#include "am_assert.hh"
#include "pages.hh"

void
pages_t::protect(unsigned long addr)
{
    unsigned long bitmap_addr = addr & (-1UL << (BITMAP_BITS_LOG2 + PAGE_SIZE_LOG2));
    unsigned long hash = (addr >> (BITMAP_BITS_LOG2 + PAGE_SIZE_LOG2)) % BITMAP_HASH_SIZE;

    page_bitmap_t *bitmap = bitmap_hash[hash];
    while (bitmap != NULL) {
        if (bitmap->addr == bitmap_addr)
            break;
        bitmap = bitmap->next;
    }

    if (bitmap == NULL) {
        bitmap = (page_bitmap_t *)am_malloc(sizeof(page_bitmap_t));
        bitmap->addr = bitmap_addr;
        memset(bitmap->bitmap, 0, sizeof(bitmap->bitmap));
        bitmap->next = bitmap_hash[hash];
        bitmap_hash[hash] = bitmap;
    }

    unsigned long pfn = addr >> PAGE_SIZE_LOG2;
    unsigned long ci = (pfn / CHAR_BITS) & (BITMAP_CHARS - 1);
    unsigned long bi = pfn & (CHAR_BITS - 1);
    AM_ASSERT((bitmap->bitmap[ci] & (1 << bi)) == 0);
    bitmap->bitmap[ci] |= 1 << bi;
}

void
pages_t::unprotect(unsigned long addr)
{
    unsigned long bitmap_addr = addr & (-1UL << (BITMAP_BITS_LOG2 + PAGE_SIZE_LOG2));
    unsigned long hash = (addr >> (BITMAP_BITS_LOG2 + PAGE_SIZE_LOG2)) % BITMAP_HASH_SIZE;

    page_bitmap_t *bitmap = bitmap_hash[hash];
    while (bitmap != NULL) {
        if (bitmap->addr == bitmap_addr)
            break;
        bitmap = bitmap->next;
    }

    AM_ASSERT(bitmap != NULL);

    unsigned long pfn = addr >> PAGE_SIZE_LOG2;
    unsigned long ci = (pfn / CHAR_BITS) & (BITMAP_CHARS - 1);
    unsigned long bi = pfn & (CHAR_BITS - 1);
    AM_ASSERT((bitmap->bitmap[ci] & (1 << bi)) != 0);
    bitmap->bitmap[ci] &= ~(1 << bi);
}

int
pages_t::is_protected(unsigned long addr)
{
    unsigned long bitmap_addr = addr & (-1UL << (BITMAP_BITS_LOG2 + PAGE_SIZE_LOG2));
    unsigned long hash = (addr >> (BITMAP_BITS_LOG2 + PAGE_SIZE_LOG2)) % BITMAP_HASH_SIZE;

    page_bitmap_t *bitmap = bitmap_hash[hash];
    while (bitmap != NULL) {
        if (bitmap->addr == bitmap_addr)
            break;
        bitmap = bitmap->next;
    }

    if (bitmap == NULL)
        return 0;

    unsigned long pfn = addr >> PAGE_SIZE_LOG2;
    unsigned long ci = (pfn / CHAR_BITS) & (BITMAP_CHARS - 1);
    unsigned long bi = pfn & (CHAR_BITS - 1);
    return (bitmap->bitmap[ci] & (1 << bi)) != 0;
}

pages_t::pages_t()
{
    for (int i = 0; i < BITMAP_HASH_SIZE; i++)
        bitmap_hash[i] = NULL;
}

pages_t::~pages_t()
{
    for (int i = 0; i < BITMAP_HASH_SIZE; i++)
        while (bitmap_hash[i] != NULL) {
            page_bitmap_t *next = bitmap_hash[i]->next;
            am_free(bitmap_hash[i]);
            bitmap_hash[i] = next;
        }
}
