/// @file acumon/pages.h

// Copyright (c) 2008, 2012 Rogue Wave Software, Inc.  All rights reserved.

#ifndef ACUMON_PAGES_HH
#define ACUMON_PAGES_HH

/** Bitmap of protected pages. */
class pages_t {
public:
    pages_t();
    ~pages_t();

    /** Mark a page as protected. */
    void protect(unsigned long addr);

    /** Mark a page as unprotected. */
    void unprotect(unsigned long addr);

    /** Returns true if the page is marked as protected, false otherwise. */
    int is_protected(unsigned long addr);

private:
    static const int PAGE_SIZE_LOG2 = 12;

    /** number of pages to group together in one page_bitmap_t.  The
	number 19 means 512k pages, or a 2GB address range, and the
	corresponding bitmap will occupy 64kb */
    static const int BITMAP_BITS_LOG2 = 19;
    static const int BITMAP_BITS = 1L << BITMAP_BITS_LOG2;
    static const int CHAR_BITS = 8;
    static const int BITMAP_CHARS = BITMAP_BITS / CHAR_BITS;

    /** Size of hash of bitmap segments */
    static const int BITMAP_HASH_SIZE = 0x100;

    /** A bitmap for a sequence of consecutive pages */
    struct page_bitmap_t {
	/** The base address of the region covered by this bitmap. */
	unsigned long addr;

	/** Bitmap with one bit per page. Bits are 1 for protected
	    pages and 0 for unprotected pages. */
	char bitmap[BITMAP_CHARS];

	/** Next bitmap in collision list. */
	page_bitmap_t *next;
    };

    /** Hash table of page bitmaps. Indexed by:
	(addr >> (BITMAP_BITS_LOG2 + PAGE_SIZE_LOG2)) % BITMAP_HASH_SIZE */
    page_bitmap_t *bitmap_hash[BITMAP_HASH_SIZE];
};


#endif /* PAGES_HH */
