/// @file acumon/safe_io.h

// Copyright (c) 2007-2008, 2010 Acumem AB.  All rights reserved.

#ifndef SAFE_IO_HH
#define SAFE_IO_HH

#include <unistd.h>
#include <poll.h>
#include <sys/socket.h>

int safe_open(const char *path, int flags);

int safe_close(int fd);

ssize_t safe_read(int fd, void *buf, size_t nbytes);

ssize_t safe_write(int fd, const void *buf, size_t nbytes);

off_t safe_lseek(int fd, off_t offset, int whence);

int safe_poll(struct pollfd *fds, nfds_t nfds, int timeout);

pid_t safe_waitpid(pid_t pid, int *status, int options);

ssize_t safe_send(int s, const void *buf, size_t len, int flags);
ssize_t safe_sendmsg(int socket, const struct msghdr *message, int flags);

ssize_t safe_recv(int s, void *buf, size_t len, int flags);

int safe_accept(int s, struct sockaddr *addr, socklen_t *addrlen);

int safe_connect(int s, const struct sockaddr *addr, socklen_t addrlen);

#endif /* SAFE_IO_HH */
