/// @file acumon/signal_handling.h

// Copyright (c) 2007-2008, 2010 Acumem AB.  All rights reserved.

#ifndef SIGNAL_HANDLING_HH
#define SIGNAL_HANDLING_HH

#include "library/lang/common_macros.h"

/* Add a handler that is called directly from the signal handler when the
   process receives the specified signal. */
void add_signal_handler(int signo, void (*handler)(void));

/* Add a handler for an event that is normally fatal. Similar to
   add_signal_handler(), but uses the SA_RESETHAND flag to prevent
   infinite signal recursion in fatal signals such as SIGABRT. */
void add_fatal_signal_handler(int signo, void (*handler)(void));

/* Remove a signal handler. */
void remove_signal_handler(int signo, void (*handler)(void));

/* Add a handler that is called through a fd event (from the poll loop) when
   the process receives the specified signal. */
void add_signal_event(int signo, void (*handler)(void));

/* Remove a signal event handler. */
void remove_signal_event(int signo, void (*handler)(void));

/* Initialize the signal handling code. */
void signal_handling_init(void);

/* Free all resource allocated by the event handling code. All signal handlers
   and events must already be removed. */
void signal_handling_fini(void);

/* Prepare the signal handling state for launching a process. Restores the
   signal mask and all signal actions to their original values. */
void signal_prepare_for_launch(void);

#endif /* SIGNAL_HANDLING_HH */
