/// @file acumon/thread_deinstrumenter.h

// Copyright (c) 2008, 2010 Acumem AB.  All rights reserved.

#ifndef THREAD_DEINSTRUMENTER_H
#define THREAD_DEINSTRUMENTER_H

BEGIN_EXTERN_C

/* Struct written to the hijacked page when deinstrumenting a thread.

   Must be identical for 32-bit and 64-bit target. */
typedef struct {
    /* Code buffer, must be first on the page. The start of this buffer block
       is the code entry point. */
    am_code_buf_t code;

    /* The start address of the instrumenter stack of this thread. */
    uint64_t stack_addr;

    /* A "deinstr done 2" message to send once the debootstrapping is
       finished. The syscall arguments needed to send it are stored in
       syscall_args. */
    am_message_t deinstr_done_2_msg;

    /* 32-bit Solaris syscalls and 32-bit Linux socketcalls take arguments in
       memory. Since we don't have any writeable memory after unmapping the
       stack, we allocate memory for the arguments to the sendto syscall to
       send deinstr_done_2_msg here and fill it in from the monitor.

       We also use these arguments in the 64-bit thread deinstrumenters for
       convenience. */
    syscall_args_t sendto_syscall_args;
} thread_deinstrumenter_page_t;

END_EXTERN_C

#endif /* THREAD_DEINSTRUMENTER_H */
