/// @file acumon/thread_instrumenter.h

// Copyright (c) 2008-2010 Acumem AB.  All rights reserved.

#ifndef THREAD_INSTRUMENTER_H
#define THREAD_INSTRUMENTER_H

BEGIN_EXTERN_C

/* Struct written to the hijacked page when instrumenting a thread.

   Must be identical for 32-bit and 64-bit target. */
typedef struct {
    /* Code buffer, must be first on the page. The start of this buffer block
       is the code entry point. */
    am_code_buf_t code;

    /* The register contents of the thread when it was stopped. */
    am_registers_t regs;

    /* True if this thread has not been instrumented before.

       When burst sampling, the instrumenter has no way of knowing whether a
       thread is the same thread as one with the same id in a previous
       burst. */
    uint64_t new_thread;

    /* Burst generation. */
    uint64_t burst_generation;

    /* The instrumentation entry point of the instrumenter image. */
    uint64_t instrumentation_entry;

    /* 32-bit Solaris syscalls take arguments in memory. Since we don't have
       any writeable memory before the initial stack mmap, we allocate memory
       for the mmap arguments here. The monitor fills in these arguments so
       that we can use them to do the mmap.

       We also use these arguments in the 32-bit Linux instrumenter loader for
       convenience. */
    syscall_args_t mmap_syscall_args;
} thread_instrumenter_page_t;

END_EXTERN_C

#endif /* THREAD_INSTRUMENTER_H */
