/// @file analysis/advice/location.hh

// Copyright (c) 2006-2008 Acumem AB.  All rights reserved.

#ifndef LOCATION_HH
#define LOCATION_HH

#include "analysis/statistics/stats_slicer.hh"
#include "analysis/structure/loop_depgroups.hh"

#include "analysis/structure/loop_identification.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace std;

using boost::shared_ptr;

/// Information about a location.
class prog_location_t {
public:
    prog_location_t(sliced_stats_cptr_t stats,
                    fnr_access_classes_t access_classes)
        : the_stats(stats),
          the_access_classes(access_classes)
        {}

    const sliced_stats_cptr_t & stats() const {
	return the_stats;
    }
    const fnr_access_classes_t &access_classes() const {
	return the_access_classes;
    }

private:
    sliced_stats_cptr_t the_stats;
    fnr_access_classes_t the_access_classes;
};

/// Vector of locations, indexed by location.
typedef vector<prog_location_t> prog_location_vector_t;

/// Information about a dependence group.
class prog_depgroup_t {
public:
    prog_depgroup_t(const location_set_t &pc_set,
		    const sliced_stats_cptr_t &stats)
        : the_pc_set(pc_set),
	  the_stats(stats)
        {}

    const location_set_t &pc_set() const {
	return the_pc_set;
    }
    const sliced_stats_cptr_t &stats() const {
	return the_stats;
    }

private:
    location_set_t the_pc_set;
    sliced_stats_cptr_t the_stats;
};

/// Vector of dependence groups, indexed by dependence group id.
typedef vector<prog_depgroup_t> prog_depgroup_vector_t;

/// prog_loop_t defines a loop, gives it a name and identifies its groups.
class prog_loop_t {
public:
    prog_loop_t(const loop_ptr_t &loop,
		const sliced_stats_cptr_t &stats,
                const depgroup_id_vector_t &depgroup_ids)
        : the_loop(loop),
          the_stats(stats),
          the_depgroup_ids(depgroup_ids)
        {}

    loop_ptr_t loop() const {
	return the_loop;
    }
    const sliced_stats_cptr_t & stats() const {
	return the_stats;
    }
    const depgroup_id_vector_t & depgroup_ids() const {
	return the_depgroup_ids;
    }

private:
    loop_ptr_t the_loop;
    sliced_stats_cptr_t the_stats;
    depgroup_id_vector_t the_depgroup_ids;
};

typedef vector<prog_loop_t> prog_loop_vector_t;

/// This struct carries the program's structure in terms of loops, dependence
/// groups and locations.
struct program_structure_t {
    /// Loops. Indexed by loop id - 1, since loop 0 (dummy loop for
    /// instructions that are not included in any real loop) is not included.
    prog_loop_vector_t loops;
    prog_depgroup_vector_t depgroups;
    prog_location_vector_t locations;
};

typedef shared_ptr<program_structure_t> program_structure_ptr_t;
typedef shared_ptr<const program_structure_t> program_structure_cptr_t;

END_NAMESPACE;
END_NAMESPACE;

#endif
