/// @file analysis/call_stack/full_stack.hh

// Copyright (c) 2007, 2009-2010 Acumem AB.  All rights reserved.

#ifndef FULL_STACK_HH
#define FULL_STACK_HH

#include "analysis/call_stack/sorted_vector.hh"
#include "library/fenris/fnr_types.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace acumem::library;

// ----------------------------------------------------------------------

// forward declaration of a type used by full_stack_node_t.
struct full_stack_children_vector_t;

// ----------------------------------------------------------------------

/// Implement a trie for call stacks. Each node fans out to all the
/// different parent call stack frames.
///
/// XXX This class makes use of unorthodox copying of pointers. A call
/// to the destroy method is required to free memory. This can't
/// easily be put in the destructor, since that would require
/// life-time control and proper ownership passing (since these
/// objects are copied in and out of stl containers, their destructor
/// will be called at times).
struct full_stack_node_t {
    full_stack_node_t(const fnr_address_t &pc = fnr_address_t())
	: pc(pc), count(0), children(0)
	{}

    void destroy();
    full_stack_node_t &find_or_create_node(const fnr_address_t &pc);

    bool operator < (const full_stack_node_t &that) const {
        return pc < that.pc;
    }

    bool operator == (const full_stack_node_t &that) const {
        return pc == that.pc;
    }

    void deposit(const fnr_address_vector_t &stack,
                 fnr_stack_depth_t level);

    fnr_address_t pc;
    int count;
    full_stack_children_vector_t *children;
};

// ----------------------------------------------------------------------

/// Implements a sorted vector of stack nodes.
struct full_stack_children_vector_t
    : public codeproject::sorted_vector<full_stack_node_t>
{
};

END_NAMESPACE;
END_NAMESPACE;

#endif
