/// @file analysis/exe_maps/exe_maps.hh

// Copyright (c) 2007-2008 Acumem AB.  All rights reserved.

#ifndef EXE_MAPS_HH
#define EXE_MAPS_HH

#include <vector>
#include <boost/shared_ptr.hpp>

#include "library/lang/common_macros.h"
#include "analysis/feature_base/feature_base.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace std;

using boost::shared_ptr;

struct exe_map_t {
    exe_map_t(uint64_t address, uint64_t size,
              const string &filename, uint64_t offset)
	: address(address), size(size),
          filename(filename), offset(offset), ignore(false) {}

    uint64_t address;
    uint64_t size;
    string filename;
    uint64_t offset;
    bool ignore;
};

typedef vector<exe_map_t> exe_map_vector_t;

struct exe_maps_result_t {
    exe_map_vector_t exe_maps;
};

typedef shared_ptr<exe_maps_result_t> exe_maps_result_ptr_t;
typedef shared_ptr<const exe_maps_result_t> exe_maps_result_cptr_t;

class exe_maps_t : feature_t<exe_maps_result_cptr_t> {
public:
    static exe_maps_result_cptr_t calculate(
        feature_context_ptr_t context,
        parser_result_cptr_t parser_result);

    exe_maps_t(feature_context_ptr_t context,
               const string &sample_file);
    exe_maps_result_cptr_t get() const;

protected:
    string name() const;
    exe_maps_result_cptr_t calculate() const;

private:
    const string sample_file;
};

END_NAMESPACE;
END_NAMESPACE;

#endif
