/// @file analysis/feature_base/exceptions.cc

// Copyright (c) 2006-2008 Acumem AB.  All rights reserved.

#include "exceptions.hh"
#include <string.h>
#include <boost/lexical_cast.hpp>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace std;

feature_exception_t::feature_exception_t(const string &feature,
					 const string &reason)
{
    this->feature = feature;
    this->reason = reason;
}

feature_exception_t::~feature_exception_t() throw()
{
}

string
feature_exception_t::what() const throw()
{
    return where() + ": " + why();
}

string
feature_exception_t::where() const throw()
{
    return feature;
}

string
feature_exception_t::why() const throw()
{
    return reason;
}

feature_errno_t::feature_errno_t(const string &feature,
				 const string &operation,
				 int error)
    : feature_exception_t(feature, operation)
{
    this->error = error;
}

string
feature_errno_t::why() const throw()
{
    return feature_exception_t::why() + " - " + strerror(error);
}

feature_samples_missing_exception_t::feature_samples_missing_exception_t(
    const string &feature,
    const string &operation)
    : feature_exception_t(feature, operation)
{
}

feature_byte_samples_missing_exception_t::feature_byte_samples_missing_exception_t(
    const string &feature,
    const string &operation)
    : feature_samples_missing_exception_t(feature, operation)
{
}

feature_line_samples_missing_exception_t::feature_line_samples_missing_exception_t(
    const string &feature,
    const string &operation,
    const fnr_line_size_t line_size)
    : feature_samples_missing_exception_t(feature, operation),
      line_size(line_size)
{
}

string
feature_line_samples_missing_exception_t::why() const throw()
{
    return feature_exception_t::why() +
	" - No samples of line size " +
	boost::lexical_cast<string>(line_size) + ".";
}

feature_page_samples_missing_exception_t::feature_page_samples_missing_exception_t(
    const string &feature,
    const string &operation)
    : feature_samples_missing_exception_t(feature, operation)
{
}

END_NAMESPACE;
END_NAMESPACE;
