/// @file analysis/feature_base/exceptions.hh

// Copyright (c) 2006-2008 Acumem AB.  All rights reserved.

#ifndef EXCEPTIONS_HH
#define EXCEPTIONS_HH

#include "library/lang/common_macros.h"
#include "library/fenris/fnr_types.hh"

#include <string>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace std;
using namespace acumem::library;

class feature_exception_t {
public:
    feature_exception_t(const string &feature,
			const string &reason);
    virtual ~feature_exception_t() throw();
    virtual string what() const throw();
    virtual string where() const throw();
    virtual string why() const throw();
protected:
    string feature;
    string reason;
};

class feature_errno_t : public feature_exception_t {
public:
    feature_errno_t(const string &feature,
		    const string &operation,
		    int error);
    string why() const throw();
protected:
    int error;
};

class feature_samples_missing_exception_t
    : public feature_exception_t {
public:
    feature_samples_missing_exception_t(const string &feature,
					const string &operation);
};

class feature_byte_samples_missing_exception_t
    : public feature_samples_missing_exception_t {
public:
    feature_byte_samples_missing_exception_t(const string &feature,
					     const string &operation);
};

class feature_line_samples_missing_exception_t
    : public feature_samples_missing_exception_t {
public:
    feature_line_samples_missing_exception_t(const string &feature,
					     const string &operation,
					     const fnr_line_size_t line_size);
    string why() const throw();

protected:
    fnr_line_size_t line_size;
};


class feature_page_samples_missing_exception_t
    : public feature_samples_missing_exception_t {
public:
    feature_page_samples_missing_exception_t(const string &feature,
					     const string &operation);
};

END_NAMESPACE;
END_NAMESPACE;

#endif
