/// @file analysis/feature_base/warnings.cc

// Copyright (c) 2007-2008 Acumem AB.  All rights reserved.

#include <sstream>

#include "warnings.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace std;
using namespace acumem::library;

// ----------------------------------------------------------------------
// feature_warning_base_t
// ----------------------------------------------------------------------
feature_warning_base_t::feature_warning_base_t(const string &feature)
    : warning_base_t(), _where(feature)
{
}

feature_warning_base_t::~feature_warning_base_t()
{
}

string
feature_warning_base_t::where() const {
    return _where;
}


// ----------------------------------------------------------------------
// feature_warning_t
// ----------------------------------------------------------------------
feature_warning_t::feature_warning_t(const string &feature,
				     const string &what)
    : feature_warning_base_t(feature), _what(what)
{
}

feature_warning_t::~feature_warning_t()
{
}

string
feature_warning_t::what() const
{
    return _what;
}

// ----------------------------------------------------------------------
// sample_file_corrupted_warning_t
// ----------------------------------------------------------------------
sample_file_corrupted_warning_t::sample_file_corrupted_warning_t(
    const string &feature,
    const string &why)
    : feature_warning_base_t(feature), _why(why)
{
}

sample_file_corrupted_warning_t::~sample_file_corrupted_warning_t()
{
}

string
sample_file_corrupted_warning_t::what() const
{
    ostringstream oss;
    oss << "Sample file may be corrupted - " << _why;

    return  oss.str();
}

// ----------------------------------------------------------------------
// line_size_missing_warning_t
// ----------------------------------------------------------------------
line_size_missing_warning_t::line_size_missing_warning_t(
    const string &feature,
    const string &why,
    const fnr_line_size_t line_size)
    : feature_warning_base_t(feature), _why(why), _line_size(line_size)
{
}

line_size_missing_warning_t::~line_size_missing_warning_t()
{
}

fnr_line_size_t
line_size_missing_warning_t::line_size() const
{
    return _line_size;
}


string
line_size_missing_warning_t::what() const
{
    ostringstream oss;
    oss << "Samples for line size " << _line_size << " missing - " << _why;

    return  oss.str();
}


// ----------------------------------------------------------------------
// cache_line_size_missing_warning_t
// ----------------------------------------------------------------------
cache_line_size_missing_warning_t::cache_line_size_missing_warning_t(
    const string &feature,
    const string &why,
    const unsigned int cache,
    const fnr_line_size_t line_size)
    : feature_warning_base_t(feature), _why(why),
      _cache(cache), _line_size(line_size)
{
}

cache_line_size_missing_warning_t::~cache_line_size_missing_warning_t()
{
}

fnr_line_size_t
cache_line_size_missing_warning_t::line_size() const
{
    return _line_size;
}


unsigned int
cache_line_size_missing_warning_t::cache() const
{
    return _cache;
}


string
cache_line_size_missing_warning_t::what() const
{
    ostringstream oss;
    oss << "Samples for line size " << _line_size
	<< " missing for cache " << _cache << " - "
	<< _why;

    return  oss.str();
}


END_NAMESPACE;
END_NAMESPACE;
