/// @file analysis/feature_base/workspace.cc

// Copyright (c) 2008 Acumem AB.  All rights reserved.

#include "analysis/feature_base/workspace.hh"

using namespace std;

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

// --- workspace_t ------------------------------------------------------

#if defined(ENABLE_WORKSPACE_PROFILING)

void
workspace_t::dump(ostream &os) const
{
	BOOST_FOREACH(const any_map_t::value_type &v, any_map) {
	    const string &key(v.first);
	    const workspace_entry_t &e(v.second);

	    os << key << endl
	       << "  Class: " << e.name << endl
	       << "  Accesses: " << e.accesses << endl;
#if defined(ENABLE_MEMORY_PROFILING)
	    os << "  Memory usage (i): " << e.inclusive_memory.total_size
	       << " bytes, " << e.inclusive_memory.chunks << " chunks" << endl
	       << "  Memory usage (e): " << e.exclusive_memory.total_size
	       << " bytes, " << e.exclusive_memory.chunks << " chunks" << endl;
#endif
#if defined(ENABLE_TIME_PROFILING)
	    os << "  Run time (i): " << e.inclusive_time << " s" << endl
	       << "  Run time (e): " << e.exclusive_time << " s" << endl;
#endif
	}

}

#else // !defined(ENABLE_WORKSPACE_PROFILING)

void
workspace_t::dump(ostream &os) const
{
    BOOST_FOREACH(const any_map_t::value_type &v, any_map) {
	const string &key(v.first);

	os << key << endl;
    }
}

#endif // !defined(ENABLE_WORKSPACE_PROFILING)

// --- workspace_stack_t ------------------------------------------------

workspace_stack_t::workspace_stack_t()
{
    push();
}

workspace_stack_t::workspace_stack_t(workspace_ptr_t &w)
{
    push(w);
}

void
workspace_stack_t::push()
{
    workspace_ptr_t w(new workspace_t());
    push(w);
}

void
workspace_stack_t::push(workspace_ptr_t &w)
{
    stack.push_front(w);
}

void
workspace_stack_t::pop()
{
    stack.pop_front();
}

workspace_ptr_t &
workspace_stack_t::top()
{
    return stack.front();
}

void
workspace_stack_t::dump(ostream &os) const
{
    BOOST_FOREACH(const workspace_ptr_t &w, stack)
	w->dump(os);
}

// --- scoped_temporary_workspace_t -------------------------------------

scoped_temporary_workspace_t::scoped_temporary_workspace_t(
    workspace_stack_ptr_t &workspace_stack)
    : workspace_stack(workspace_stack)
{
    workspace_stack->push();
}

scoped_temporary_workspace_t::~scoped_temporary_workspace_t()
{
    workspace_stack->pop();
}

END_NAMESPACE;
END_NAMESPACE;
