/// @file analysis/fetch_ratio/repl_fr.hh

// Copyright (c) 2006-2009 Acumem AB.  All rights reserved.

#ifndef REPL_FR_HH
#define REPL_FR_HH

#include "analysis/fetch_ratio/repl_fr_engine.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace std;
using namespace acumem::library;

using boost::shared_ptr;

// ----- repl_fr_result_t -----------------------------------------------------

typedef vector<double> ratio_vector_t;

struct repl_fr_burst_t {
    /// Replacement ratio between the valid and end accesses.
    ratio_vector_t val_to_end;
};

typedef vector<repl_fr_burst_t> repl_fr_burst_vector_t;

struct repl_fr_result_t {
    repl_fr_result_t(const parser_burst_id_t num_bursts)
	: bursts(num_bursts) {}

    repl_fr_burst_vector_t bursts;
};

typedef shared_ptr<repl_fr_result_t> repl_fr_result_ptr_t;
typedef shared_ptr<const repl_fr_result_t> repl_fr_result_cptr_t;

// ----- repl_fr_t ------------------------------------------------------------

class repl_fr_t : public feature_t<repl_fr_result_cptr_t> {
public:
    static repl_fr_result_cptr_t calculate(
	sample_mapping_result_cptr_t sample_mapping,
	repl_fr_engine_result_cptr_t engine_data,
	mapped_sample_type_t sample_type);

    repl_fr_t(const feature_context_ptr_t &context,
	      const string &sample_file,
	      fnr_stack_depth_t stack_depth,
	      mapped_sample_type_t window_sample_type,
	      mapped_sample_type_t sample_sample_type,
	      fnr_line_size_t line_size,
	      fnr_line_size_t page_size,
	      fnr_cache_size_t cache_size,
	      thread_mapping_cptr_t thread_mapping,
	      const cpu_model_t &cpu_model,
	      repl_fr_engine_cptr_t engine);

    ~repl_fr_t();

    repl_fr_result_cptr_t get(bool cache_results = true) const;

protected:
    string name() const;
    repl_fr_result_cptr_t calculate() const;

    const string sample_file;
    const fnr_stack_depth_t stack_depth;
    const mapped_sample_type_t window_sample_type;
    const mapped_sample_type_t sample_sample_type;
    const fnr_line_size_t line_size;
    const fnr_line_size_t page_size;
    const fnr_cache_size_t cache_size;
    const thread_mapping_cptr_t thread_mapping;
    const cpu_model_t cpu_model;
    const repl_fr_engine_cptr_t engine;
};

typedef shared_ptr<repl_fr_t> repl_fr_ptr_t;

END_NAMESPACE;
END_NAMESPACE;

#endif
