/// @file analysis/fetch_ratio/repl_fr_engine.cc

// Copyright (c) 2009 Acumem AB.  All rights reserved.

#include "analysis/fetch_ratio/repl_fr_engine.hh"

#include <boost/lexical_cast.hpp>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace boost;

static window_vector_t::size_type
find_window(const window_vector_t &windows,
            window_vector_t::size_type end,
            tm_local_time_t time2)
{
    while (windows[end]->begin_time > time2)
	end--;

    return end;
}

// ----- repl_fr_engine_result_t ----------------------------------------------

repl_fr_engine_result_t::repl_fr_engine_result_t(
    const windowing_result_cptr_t &windowing,
    fnr_line_count_t num_cache_lines)
    : windowing(windowing), num_cache_lines(num_cache_lines)
{
}

double
repl_fr_engine_result_t::get_repl_fr(parser_burst_id_t burst,
				     tm_cache_id_t cache,
				     tm_local_time_t time1,
				     tm_local_time_t time2) const
{
    const window_vector_t &windows(windowing->windows[burst][cache]);
    window_vector_t::size_type pc2_window(
	find_window(windows, windows.size() - 1, time2));

    return get_repl_fr(burst, cache, pc2_window, time1, time2);
}

// ----- repl_fr_engine_t -----------------------------------------------------

repl_fr_engine_t::repl_fr_engine_t(const feature_context_ptr_t &context,
				   const string &sample_file,
				   fnr_stack_depth_t stack_depth,
				   thread_mapping_cptr_t thread_mapping,
				   const windowing_engine_cptr_t &windowing_engine,
				   const cpu_model_t &cpu_model)
    : context(context),
      sample_file(sample_file),
      stack_depth(stack_depth),
      thread_mapping(thread_mapping),
      windowing_engine(windowing_engine),
      cpu_model(cpu_model)
{
}

repl_fr_engine_t::~repl_fr_engine_t()
{
}

repl_fr_engine_result_cptr_t
repl_fr_engine_t::get(mapped_sample_type_t window_sample_type,
		      mapped_sample_type_t sample_sample_type,
		      fnr_line_size_t line_size,
		      fnr_line_size_t page_size,
		      fnr_cache_size_t cache_size) const
{
    sample_mapping_result_cptr_t sample_mapping(
	sample_mapping_t(context, sample_file, stack_depth, line_size,
			 page_size, thread_mapping, cpu_model).get());

    windowing_result_cptr_t windowing(
        windowing_t(context, sample_file, stack_depth, line_size, page_size,
                    thread_mapping, cpu_model, window_sample_type,
		    windowing_engine).get());

    // Calculate the number of cache lines.
    fnr_line_count_t num_cache_lines;
    switch (window_sample_type) {
    case Mapped_Line:
	num_cache_lines = cache_size / line_size;
	break;
    case Mapped_Page_Thread:
    case Mapped_Page_Cache:
	num_cache_lines = cache_size / page_size;
	break;
    default:
	assert(0);
        while (1);
    }


    // Calculate replacement fetch ratios.
    return calculate_repl_fr(window_sample_type, sample_sample_type,
			     line_size, page_size, cache_size,
			     sample_mapping, windowing, num_cache_lines);
}

string
repl_fr_engine_t::name() const
{
    return "repl_fr_engine_t(" +
	sample_file + "," +
	lexical_cast<string>(stack_depth) + "," +
	thread_mapping->name() + "," +
	windowing_engine->name() + "," +
	cpu_model.name() + ")";
}

END_NAMESPACE;
END_NAMESPACE;
