/// @file analysis/fetch_ratio/window_rr.hh

// Copyright (c) 2008 Acumem AB.  All rights reserved.

#ifndef WINDOW_RR_HH
#define WINDOW_RR_HH

#include "analysis/fetch_ratio/windowing.hh"
#include "analysis/thread_mapping/sample_mapping.hh"
#include "analysis/call_stack/stack_pc_map.hh"
#include "analysis/misc/histogram.hh"
#include "analysis/misc/parser.hh"
#include "analysis/feature_base/feature_base.hh"
#include "library/fenris/fenris.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace std;
using namespace acumem::library;

using boost::shared_ptr;

// ----- window_rr_result_t ---------------------------------------------------

typedef vector<double> window_rr_vector_t;
typedef vector<window_rr_vector_t> window_rr_cache_vector_t;
typedef vector<window_rr_cache_vector_t> window_rr_burst_vector_t;

/// Window replacement ratio result.
struct window_rr_result_t {
    window_rr_result_t(parser_burst_id_t num_bursts)
	: replacement_ratio(num_bursts) {}

    /// Replacement ratios, in the same order as the windows.
    window_rr_burst_vector_t replacement_ratio;
};

typedef shared_ptr<window_rr_result_t> window_rr_result_ptr_t;
typedef shared_ptr<const window_rr_result_t> window_rr_result_cptr_t;

// ----- window_rr_t ----------------------------------------------------------

/// Window replacement ratio module.
class window_rr_t : public feature_t<window_rr_result_cptr_t> {
public:
    static window_rr_result_cptr_t calculate(
	fnr_line_count_t num_cache_lines,
        windowing_result_cptr_t windowing);

    window_rr_t(feature_context_ptr_t context,
                const string &sample_file,
                fnr_stack_depth_t stack_depth,
                fnr_line_size_t line_size,
                fnr_line_size_t page_size,
                thread_mapping_cptr_t thread_mapping,
		const cpu_model_t &cpu_model,
		mapped_sample_type_t sample_type,
                fnr_cache_size_t cache_size,
		const windowing_engine_cptr_t &windowing);
    window_rr_result_cptr_t get() const;

protected:
    string name() const;
    window_rr_result_cptr_t calculate() const;

private:
    const feature_context_ptr_t context;
    const string sample_file;
    const fnr_stack_depth_t stack_depth;
    const fnr_line_size_t line_size;
    const fnr_line_size_t page_size;
    const thread_mapping_cptr_t thread_mapping;
    const cpu_model_t cpu_model;
    const mapped_sample_type_t sample_type;
    const fnr_cache_size_t cache_size;
    const windowing_engine_cptr_t windowing;
};

END_NAMESPACE;
END_NAMESPACE;

#endif
