/// @file analysis/filter/filter_builder.hh

// Copyright (c) 2006-2008 Acumem AB.  All rights reserved.

#ifndef FILTER_BUILDER_HH
#define FILTER_BUILDER_HH

#include "library/lang/common_macros.h"
#include "filter.hh"

#include <string>
#include <map>
#include <vector>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace std;

class builder_base_t {
public:
    virtual ~builder_base_t();
    virtual filter_ptr_t create(const string_vector_t &parameters) const =0;
};

template<class T>
class builder_t : public builder_base_t {
public:
    filter_ptr_t create(const string_vector_t &parameters) const
	{
	    return filter_ptr_t(T::create(parameters));
	}
};

class filter_registry_base_t {
public:
    void add(const string &name, builder_base_t *b);
    const builder_base_t *get_builder(const string &name) const;

private:
    typedef map<string, builder_base_t*> builder_map_t;
    builder_map_t builder_map;
};

class filter_registry_t : public filter_registry_base_t {
public:
    filter_registry_t();
};

class filter_builder_t {
public:
    filter_builder_t();
    filter_ptr_t create(const string &filter) const;

private:
    static filter_registry_base_t *filter_registry;
};

END_NAMESPACE;
END_NAMESPACE;
#endif
