/// @file analysis/hw_prefetch/watermark.hh

// Copyright (c) 2009 Acumem AB.  All rights reserved.

#ifndef WATERMARK_HH
#define WATERMARK_HH

#include "library/lang/common_macros.h"
#include "library/lang/common_types.h"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);


/// Maintain a high level watermark and count the longest stretch of
/// 1-increments of the tracked value.
///
/// XXX There is a problem here, that depends on where we start in the
/// sequence: (2, 1, -1) leads to total stride 2, but only one
/// consequitive +1 increase.  The permutation (-1, 2, 1) leads to total
/// stride 2 and two consequitive +1 increases of the high water mark
/// level (which is the desired result).
///
/// XXX We could also build in a "reach" analysis.
struct watermark_t {
    watermark_t(int64_t reach=0) : watermark(0), stretch(0), reach(reach) {}
    watermark_t(int64_t watermark, int64_t stretch, int64_t reach)
	: watermark(watermark), stretch(stretch), reach(reach) {}

    watermark_t mark(int64_t value) const;

    int64_t watermark;
    int64_t stretch;
    int64_t reach;
};

END_NAMESPACE;
END_NAMESPACE;

#endif
