/// @file analysis/issues/comm_spot.hh

// Copyright (c) 2008-2009 Acumem AB.  All rights reserved.

#ifndef COMM_HOT_SPOT_HH
#define COMM_HOT_SPOT_HH

#include "analysis/communication/communication.hh"

#include "loop_engine.hh"
#include "issue.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);
BEGIN_NAMESPACE(issues);

using namespace std;
using namespace acumem::library;

using boost::shared_ptr;

/// Result from comm_spot_t::get().
struct comm_spot_result_t {
    comm_spot_issue_vector_t comm_spots;
    comm_utilization_issue_vector_t comm_utilization;
};

typedef shared_ptr<const comm_spot_result_t> comm_spot_result_cptr_t;
typedef shared_ptr<comm_spot_result_t> comm_spot_result_ptr_t;

/// Module for finding communication and report hot-spot and
/// communication utilization issues.
class comm_spot_t : public loop_analysis_base_t {
public:
    comm_spot_t(const feature_context_ptr_t &context);

    void set_configuration(const loop_driver_conf_t &c);
    comm_spot_result_ptr_t get();

    virtual void process_dep(loop_ptr_t &loop,
			     const location_set_t &loop_pc_set,
                             const stats_cptr_t &loop_stat,
			     depgroup_id_t dep_id,
			     const location_set_t &dep_pc_set,
			     const stats_cptr_t &stat);

    bool want_depgroup() const;

private:
    feature_context_ptr_t context;
    loop_driver_conf_t conf;

    locationx_mapper_cptr_t lx_mapper;
    locationx_mapper_result_cptr_t lxm_res;
    communication_result_cptr_t comm_res;

    comm_spot_result_ptr_t res;
};

END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;

#endif
