/// @file analysis/issues/fs.cc

// Copyright (c) 2008-2009 Acumem AB.  All rights reserved.

#include "fs.hh"

#include "analysis/structure/dep_tree_search.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);
BEGIN_NAMESPACE(issues);

using namespace std;
using namespace boost;
using namespace acumem::library;

// ----------------------------------------------------------------------

fs_t::fs_t(const feature_context_ptr_t &c)
    : context(c)
{
}

void
fs_t::set_configuration(const loop_driver_conf_t &c)
{
    conf = c;

    lx_mapper = locationx_mapper_cptr_t(
	new locationx_mapper_t(
	    context, c.sample_file, c.stack_depth, c.thread_mapping,
	    true, false, false));
    lxm_res = lx_mapper->get();

    fs_res = false_sharing_t(context, c.sample_file, c.stack_depth,
			     c.line_size, c.page_size, c.cache_size,
			     c.thread_mapping, c.cpu_model, c.engine,
			     lx_mapper).get();

    res = fs_result_ptr_t(new fs_result_t);
}

fs_result_ptr_t
fs_t::get()
{
    return res;
}

void
fs_t::process_dep(loop_ptr_t &loop,
		  const location_set_t &loop_pc_set,
		  const stats_cptr_t &loop_stat,
		  depgroup_id_t dep_id,
		  const location_set_t &dep_pc_set,
		  const stats_cptr_t &stat)
{
    const locationx_set_t lx_set(lxm_res->get_locationx_set(dep_pc_set));
    const false_sharing_info_t site(fs_res->get(lx_set));

    if (site.valid &&
        conf.issue_relevant->issue_relevant_fetches(site.waste / conf.line_size * conf.comm_cost)) {
	fs_issue_ptr_t issue(
	    new fs_issue_t(loop, dep_id,
			   site.ratio, site.waste));

	const false_sharing_map_t &fsm(fs_res->false_sharing);
	BOOST_FOREACH(const locationx_t lx, lx_set) {
	    false_sharing_map_t::const_iterator it(fsm.find(lx));
	    if (it != fsm.end()) {
		BOOST_FOREACH(const false_sharing_offenders_t::value_type &v,
			      it->second.offenders)
		    issue->offenders.insert(lxm_res->get_location(v.first));
	    }
	}

	res->issues.push_back(issue);
    }
}

bool
fs_t::want_depgroup() const
{
    return true;
}

END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;
