/// @file analysis/issues/fs.hh

// Copyright (c) 2008-2009 Acumem AB.  All rights reserved.

#ifndef FS_HH
#define FS_HH

#include "analysis/communication/false_sharing.hh"

#include "loop_engine.hh"
#include "issue.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);
BEGIN_NAMESPACE(issues);

using namespace std;
using namespace acumem::library;

using boost::shared_ptr;

/// Result from hot_spot_t::get().
struct fs_result_t {
    fs_issue_vector_t issues;
};

typedef shared_ptr<const fs_result_t> fs_result_cptr_t;
typedef shared_ptr<fs_result_t> fs_result_ptr_t;

/// Module for finding communication hot-spots.
class fs_t : public loop_analysis_base_t {
public:
    fs_t(const feature_context_ptr_t &context);

    void set_configuration(const loop_driver_conf_t &c);
    fs_result_ptr_t get();

    virtual void process_dep(loop_ptr_t &loop,
			     const location_set_t &loop_pc_set,
                             const stats_cptr_t &loop_stat,
			     depgroup_id_t dep_id,
			     const location_set_t &dep_pc_set,
			     const stats_cptr_t &stat);

    bool want_depgroup() const;

private:
    feature_context_ptr_t context;
    loop_driver_conf_t conf;

    locationx_mapper_cptr_t lx_mapper;
    locationx_mapper_result_cptr_t lxm_res;
    false_sharing_result_cptr_t fs_res;

    double waste_threshold;

    fs_result_ptr_t res;
};


END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;

#endif
