/// @file analysis/issues/hot_spot.cc

// Copyright (c) 2008-2009 Acumem AB.  All rights reserved.

#include "hot_spot.hh"

#include "analysis/structure/dep_tree_search.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);
BEGIN_NAMESPACE(issues);

using namespace std;
using namespace boost;
using namespace acumem::library;

// ----------------------------------------------------------------------

hot_spot_t::hot_spot_t(const feature_context_ptr_t &c)
    : context(c)
{
}

void
hot_spot_t::set_configuration(loop_driver_conf_t const &c)
{
    conf = c;

    parser_result_cptr_t p_res(
        parser_t(context, conf.sample_file).get());
    mt_sampled = p_res->mt_sampled();

    locationx_mapper_cptr_t lxm(
	new locationx_mapper_t(context,
			       conf.sample_file,
			       conf.stack_depth,
			       conf.thread_mapping,
			       false, false, false));
    locationx_mapper_result_cptr_t lxm_res(lxm->get());
    locationx_t lx_global(0);

    // Get fetch ratio results.
    fr_lookup_result_cptr_t frl_res(
	fr_lookup_t(
	    context, conf.sample_file, conf.stack_depth, conf.line_size,
	    conf.page_size, conf.thread_mapping, conf.cpu_model, lxm,
	    conf.cache_size, conf.engine, FRL_Type_All).get());

    res.reset(new hot_spot_result_t);
}

hot_spot_result_ptr_t
hot_spot_t::get()
{
    return res;
}

void
hot_spot_t::create_issue(const loop_ptr_t &loop,
			 depgroup_id_t dep_id,
			 const location_set_t &dep_pc_set,
			 hot_spot_type_t type)
{
    // Dig out the point where each instruction in this dependency
    // group was written.

    /// @todo XXX Consider performing write-point tracking for each pc address
    /// separately, instead of for the entire dependency group.
    dep_tree_search_result_cptr_t span_backwards = dep_tree_search_access_t(
        context, conf.sample_file, conf.stack_depth, conf.line_size,
	conf.page_size, conf.thread_mapping, conf.cpu_model, Mapped_Byte,
	&fnr_access_filter_wr_wrnta, true, true, dep_pc_set,
	(fnr_time_t)-1).get();

    // Create a hot-spot issue for the dependence group.
    hot_spot_issue_ptr_t issue(new hot_spot_issue_t(type, loop->id, dep_id));

    BOOST_FOREACH(const location_time_map_t::value_type &v,
		  span_backwards->location_time_map)
	issue->previous_writers.insert(v.first);
    res->hot_spot_issues.push_back(issue);
}

void
hot_spot_t::process_dep(loop_ptr_t &loop,
			const location_set_t &loop_pc_set,
                        const stats_cptr_t &loop_stat,
                        depgroup_id_t dep_id,
			const location_set_t &dep_pc_set,
			const stats_cptr_t &stat)
{
    if (conf.issue_relevant->issue_relevant_fetches(
            stat->all_ratio.total_fetches()))
	create_issue(loop, dep_id, dep_pc_set, Hot_Spot_Fetch);

    if (mt_sampled &&
	conf.issue_relevant->issue_relevant_write_backs(
            stat->all_ratio.total_write_backs()))
	create_issue(loop, dep_id, dep_pc_set, Hot_Spot_Write_Back);
}

bool
hot_spot_t::want_depgroup() const
{
    return true;
}

END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;
