/// @file analysis/issues/hot_spot.hh

// Copyright (c) 2008-2009 Acumem AB.  All rights reserved.

#ifndef HOT_SPOT_HH
#define HOT_SPOT_HH

#include "loop_engine.hh"
#include "issue.hh"
#include "analysis/structure/access_lookup.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);
BEGIN_NAMESPACE(issues);

using namespace std;
using namespace acumem::library;

using boost::shared_ptr;

/// Result from hot_spot_t::get().
struct hot_spot_result_t {
    hot_spot_issue_vector_t hot_spot_issues;
};

typedef shared_ptr<hot_spot_result_t> hot_spot_result_ptr_t;

/// Module for finding hotspots.
class hot_spot_t : public loop_analysis_base_t {
public:
    hot_spot_t(const feature_context_ptr_t &c);
    void set_configuration(loop_driver_conf_t const &c);
    hot_spot_result_ptr_t get();

    virtual void process_dep(loop_ptr_t &loop,
			     const location_set_t &loop_pc_set,
                             const stats_cptr_t &loop_stat,
                             depgroup_id_t dep_id,
			     const location_set_t &dep_pc_set,
			     const stats_cptr_t &stat);

    bool want_depgroup() const;

private:
    void create_issue(const loop_ptr_t &loop,
		      depgroup_id_t dep_id,
		      const location_set_t &dep_pc_set,
		      hot_spot_type_t type);

    feature_context_ptr_t context;
    loop_driver_conf_t conf;
    hot_spot_result_ptr_t res;
    bool mt_sampled;
};

END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;

#endif
