/// @file analysis/issues/loop_order.hh

// Copyright (c) 2006-2010 Acumem AB.  All rights reserved.

#ifndef LOOP_ORDER_HH
#define LOOP_ORDER_HH

#include "library/lang/common_macros.h"
#include "library/fenris/fnr_types.hh"

#include "loop_engine.hh"
#include "issue.hh"
#include "analysis/feature_base/feature_base.hh"
#include "analysis/feature_base/workspace.hh"
#include "analysis/structure/loop_identification.hh"
#include "analysis/structure/dependence_group.hh"
#include "analysis/structure/pc_lookup.hh"
#include "analysis/thread_mapping/sample_mapping.hh"

#include <boost/shared_ptr.hpp>
#include <vector>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);
BEGIN_NAMESPACE(issues);

using namespace std;
using namespace acumem::library;

using boost::shared_ptr;

// ----- loop_order_result_t --------------------------------------------------

/// Loop order analysis result.
struct loop_order_result_t {
    loop_order_issue_vector_t loop_order_issues;
};

typedef shared_ptr<loop_order_result_t> loop_order_result_ptr_t;

// ----- loop_order_t ---------------------------------------------------------

/// Module for detecting incorrect loop order.
class loop_order_t : public loop_analysis_base_t {
public:
    loop_order_t(const feature_context_ptr_t &c);
    void set_configuration(loop_driver_conf_t const &c);
    loop_order_result_ptr_t get();

    virtual void process_dep(loop_ptr_t &loop,
			     const location_set_t &loop_pc_set,
                             const stats_cptr_t &loop_stat,
                             depgroup_id_t dep_id,
			     const location_set_t &dep_pc_set,
			     const stats_cptr_t &stat);

    bool want_depgroup() const;

private:
    feature_context_ptr_t context;
    loop_driver_conf_t conf;

    pc_lookup_result_cptr_t pcl_byte_res;
    pc_lookup_result_cptr_t pcl_line_res;

    /// Accumulated issues so far.
    loop_order_result_ptr_t loop_order_result;

    void get_long_rdists(const location_set_t &pc_set,
                         pc_lookup_result_cptr_t &pcl_res,
                         fnr_time_t min_rdist,
                         fnr_time_vector_t &rdists);

    // Returns true in case of ineffiecient loop nesting.
    bool inefficient_loop_nesting(const location_set_t &dep_group,
                                  loop_ptr_t loop);
};

END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;

#endif
