/// @file analysis/issues/prefetch.hh

// Copyright (c) 2007-2009 Acumem AB.  All rights reserved.

#ifndef PREFETCH_HH
#define PREFETCH_HH

#include "analysis/issues/loop_engine.hh"
#include "analysis/issues/issue.hh"
#include "analysis/structure/dep_tree_search.hh"
#include "analysis/fetch_ratio/repl_fr_line.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);
BEGIN_NAMESPACE(issues);

using namespace std;
using namespace acumem::library;

using boost::shared_ptr;

/// Result of the prefetch_t module
struct prefetch_result_t {
    prefetch_issue_vector_t issues;
};
typedef shared_ptr<prefetch_result_t> prefetch_result_ptr_t;

/// Module for detecting bad use of prefetch instructions
class prefetch_t : public loop_analysis_base_t {
public:
    prefetch_t(const feature_context_ptr_t &context);
    void set_configuration(loop_driver_conf_t const &c);
    prefetch_result_ptr_t get();

    void process_dep(loop_ptr_t &loop,
                     const location_set_t &loop_pc_set,
                     const stats_cptr_t &loop_stat,
                     depgroup_id_t dep_id,
                     const location_set_t &dep_pc_set,
                     const stats_cptr_t &stat);
    bool want_depgroup() const;

private:
    void analyze_pf_hits(location_t pc,
			 loop_id_t loop_id,
			 depgroup_id_t depgroup_id);

    void analyze_pf_too_far(location_t pc,
			    loop_id_t loop_id,
			    depgroup_id_t depgroup_id);

    void analyze_pf_too_close(location_t pc,
			      loop_id_t loop_id,
			      depgroup_id_t depgroup_id);

    prefetch_issue_ptr_t create_pf_issue(prefetch_problem_t problem,
					 location_t pc,
					 loop_id_t loop_id,
					 depgroup_id_t depgroup_id,
					 double cost,
					 double fetch_ratio);
    prefetch_issue_ptr_t create_pf_issue(prefetch_problem_t problem,
					 location_t pc,
					 loop_id_t loop_id,
					 depgroup_id_t depgroup_id,
					 double cost,
					 fnr_time_t median_rdist);

    /// Context.
    feature_context_ptr_t context;

    /// Configuration.
    loop_driver_conf_t conf;

    /// Accumulated result.
    prefetch_result_ptr_t res;

    /// Cached values calculated in set_configuration().
    sample_mapping_result_cptr_t sample_mapping;

    typedef vector<fnr_sample_count_t> sample_index_vector_t;
    typedef map<location_t, sample_index_vector_t> pf_pc_samples_map_t;
    typedef map<location_t, double> loc_repl_map_t;

    struct burst_info_t {
	pf_pc_samples_map_t pc1_pf_samples;
	pf_pc_samples_map_t pc2_pf_samples;
	loc_repl_map_t pf_repl_fr;
    };

    typedef vector<burst_info_t> burst_info_vector_t;
    burst_info_vector_t burst_info;

    /// Cached results calculated in set_configuration().
    repl_fr_line_result_cptr_t fr_res;
};

END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;

#endif
