/// @file analysis/issues/prefetchnta.hh

// Copyright (c) 2009 Acumem AB.  All rights reserved.

#ifndef PREFETCHNTA_HH
#define PREFETCHNTA_HH

#include "analysis/issues/loop_engine.hh"
#include "analysis/fetch_ratio/repl_fr_line.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);
BEGIN_NAMESPACE(issues);

// ----- prefetchnta_result_t -------------------------------------------------

struct prefetchnta_result_t {
    prefetchnta_issue_vector_t issues;
};

typedef shared_ptr<const prefetchnta_result_t> prefetchnta_result_cptr_t;
typedef shared_ptr<prefetchnta_result_t> prefetchnta_result_ptr_t;

// ----- prefetchnta_t --------------------------------------------------------

/// Map from stack distance to its fraction of the non-temporal reuses.
typedef map<double, double> sd_set_t;
typedef map<location_t, sd_set_t> pc1_sd_set_t;

class prefetchnta_t : public loop_analysis_base_t {
public:
    prefetchnta_t(const feature_context_ptr_t &context);

    void set_configuration(const loop_driver_conf_t &c);
    prefetchnta_result_ptr_t get();

    virtual void process_dep(loop_ptr_t &loop,
			     const location_set_t &loop_pc_set,
                             const stats_cptr_t &loop_stat,
			     depgroup_id_t dep_id,
			     const location_set_t &dep_pc_set,
			     const stats_cptr_t &stat);

    bool want_depgroup() const;

private:
    feature_context_ptr_t context;
    loop_driver_conf_t conf;

    access_lookup_result_cptr_t access_lookup;

    struct reuse_stat_t {
        reuse_stat_t()
            : accesses(0.0),
              fetches(0.0)
            {}

        reuse_stat_t(double accesses,
                     double fetches)
            : accesses(accesses),
              fetches(fetches)
            {}

        void operator+=(const reuse_stat_t &rs) {
            this->accesses += rs.accesses;
            this->fetches += rs.fetches;
        }

        void operator-=(const reuse_stat_t &rs) {
            this->accesses -= rs.accesses;
            this->fetches -= rs.fetches;
        }

        double accesses;
        double fetches;
    };

    typedef map<location_t, reuse_stat_t> pc1_reuse_stat_t;
    typedef map<location_t, reuse_stat_t> pc2_reuse_stat_t;
    typedef map<location_t, pc2_reuse_stat_t> pc1_pc2_reuse_stat_t;

    pc1_reuse_stat_t pc1_reuse_stat;
    pc1_pc2_reuse_stat_t pc1_pc2_reuse_stat;

    typedef map<fnr_thread_id_t, reuse_stat_t> thread_reuse_stat_t;
    typedef map<location_t, thread_reuse_stat_t> pc1_thread_reuse_stat_t;

    pc1_thread_reuse_stat_t pc1_thread_reuse_stat;

    pc1_sd_set_t pc1_sd_sets;

    void prefetchnta_lru(sample_mapping_result_cptr_t sample_mapping,
                         repl_fr_line_result_cptr_t repl_fr_line_l1,
                         pc1_sd_set_t &pc1_sd_set);

    prefetchnta_result_ptr_t res;
};

END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;

#endif /* PREFETCHNTA_HH */
