/// @file analysis/issues/random_issue.cc

// Copyright (c) 2008-2009 Acumem AB.  All rights reserved.

#include "random_issue.hh"

#include "analysis/structure/dep_tree_search.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);
BEGIN_NAMESPACE(issues);

using namespace std;
using namespace boost;
using namespace acumem::library;

// ----------------------------------------------------------------------

random_t::random_t(const feature_context_ptr_t &c)
    : context(c)
{}

void
random_t::set_configuration(loop_driver_conf_t const &c)
{
    conf = c;

    locationx_mapper_cptr_t lxm(
	new locationx_mapper_t(context,
			       conf.sample_file,
			       conf.stack_depth,
			       conf.thread_mapping,
			       false, false, false));
    locationx_mapper_result_cptr_t lxm_res(lxm->get());
    locationx_t lx_global(0);

    // Get miss ratio results.
    fr_lookup_res= fr_lookup_t(
        context, conf.sample_file, conf.stack_depth, conf.line_size,
	conf.page_size, conf.thread_mapping, conf.cpu_model, lxm,
	conf.cache_size, conf.engine, FRL_Type_Stalls).get();

    res.reset(new random_result_t);
}

random_result_ptr_t
random_t::get()
{
    return res;
}

void
random_t::process_dep(loop_ptr_t &loop,
                      const location_set_t &loop_pc_set,
                      const stats_cptr_t &loop_stat,
                      depgroup_id_t dep_id,
                      const location_set_t &dep_pc_set,
                      const stats_cptr_t &stat)
{
    double waste =
        max(stat->wasted_space.value(),
            max(stat->fetch_waste.value(),
                stat->write_back_waste.value()));

    // Check if the dependence group has too little wasted space to report
    if (stat->random_ratio > conf.random_threshold &&
        conf.issue_relevant->issue_relevant_fetches(
            waste /conf.line_size)) {

	// Dig out the point where each instruction in this dependency
	// group was written.

	/// @todo XXX Consider performing write-point tracking for each
	/// pc address separately, instead of for the entire dependency group.
	dep_tree_search_result_cptr_t span_backwards =
	    dep_tree_search_access_t(
		context, conf.sample_file, conf.stack_depth, conf.line_size,
		conf.page_size, conf.thread_mapping, conf.cpu_model,
		Mapped_Byte, &fnr_access_filter_wr_wrnta, true, true,
		dep_pc_set, (fnr_time_t)-1).get();

	random_access_issue_ptr_t issue(
	    new random_access_issue_t(loop->id, dep_id));
	BOOST_FOREACH(const location_time_map_t::value_type &v,
		      span_backwards->location_time_map)
	    issue->previous_writers.insert(v.first);
	res->issues.push_back(issue);
    }
}

bool
random_t::want_depgroup() const
{
    return true;
}

END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;
