/// @file analysis/issues/random_issue.hh

// Copyright (c) 2008-2009 Acumem AB.  All rights reserved.

#ifndef RANDOM_ISSUE_HH
#define RANDOM_ISSUE_HH

#include "issue.hh"
#include "loop_engine.hh"
#include "analysis/fetch_ratio/fr_lookup.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);
BEGIN_NAMESPACE(issues);

using namespace std;
using namespace acumem::library;

using boost::shared_ptr;

/// Result from random_t::get().
struct random_result_t {
    random_access_issue_vector_t issues;
};

typedef shared_ptr<random_result_t> random_result_ptr_t;

/// Module for finding which loops and dependency groups exhibit a
/// random access behaviour.
class random_t : public loop_analysis_base_t {
public:
    random_t(const feature_context_ptr_t &c);
    void set_configuration(loop_driver_conf_t const &c);
    random_result_ptr_t get();

    virtual void process_dep(loop_ptr_t &loop,
			     const location_set_t &loop_pc_set,
                             const stats_cptr_t &loop_stat,
                             depgroup_id_t dep_id,
			     const location_set_t &dep_pc_set,
			     const stats_cptr_t &stat);

    bool want_depgroup() const;

private:
    feature_context_ptr_t context;
    fr_lookup_result_cptr_t fr_lookup_res;
    loop_driver_conf_t conf;
    random_result_ptr_t res;
};

END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;

#endif
