/// @file analysis/issues/utilization_issue.cc

// Copyright (c) 2008-2009 Acumem AB.  All rights reserved.

#include "utilization_issue.hh"

#include "analysis/structure/dep_tree_search.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);
BEGIN_NAMESPACE(issues);

using namespace std;
using namespace boost;
using namespace acumem::library;

// ----------------------------------------------------------------------

utilization_t::utilization_t(const feature_context_ptr_t &c)
    : context(c)
{}

void
utilization_t::set_configuration(loop_driver_conf_t const &c)
{
    conf = c;

    parser_result_cptr_t p_res =
        parser_t(context, conf.sample_file).get();

    mt_sampled = p_res->mt_sampled();

    res.reset(new utilization_result_t);
}

utilization_result_ptr_t
utilization_t::get()
{
    return res;
}

void
utilization_t::process_dep(loop_ptr_t &loop,
			   const location_set_t &loop_pc_set,
			   const stats_cptr_t &loop_stat,
			   depgroup_id_t dep_id,
			   const location_set_t &dep_pc_set,
			   const stats_cptr_t &stat)
{

    // Check if the dependence group has too little wasted space to
    // report.
    if (mt_sampled) {
	/// @todo Configurable threshold value.
	/// @todo Consider whether the value is valid.
	if (conf.issue_relevant->issue_relevant_fetches(
                stat->fetch_waste.value() / conf.line_size) &&
	    stat->fetch_util.value() < 0.70 &&
	    stat->fetch_util.valid())
	    create_issue(loop, dep_id, dep_pc_set, Utilization_Fetch);

	/// @todo Configurable threshold value.
	/// @todo Consider whether the value is valid.
	if (conf.issue_relevant->issue_relevant_write_backs(
                stat->write_back_waste.value() / conf.line_size) &&
	    stat->write_back_util.value() < 0.70 &&
	    stat->write_back_util.valid())
	    create_issue(loop, dep_id, dep_pc_set, Utilization_Write_Back);
    } else {
	/// @todo Configurable threshold value.
	/// @todo Consider whether the value is valid.
	if (conf.issue_relevant->issue_relevant_fetches(
                stat->wasted_space.value() / conf.line_size) &&
	    stat->spatial_util.value() < 0.70 &&
	    stat->spatial_util.valid())
	    create_issue(loop, dep_id, dep_pc_set, Utilization_Legacy);
    }
}

bool
utilization_t::want_depgroup() const
{
    return true;
}

void
utilization_t::create_issue(const loop_ptr_t &loop,
			    depgroup_id_t dep_id,
			    const location_set_t &dep_pc_set,
			    utilization_type_t type)
{
    // Dig out the point where each instruction in this dependency
    // group was written.

    /// @todo XXX Consider performing write-point tracking for each pc address
    /// separately, instead of for the entire dependency group.
    dep_tree_search_result_cptr_t span_backwards = dep_tree_search_access_t(
        context, conf.sample_file, conf.stack_depth, conf.line_size,
	conf.page_size, conf.thread_mapping, conf.cpu_model, Mapped_Byte,
	&fnr_access_filter_wr_wrnta, true, true, dep_pc_set, (fnr_time_t)-1).get();

    // Create an issue for the dependence group.
    utilization_issue_ptr_t issue(
	new utilization_issue_t(type, loop->id, dep_id));

    BOOST_FOREACH(const location_time_map_t::value_type &v,
		  span_backwards->location_time_map)
	issue->previous_writers.insert(v.first);

    res->issues.push_back(issue);
}


END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;
