/// @file analysis/issues/utilization_issue.hh

// Copyright (c) 2008-2009 Acumem AB.  All rights reserved.

#ifndef UTILIZATION_ISSUE_HH
#define UTILIZATION_ISSUE_HH

#include "issue.hh"
#include "loop_engine.hh"
#include "analysis/fetch_ratio/fr_lookup.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);
BEGIN_NAMESPACE(issues);

using namespace std;
using namespace acumem::library;

using boost::shared_ptr;

/// Result from utilization_t::get().
struct utilization_result_t {
    utilization_issue_vector_t issues;
};

typedef shared_ptr<utilization_result_t> utilization_result_ptr_t;

/// Module for finding which loops and dependency groups exhibit poor
/// spatial usage.
///
/// There are many reasons for displaying poor spatial usage, out of
/// which hot/cold fields is but one.
///
/// @todo Rename this class to something more general, such as
/// poor_spatial_util_t.
class utilization_t : public loop_analysis_base_t {
public:
    utilization_t(const feature_context_ptr_t &c);
    void set_configuration(loop_driver_conf_t const &c);
    utilization_result_ptr_t get();

    virtual void process_dep(loop_ptr_t &loop,
			     const location_set_t &loop_pc_set,
                             const stats_cptr_t &loop_stat,
                             depgroup_id_t dep_id,
			     const location_set_t &dep_pc_set,
			     const stats_cptr_t &stat);

    bool want_depgroup() const;

private:
    void create_issue(const loop_ptr_t &loop,
		      depgroup_id_t dep_id,
		      const location_set_t &dep_pc_set,
		      utilization_type_t type);

    feature_context_ptr_t context;
    loop_driver_conf_t conf;
    utilization_result_ptr_t res;
    bool mt_sampled;
};

END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;

#endif
