/// @file analysis/locationa/locationa.cc

// Copyright (c) 2009-2010 Acumem AB.  All rights reserved.

#include "analysis/locationa/locationa.hh"

#include <boost/lexical_cast.hpp>

using namespace boost;

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

typedef set<locationa_specifier_t> locationa_specifier_set_t;

static void
add_locationa(
    locationa_specifier_set_t &locationas,
    const thread_mapping_burst_t &thread_mapping,
    const stack_pc_map_result_cptr_t &stack_pc_map,
    fnr_address_t pc,
    fnr_stack_id_t stack,
    fnr_thread_id_t thread,
    fnr_time_t time)
{
    const locationa_specifier_t spec(
        stack_pc_map->get_location(stack, pc),
	thread,
	thread_mapping.get_cache_id(time, thread));
    locationas.insert(spec);
}

static void
add_locationa(
    locationa_specifier_set_t &locationas,
    const thread_mapping_burst_t &thread_mapping,
    const stack_pc_map_result_cptr_t &stack_pc_map,
    const fnr_sample_access_t &access,
    fnr_time_t time)
{
    add_locationa(locationas, thread_mapping, stack_pc_map,
                  access.pc, access.stack, access.thread, time);
}

locationa_result_cptr_t
locationa_mapper_t::calculate(
    feature_context_ptr_t context,
    parser_result_cptr_t samples,
    thread_mapping_result_cptr_t thread_mapping,
    stack_pc_map_result_cptr_t stack_pc_map)
{
    locationa_result_ptr_t result(new locationa_result_t());

    locationa_specifier_set_t locationas;
    thread_mapping_burst_vector_t::const_iterator it_tmb(
	thread_mapping->bursts.begin());
    BOOST_FOREACH(const parser_burst_t &burst, samples->bursts) {
	context->feedback->test_cancel();
	const thread_mapping_burst_t &tmb(*it_tmb++);
	BOOST_FOREACH(const sample_ptr_t &sample, burst.samples_by_pc1) {
	    fnr_time_t time1(sample->begin.time);

	    add_locationa(locationas, tmb, stack_pc_map, sample->begin, time1);
	    add_locationa(locationas, tmb, stack_pc_map, sample->pc_1_plus,
                          sample->pc_1_plus_stack, sample->begin.thread,
                          time1);

	    BOOST_FOREACH(const fnr_sample_access_t &access, sample->end)
		add_locationa(locationas, tmb, stack_pc_map, access, time1);
	}
    }

    result->locationas.reserve(locationas.size());
    BOOST_FOREACH(const locationa_specifier_t &las, locationas)
        result->locationas.push_back(las);

    return result;
}

locationa_mapper_t::locationa_mapper_t(
    feature_context_ptr_t context,
    const string &sample_file,
    fnr_stack_depth_t stack_depth,
    const thread_mapping_cptr_t &thread_mapping)
    : feature_t<locationa_result_cptr_t>(context),
      sample_file(sample_file),
      stack_depth(stack_depth),
      thread_mapping(thread_mapping)
{
}

locationa_result_cptr_t
locationa_mapper_t::get() const
{
    return get_data();
}

string
locationa_mapper_t::name() const
{
    return "locationa_mapper_t(" +
        sample_file + "," +
        lexical_cast<string>(stack_depth) + "," +
        thread_mapping->name();
}

locationa_result_cptr_t
locationa_mapper_t::calculate() const
{
    return calculate(
	context,
        parser_t(context, sample_file).get(),
        thread_mapping->get(),
        stack_pc_map_t(context, sample_file, stack_depth).get());
}

END_NAMESPACE;
END_NAMESPACE;
