/// @file analysis/locationa/locationa.hh

// Copyright (c) 2009-2010 Acumem AB.  All rights reserved.

#ifndef LOCATIONA_HH
#define LOCATIONA_HH

#include "analysis/thread_mapping/mapped_sample.hh"
#include "analysis/call_stack/stack_pc_map.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

// ----- locationa_t ----------------------------------------------------------

typedef fnr_sample_count_t locationa_t;

typedef set<locationa_t> locationa_set_t;

// ----- locationa_specifier_t ------------------------------------------------

struct locationa_specifier_t {

    locationa_specifier_t(
        const location_t location,
        const fnr_thread_id_t thread,
        const tm_cache_id_t cache)
        : location(location),
          thread(thread),
          cache(cache)
        {}

    locationa_specifier_t(const locationa_specifier_t &las)
        : location(las.location),
          thread(las.thread),
          cache(las.cache)
        {}

    inline bool operator<(const locationa_specifier_t &rhs) const {
	if (location != rhs.location)
	    return location < rhs.location;
	else if (cache != rhs.cache)
	    return cache < rhs.cache;
	else
	    return thread < rhs.thread;
    }

    inline bool operator==(const locationa_specifier_t &rhs) const {
	return
            location == rhs.location &&
	    cache == rhs.cache &&
	    thread == rhs.thread;
    }

    location_t location;
    fnr_thread_id_t thread;
    tm_cache_id_t cache;
};

typedef vector<locationa_specifier_t> locationa_specifier_vector_t;

// ----- locationa_result_t ---------------------------------------------------

struct locationa_result_t {
    locationa_t find(location_t location,
                     fnr_thread_id_t thread,
                     tm_cache_id_t cache) const {
        locationa_specifier_t las(location, thread, cache);

        locationa_specifier_vector_t::const_iterator lower =
            lower_bound(locationas.begin(), locationas.end(), las);
        assert(lower != locationas.end() && *lower == las);
        return locationa_t(lower - locationas.begin());
    }

    locationa_t find(const mapped_access_t &access) const {
	return find(access.pc, access.thread(), access.cache);
    }

    locationa_specifier_vector_t locationas;
};

typedef shared_ptr<locationa_result_t> locationa_result_ptr_t;
typedef shared_ptr<const locationa_result_t> locationa_result_cptr_t;

// ----- locationa_mapper_t ---------------------------------------------------

class locationa_mapper_t : feature_t<locationa_result_cptr_t> {
public:
    static locationa_result_cptr_t calculate(
	feature_context_ptr_t context,
        parser_result_cptr_t samples,
        thread_mapping_result_cptr_t thread_mapping,
        stack_pc_map_result_cptr_t stack_pc_map);

    locationa_mapper_t(feature_context_ptr_t context,
                       const string &sample_file,
                       fnr_stack_depth_t stack_depth,
                       const thread_mapping_cptr_t &thread_mapping);
    locationa_result_cptr_t get() const;

protected:
    string name() const;
    locationa_result_cptr_t calculate() const;

private:
    const string sample_file;
    const fnr_stack_depth_t stack_depth;
    const thread_mapping_cptr_t thread_mapping;
};

END_NAMESPACE;
END_NAMESPACE;

#endif
