/// @file analysis/misc/histogram.cc

// Copyright (c) 2006-2008 Acumem AB.  All rights reserved.

#include "histogram.hh"

#include <map>
#include <iostream>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace std;

// -------------------------------------------------------------------

exp_2_size_histogram_t::exp_2_size_histogram_t()
{}

int
exp_2_size_histogram_t::get_bin(uint64_t value) const
{
    int i = 0;

    for (; value != 0; value /= 2)
	i++;

    return i;
}

uint64_t
exp_2_size_histogram_t::get_bin_size(int bin) const
{
    if (bin == 0)
	return 1;

    return (uint64_t) 1 << (bin-1);
}

uint64_t
exp_2_size_histogram_t::get_bin_start(int bin) const
{
    return get_bin_size(bin);
}

// -------------------------------------------------------------------

int
exp_8_bit_t::get_bin(uint64_t value) const
{
    if (value < (1 << 8))
	return (int) value;
    if (value < (1 << 16))
	return (int) ((1<<8) + (value >> 8));
    if (value < (1 << 24))
	return (int) ((2<<8) + (value >> 16));
    if (value < (1LL << 32))
	return (int) ((3<<8) + (value >> 24));
    if (value < (1LL << 40))
	return (int) ((4<<8) + (value >> 32));
    if (value < (1LL << 48))
	return (int) ((5<<8) + (value >> 40));
    if (value < (1LL << 56))
	return (int) ((6<<8) + (value >> 48));

    return (int) ((7<<8) + (value >> 56));
}

uint64_t
exp_8_bit_t::get_bin_size(int bin) const
{
    if (bin < (1<<8)) return (1<<0);
    if (bin < (2<<8)) return (1<<8);
    if (bin < (3<<8)) return (1<<16);
    if (bin < (4<<8)) return (1<<24);
    if (bin < (5<<8)) return (1LL<<32);
    if (bin < (6<<8)) return (1LL<<40);
    if (bin < (7<<8)) return (1LL<<48);
    return 1LL<<56;
}

uint64_t
exp_8_bit_t::get_bin_start(int bin) const
{
    if (bin < (1<<8)) return bin;
    if (bin < (2<<8)) return (bin-(1LL<<8))<<8;
    if (bin < (3<<8)) return (bin-(2LL<<8))<<16;
    if (bin < (4<<8)) return (bin-(3LL<<8))<<24;
    if (bin < (5<<8)) return (bin-(4LL<<8))<<32;
    if (bin < (6<<8)) return (bin-(5LL<<8))<<40;
    if (bin < (7<<8)) return (bin-(6LL<<8))<<48;
    return (bin-(7LL<<8))<<56;
}

// -------------------------------------------------------------------

uint32_t
exp_histogram_base_t::ones32(register uint32_t x)
{
    /* 32-bit recursive reduction using SWAR...
       but first step is mapping 2-bit values
       into sum of 2 1-bit values in sneaky way
    */
    x -= ((x >> 1) & 0x55555555);
    x = (((x >> 2) & 0x33333333) + (x & 0x33333333));
    x = (((x >> 4) + x) & 0x0f0f0f0f);
    x += (x >> 8);
    x += (x >> 16);
    return x & 0x0000003f;
}

uint32_t
exp_histogram_base_t::ones64(register uint64_t x)
{
    return ones32((uint32_t) x & 0xffffffff) + ones32((uint32_t) (x >> 32));
}

uint32_t
exp_histogram_base_t::floor_log2_32(register uint32_t x)
{
    x |= (x >> 1);
    x |= (x >> 2);
    x |= (x >> 4);
    x |= (x >> 8);
    x |= (x >> 16);
    return ones32(x >> 1);
}

uint32_t
exp_histogram_base_t::floor_log2_64(register uint64_t x)
{
    x |= (x >> 1);
    x |= (x >> 2);
    x |= (x >> 4);
    x |= (x >> 8);
    x |= (x >> 16);
    x |= (x >> 32);
    return ones64(x >> 1);
}


// -------------------------------------------------------------------

// histogram_t::histogram_t(const histogram_distribution_t &distribution)
//     : distribution_(distribution)
// {
// }

// -------------------------------------------------------------------

END_NAMESPACE
END_NAMESPACE
