/// @file analysis/misc/sweep.hh

// Copyright (c) 2006-2008 Acumem AB.  All rights reserved.

#ifndef SWEEP_HH
#define SWEEP_HH

#include "library/lang/common_macros.h"
#include "library/fenris/fnr_types.hh"

#include <string>
#include <math.h>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace std;
using namespace acumem::library;

struct sweep_point_t {
    sweep_point_t(fnr_cache_size_t cache_size_ =0,
		  fnr_line_size_t line_size_ =0)
	: cache_size(cache_size_), line_size(line_size_)
	{
	}

    fnr_cache_size_t cache_size;
    fnr_line_size_t line_size;
};

enum sweep_mode_t {
    SWEEP_INVALID,
    SWEEP_CACHE_SIZE,
    SWEEP_CACHE_LINE_SIZE,
    SWEEP_NONE
};

struct parameters_t {
    sweep_mode_t sweep_mode;

    fnr_cache_size_t begin_cache_size;
    fnr_cache_size_t end_cache_size;
    int subdivisions;
    fnr_line_size_t begin_cache_line_size;
    fnr_line_size_t end_cache_line_size;
};

class sweep_iterator_t
    : public std::iterator<input_iterator_tag, sweep_point_t,
			   ptrdiff_t, sweep_point_t*,
			   const sweep_point_t>
{
public:
    reference operator *() const;
    sweep_iterator_t &operator++();
    bool operator == (const sweep_iterator_t &i) const;
    bool operator != (const sweep_iterator_t &i) const
	{
	    return ! this->operator == (i);
	}

private:
    friend class sweep_t;

    // This constructor is used to create an iterator representing end()
    sweep_iterator_t(int)
	: step(-1) {}

    // This constructor is used for begin
    sweep_iterator_t(const parameters_t &p_, int size);

    int step;
    int max_steps;
    parameters_t p;
};


class sweep_t {
public:
    sweep_t();
    sweep_t(fnr_cache_size_t begin_cache_size_,
	    fnr_cache_size_t end_cache_size_,
	    int subdivisions_,
	    fnr_line_size_t cache_line_size_);
    sweep_t(fnr_line_size_t begin_cache_line_size_,
	    fnr_line_size_t end_cache_line_size_,
	    fnr_cache_size_t cache_size_);
    sweep_t(fnr_cache_size_t cache_size_,
            fnr_line_size_t line_size_);

    typedef sweep_iterator_t iterator;
    typedef sweep_iterator_t const_iterator;

    iterator begin() {
	return sweep_iterator_t(parameters, size());
    }
    iterator end() {
	return sweep_iterator_t(0);
    }
    const_iterator begin() const {
	return sweep_iterator_t(parameters, size());
    }
    const_iterator end() const {
	return sweep_iterator_t(0);
    }

    string name() const;
    int size() const;

private:
    parameters_t parameters;
};

END_NAMESPACE;
END_NAMESPACE;

#endif
