/// @file analysis/random_access/random_access.hh

// Copyright (c) 2007-2008 Acumem AB.  All rights reserved.

#ifndef RANDOM_ACCESS_HH
#define RANDOM_ACCESS_HH

#include "analysis/fetch_ratio/fr_lookup.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace acumem::library;
using namespace std;

using boost::shared_ptr;

// ----- random_access_result_t -----------------------------------------------

struct random_access_result_t {
    typedef map<locationx_specifier_t, double> random_info_t;
    /// Maps pcs to their random sums. Pcs not in the map should be considered
    /// to have random sum 0.0.
    random_info_t random_info;
};

typedef shared_ptr<random_access_result_t> random_access_result_ptr_t;
typedef shared_ptr<const random_access_result_t> random_access_result_cptr_t;

// ----- random_access_t ------------------------------------------------------

/// Module for random access analysis.
class random_access_t : public feature_t<random_access_result_cptr_t> {
public:
    static random_access_result_cptr_t calculate(
        const feature_context_ptr_t &context,
        const sample_mapping_result_cptr_t &sample_mapping,
        const locationx_mapper_result_cptr_t &lx_full_res,
        const fr_lookup_result_cptr_t &fr_lookup);

    random_access_t(const feature_context_ptr_t &context,
                    const string &sample_file,
                    const fnr_stack_depth_t stack_depth,
                    const fnr_line_size_t line_size,
                    const fnr_line_size_t page_size,
                    const thread_mapping_cptr_t &thread_mapping,
		    const cpu_model_t &cpu_model,
                    const fnr_cache_size_t cache_size,
                    const repl_fr_engine_cptr_t &engine);
    random_access_result_cptr_t get() const;

protected:
    string name() const;
    random_access_result_cptr_t calculate() const;

private:
    const string sample_file;
    const fnr_stack_depth_t stack_depth;
    const fnr_line_size_t line_size;
    const fnr_line_size_t page_size;
    const thread_mapping_cptr_t thread_mapping;
    const cpu_model_t cpu_model;
    const fnr_cache_size_t cache_size;
    const repl_fr_engine_cptr_t engine;
};

END_NAMESPACE;
END_NAMESPACE;

#endif /* RANDOM_ACCESS_HH */
