/// @file analysis/report/report_debug.cc

// Copyright (c) 2010 Acumem AB.  All rights reserved.

#include "analysis/report/report_debug.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

#define EQUAL(field) (field == that.field)

#define LESS_THAN(field) do {                   \
    if (field < that.field)                     \
        return true;                            \
    if (field > that.field)                     \
        return false;                           \
} while (0)

// ----- rpt_binary_t ---------------------------------------------------------

bool
rpt_binary_t::operator==(const rpt_binary_t &that) const
{
    return
        EQUAL(filename) &&
        EQUAL(base);
}

bool
rpt_binary_t::operator<(const rpt_binary_t &that) const
{
    LESS_THAN(filename);
    LESS_THAN(base);
    return false;
}

// ----- rpt_symbol_t ---------------------------------------------------------

bool
rpt_symbol_t::operator==(const rpt_symbol_t &that) const
{
    return
        EQUAL(name) &&
        EQUAL(base);
}

bool
rpt_symbol_t::operator<(const rpt_symbol_t &that) const
{
    LESS_THAN(name);
    LESS_THAN(base);
    return false;
}

// ----- rpt_function_t ---------------------------------------------------------

bool
rpt_function_t::operator==(const rpt_function_t &that) const
{
    return
        EQUAL(name) &&
        EQUAL(base);
}

bool
rpt_function_t::operator<(const rpt_function_t &that) const
{
    LESS_THAN(name);
    LESS_THAN(base);
    return false;
}

// ----- rpt_source_line_t ----------------------------------------------------

bool
rpt_source_line_t::operator==(const rpt_source_line_t &that) const
{
    return
        EQUAL(source_file_id) &&
        EQUAL(line);
}

bool
rpt_source_line_t::operator<(const rpt_source_line_t &that) const
{
    LESS_THAN(source_file_id);
    LESS_THAN(line);
    return false;
}

// ----- rpt_debug_pc_t -------------------------------------------------------

bool
rpt_debug_pc_t::operator==(const rpt_debug_pc_t &that) const
{
    return
        EQUAL(pc_id) &&
        EQUAL(return_address);
}

bool
rpt_debug_pc_t::operator<(const rpt_debug_pc_t &that) const
{
    LESS_THAN(pc_id);
    LESS_THAN(return_address);
    return false;
}

END_NAMESPACE;
END_NAMESPACE;
