/// @file analysis/report/report_debug.hh

// Copyright (c) 2010 Acumem AB.  All rights reserved.

#ifndef REPORT_REPORT_DEBUG_HH
#define REPORT_REPORT_DEBUG_HH

#include "analysis/report/report_types.hh"

#include <boost/shared_array.hpp>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace boost;

/// Pointer too a char array.
typedef shared_array<char> char_array_ptr_t;

// ----- rpt_binary_t ---------------------------------------------------------

enum rpt_debug_info_status_t {
    Rpt_Debug_File_Not_Found,
    Rpt_Debug_Error_Reading_File,
    Rpt_Debug_No_Info_Found,
    Rpt_Debug_Symbols_Found,
    Rpt_Debug_Debug_Info_Found
};

/// TODO
struct rpt_binary_t {
    rpt_binary_t()
        {}

    rpt_binary_t(const string &filename,
                 rpt_address_t base,
                 rpt_address_t size,
                 rpt_debug_info_status_t debug_info_status)
        : filename(filename),
          base(base),
          size(size),
          debug_info_status(debug_info_status)
        {}

    bool operator==(const rpt_binary_t &that) const;
    bool operator<(const rpt_binary_t &that) const;

    string filename;
    rpt_address_t base;
    rpt_address_t size;
    rpt_debug_info_status_t debug_info_status;
};

/// TODO
typedef uint32_t rpt_binary_id_t;

/// TODO
#define RPT_NO_BINARY ((rpt_binary_id_t)-1)

/// TODO
typedef vector<rpt_binary_t> rpt_binary_vector_t;

/// TODO
typedef set<rpt_binary_t> rpt_binary_set_t;

// ----- rpt_symbol_t ---------------------------------------------------------

/// TODO
struct rpt_symbol_t {
    rpt_symbol_t()
        {}

    rpt_symbol_t(const string &name,
                 rpt_address_t base)
        : name(name),
          base(base)
        {}

    bool operator==(const rpt_symbol_t &that) const;
    bool operator<(const rpt_symbol_t &that) const;

    string name;
    rpt_address_t base;
};

/// TODO
typedef uint32_t rpt_symbol_id_t;

/// TODO
#define RPT_NO_SYMBOL ((rpt_symbol_id_t)-1)

/// TODO
typedef vector<rpt_symbol_t> rpt_symbol_vector_t;

/// TODO
typedef set<rpt_symbol_t> rpt_symbol_set_t;

// ----- rpt_function_t -------------------------------------------------------

/// TODO
struct rpt_function_t {
    rpt_function_t()
        {}

    rpt_function_t(const string &name,
                   rpt_address_t base)
        : name(name),
          base(base)
        {}

    bool operator==(const rpt_function_t &that) const;
    bool operator<(const rpt_function_t &that) const;

    string name;
    rpt_address_t base;
};

/// TODO
typedef uint32_t rpt_function_id_t;

/// TODO
#define RPT_NO_FUNCTION ((rpt_function_id_t)-1)

/// TOD
typedef vector<rpt_function_t> rpt_function_vector_t;

/// TODO
typedef set<rpt_function_t> rpt_function_set_t;

// ----- rpt_source_file_t ----------------------------------------------------

/// TODO
struct rpt_source_file_t {
    rpt_source_file_t()
        {}

    rpt_source_file_t(const string &filename)
        : filename(filename),
          data(),
          data_length(0)
        {}

    bool operator<(const rpt_source_file_t &that) const {
        return filename < that.filename;
    }

    bool operator==(const rpt_source_file_t &that) const {
        return filename == that.filename;
    }

    string filename;
    char_array_ptr_t data;
    uint32_t data_length;
};

/// TODO
typedef uint32_t rpt_source_file_id_t;

/// TODO
typedef vector<rpt_source_file_t> rpt_source_file_vector_t;

/// TODO
typedef set<rpt_source_file_t> rpt_source_file_set_t;

// ----- rpt_source_line_t ----------------------------------------------------

/// TODO
struct rpt_source_line_t {
    rpt_source_line_t()
        {}

    rpt_source_line_t(rpt_source_file_id_t source_file_id,
                      uint32_t line)
        : source_file_id(source_file_id),
          line(line)
        {}

    bool operator==(const rpt_source_line_t &that) const;
    bool operator<(const rpt_source_line_t &that) const;

    rpt_source_file_id_t source_file_id;
    uint32_t line;
};

/// TODO
typedef uint32_t rpt_source_line_id_t;

/// TODO
#define RPT_NO_SOURCE_LINE ((rpt_source_line_id_t)-1)

/// TODO
typedef vector<rpt_source_line_t> rpt_source_line_vector_t;

/// TODO
typedef set<rpt_source_line_t> rpt_source_line_set_t;

// ----- rpt_debug_pc_t -------------------------------------------------------

/// TODO
struct rpt_debug_pc_t {
    rpt_debug_pc_t()
        {}

    rpt_debug_pc_t(rpt_address_id_t pc_id,
                   bool return_address)
        : pc_id(pc_id),
          return_address(return_address),
          binary_id(RPT_NO_BINARY),
          symbol_id(RPT_NO_SYMBOL),
          function_id(RPT_NO_FUNCTION),
          source_line_id(RPT_NO_SOURCE_LINE)
        {}

    bool operator==(const rpt_debug_pc_t &that) const;
    bool operator<(const rpt_debug_pc_t &that) const;

    rpt_address_id_t pc_id;
    bool return_address;
    rpt_binary_id_t binary_id;
    rpt_symbol_id_t symbol_id;
    rpt_function_id_t function_id;
    rpt_source_line_id_t source_line_id;
};

/// TODO
typedef uint32_t rpt_debug_pc_id_t;

/// TODO
typedef vector<rpt_debug_pc_t> rpt_debug_pc_vector_t;

/// TODO
typedef set<rpt_debug_pc_t> rpt_debug_pc_set_t;

END_NAMESPACE;
END_NAMESPACE;

#endif /* REPORT_DEBUG_HH */
