/// @file analysis/report/report_stats.cc

// Copyright (c) 2010 Acumem AB.  All rights reserved.

#include "analysis/report/report_stats.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

#define EQUAL(field) (field == that.field)

#define LESS_THAN(field) do {                   \
    if (field < that.field)                     \
        return true;                            \
    if (field > that.field)                     \
        return false;                           \
} while (0)

// ----- rpt_stat_fp_valid_t --------------------------------------------------

bool
rpt_stat_fp_valid_t::operator==(const rpt_stat_fp_valid_t &that) const
{
    return
        EQUAL(value) &&
        EQUAL(valid);
}

bool
rpt_stat_fp_valid_t::operator<(const rpt_stat_fp_valid_t &that) const
{
    LESS_THAN(value);
    LESS_THAN(valid);
    return false;
}

bool
rpt_stat_fp_valid_t::operator>(const rpt_stat_fp_valid_t &that) const
{
    return !operator<(that) && !operator==(that);
}

// ----- rpt_stats_t ----------------------------------------------------------

bool
rpt_stats_t::operator==(const rpt_stats_t &that) const
{
    return
        EQUAL(accesses) &&
        EQUAL(cold_fetches) &&
        EQUAL(replacement_fetches) &&
        EQUAL(coherence_fetches) &&
        EQUAL(flush_fetches) &&
        EQUAL(cold_misses) &&
        EQUAL(replacement_misses) &&
        EQUAL(coherence_misses) &&
        EQUAL(flush_misses) &&
        EQUAL(upgrades) &&
        EQUAL(cold_write_backs) &&
        EQUAL(replacement_write_backs) &&
        EQUAL(coherence_write_backs) &&
        EQUAL(flush_write_backs) &&
        EQUAL(spatial_utilization) &&
        EQUAL(fetch_utilization) &&
        EQUAL(write_back_utilization) &&
        EQUAL(communication_utilization) &&
        EQUAL(communicated_lines) &&
        EQUAL(hardware_prefetch_ratio) &&
        EQUAL(random_ratio) &&
        EQUAL(false_sharing_ratio) &&
        EQUAL(util_corr_fetch_ratio) &&
        EQUAL(util_corr_write_back_ratio) &&
        EQUAL(avoidable_fetches);
}

bool
rpt_stats_t::operator<(const rpt_stats_t &that) const
{
    LESS_THAN(accesses);
    LESS_THAN(cold_fetches);
    LESS_THAN(replacement_fetches);
    LESS_THAN(coherence_fetches);
    LESS_THAN(flush_fetches);
    LESS_THAN(cold_misses);
    LESS_THAN(replacement_misses);
    LESS_THAN(coherence_misses);
    LESS_THAN(flush_misses);
    LESS_THAN(upgrades);
    LESS_THAN(cold_write_backs);
    LESS_THAN(replacement_write_backs);
    LESS_THAN(coherence_write_backs);
    LESS_THAN(flush_write_backs);
    LESS_THAN(spatial_utilization);
    LESS_THAN(fetch_utilization);
    LESS_THAN(write_back_utilization);
    LESS_THAN(communication_utilization);
    LESS_THAN(communicated_lines);
    LESS_THAN(hardware_prefetch_ratio);
    LESS_THAN(random_ratio);
    LESS_THAN(false_sharing_ratio);
    LESS_THAN(util_corr_fetch_ratio);
    LESS_THAN(util_corr_write_back_ratio);
    LESS_THAN(avoidable_fetches);
    return false;
}

// ----- rpt_sliced_stats_t ---------------------------------------------------

bool
rpt_sliced_stats_t::operator==(const rpt_sliced_stats_t &that) const
{
    return
        EQUAL(global_stats) &&
        EQUAL(thread_stats);
}

bool
rpt_sliced_stats_t::operator<(const rpt_sliced_stats_t &that) const
{
    LESS_THAN(global_stats);
    LESS_THAN(thread_stats);
    return false;
}

// ----- rpt_stats_sweep_t ----------------------------------------------------

bool
rpt_stats_sweep_t::operator==(const rpt_stats_sweep_t &that) const
{
    return
        EQUAL(sweep_id) &&
        EQUAL(sliced_stats_ids);
}

bool
rpt_stats_sweep_t::operator<(const rpt_stats_sweep_t &that) const
{
    LESS_THAN(sweep_id);
    LESS_THAN(sliced_stats_ids);
    return false;
}

END_NAMESPACE;
END_NAMESPACE;
