/// @file analysis/report_codec/report_codec.hh

// Copyright (c) 2010 Acumem AB.  All rights reserved.

#ifndef REPORT_CODEC_HH
#define REPORT_CODEC_HH

#define __STDC_LIMIT_MACROS

#include "analysis/report/report.hh"
#include "library/codec/buffer.hh"
#include "library/stream/stream.hh"
#include "library/warning/warning_consumer.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace acumem::library;

// ----- report_encoder_t -----------------------------------------------------

class report_encoder_t {
public:
    /// Throws codec_exception_t on error.
    report_encoder_t(stream_t &stream);

    /// Throws codec_exception_t on error.
    void encode(rpt_report_cptr_t report);

private:
    stream_t &stream;

    codec_buffer_t header_buffer;
    codec_buffer_t packet_buffer;

    void write_buffer(codec_buffer_t &buffer);
    void write_packet(uint16_t type);

    void encode_location_id_vector(
        const rpt_location_id_vector_t &location_id_vector);
    void encode_location_set(
        const rpt_location_set_t &location_set);
    void encode_location_set_vector(
        const rpt_location_set_vector_t &location_set_vector);
    void encode_stat_fp_vector(
        const rpt_stat_fp_vector_t &stat_fp_vector);

    void encode_uuid(rpt_report_cptr_t report);
    void encode_sampling_info(rpt_report_cptr_t report);
    void encode_analysis_info(rpt_report_cptr_t report);
    void encode_hardware_config(rpt_report_cptr_t report);
    void encode_hardware_config2(rpt_report_cptr_t report);
    void encode_addresses(rpt_report_cptr_t report);
    void encode_access_classes(rpt_report_cptr_t report);
    void encode_trunc_stacks(rpt_report_cptr_t report);
    void encode_full_stacks(rpt_report_cptr_t report);
    void encode_locations(rpt_report_cptr_t report);
    void encode_location_sets(rpt_report_cptr_t report);
    void encode_depgroups(rpt_report_cptr_t report);
    void encode_loops(rpt_report_cptr_t report);
    void encode_stats(rpt_report_cptr_t report);
    void encode_sliced_stats(rpt_report_cptr_t report);
    void encode_location_stats(rpt_report_cptr_t report);
    void encode_location_set_stats(rpt_report_cptr_t report);
    void encode_global_stats(rpt_report_cptr_t report);
    void encode_contexts(rpt_report_cptr_t report);
    void encode_sweeps(rpt_report_cptr_t report);
    void encode_stats_sweeps(rpt_report_cptr_t report);
    void encode_location_set_stats_sweeps(rpt_report_cptr_t report);
    void encode_global_stats_sweep(rpt_report_cptr_t report);
    void encode_thread_mapping(rpt_report_cptr_t report);
    void encode_binaries(rpt_report_cptr_t report);
    void encode_symbols(rpt_report_cptr_t report);
    void encode_functions(rpt_report_cptr_t report);
    void encode_source_files(rpt_report_cptr_t report);
    void encode_source_lines(rpt_report_cptr_t report);
    void encode_debug_pcs(rpt_report_cptr_t report);
    void encode_issue_count(rpt_report_cptr_t report);
    void encode_prefetch_issue(rpt_report_cptr_t report,
                               rpt_issue_cptr_t issue_ptr);
    void encode_hot_spot_issue(rpt_report_cptr_t report,
                               rpt_issue_cptr_t issue_ptr);
    void encode_utilization_issue(rpt_report_cptr_t report,
                                  rpt_issue_cptr_t issue_ptr);
    void encode_loop_order_issue(rpt_report_cptr_t report,
                                 rpt_issue_cptr_t issue_ptr);
    void encode_random_issue(rpt_report_cptr_t report,
                             rpt_issue_cptr_t issue_ptr);
    void encode_loop_fusion_issue(rpt_report_cptr_t report,
                                  rpt_issue_cptr_t issue_ptr);
    void encode_block_issue(rpt_report_cptr_t report,
                            rpt_issue_cptr_t issue_ptr);
    void encode_comm_spot_issue(rpt_report_cptr_t report,
                                rpt_issue_cptr_t issue_ptr);
    void encode_comm_utilization_issue(rpt_report_cptr_t report,
                                       rpt_issue_cptr_t issue_ptr);
    void encode_false_sharing_issue(rpt_report_cptr_t report,
                                    rpt_issue_cptr_t issue_ptr);
    void encode_wnt_possible_issue(rpt_report_cptr_t report,
                                   rpt_issue_cptr_t issue_ptr);
    void encode_prefetchnta_issue(rpt_report_cptr_t report,
                                  rpt_issue_cptr_t issue_ptr);
};

// ----- report_decoder_t -----------------------------------------------------

class report_decoder_t : public stream_t {
public:
    report_decoder_t(warning_consumer_ptr_t warning_consumer);

    stream_status_t process_data(const char *buffer, size_t len);
    stream_status_t flush();

    rpt_report_cptr_t get_report();

private:
    warning_consumer_ptr_t warning_consumer;

    rpt_report_ptr_t report;

    codec_buffer_t buffer;

    bool file_header_handled;

    bool waiting_for_header;
    uint16_t packet_type;
    uint32_t packet_size;

    void decode_location_id_vector(
        rpt_location_id_vector_t &location_id_vector);
    void decode_location_set(
        rpt_location_set_t &location_set);
    void decode_location_set_vector(
        rpt_location_set_vector_t &location_set_vector);
    void decode_stat_fp_vector(
        rpt_stat_fp_vector_t &stat_fp_vector);

    void decode_uuid();
    void decode_sampling_info();
    void decode_analysis_info();
    void decode_hardware_config();
    void decode_hardware_config2();
    void decode_addresses();
    void decode_access_classes();
    void decode_trunc_stacks();
    void decode_full_stacks();
    void decode_locations();
    void decode_location_sets();
    void decode_depgroups();
    void decode_loops();
    void decode_stats();
    void decode_sliced_stats();
    void decode_location_stats();
    void decode_location_set_stats();
    void decode_global_stats();
    void decode_contexts();
    void decode_sweeps();
    void decode_stats_sweeps();
    void decode_location_set_stats_sweeps();
    void decode_global_stats_sweep();
    void decode_thread_mapping();
    void decode_binaries();
    void decode_symbols();
    void decode_functions();
    void decode_source_files();
    void decode_source_lines();
    void decode_debug_pcs();
    void decode_issue_count();
    void decode_prefetch_issue();
    void decode_hot_spot_issue();
    void decode_utilization_issue();
    void decode_loop_order_issue();
    void decode_random_issue();
    void decode_loop_fusion_issue();
    void decode_block_issue();
    void decode_comm_spot_issue();
    void decode_comm_utilization_issue();
    void decode_false_sharing_issue();
    void decode_wnt_possible_issue();
    void decode_prefetchnta_issue();
};

END_NAMESPACE;
END_NAMESPACE;

#endif /* REPORT_CODEC_HH */
