/// @file analysis/report_generator/conf.hh

// Copyright (c) 2011 Acumem AB.  All rights reserved.

#ifndef REPORT_GENERATOR_CONF_HH
#define REPORT_GENERATOR_CONF_HH

#include "debug_lookup.hh"
#include "analysis/thread_mapping/thread_mapping_explicit_context.hh"
#include "analysis/fetch_ratio/windowing.hh"
#include "library/cpudb/cpudb.hh"

#include <string>
#include <vector>
#include <boost/filesystem.hpp>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);
BEGIN_NAMESPACE(report_generator);

using namespace acumem::analysis;
using namespace acumem::library;
using namespace boost::filesystem;
using namespace std;

struct conf_t {
    conf_t(bool &exit,
           int &exit_code,
           warning_consumer_ptr_t &warning_consumer,
	   feedback_ptr_t feedback,
           int argc, char **argv,
           bool feature_mt_advice,
           bool feature_multisocket,
           bool feature_unlimited_callstack,
           bool feature_blocking_expert,
	   bool feature_thread_binding,
           const string &version);

    /// Path to the sample file.
    string sample_file;

    /// Path to the report file.
    string report_file;

    double percentage;

    vector<string> extra_binaries;
    vector<string> debug_directories;
    vector<string> source_directories;

    fnr_cache_size_t cache_size;
    fnr_line_size_t line_size;
    string replacement_policy;

    // cpu_manufacturer may be NULL if the cpu was autodetected.
    cpudb_manufacturer_cptr_t cpu_manufacturer;
    cpudb_cpu_cptr_t cpu;
    unsigned int num_cpus;
    cpudb_cache_cptr_t cache;
    unsigned int cache_level; // 1, 2, 3 ...
    bool autodetect_failed;

    vector<string> analysis_argv;
    fnr_real_time_t analysis_time;

    fnr_stack_depth_t stack_depth;
    string title;

    int debug_symbol_level;

    string cppfilt;

    acumem::analysis::windowing_engine_cptr_t windowing;
    acumem::analysis::windowing_engine_cptr_t windowing_burst;

    // If true, disregard barriers for blocking and fusion issues
    bool blocking_expert;

    // Number of caches, 0 = one cache per thread
    unsigned int num_caches;

    // explicit thread/cache mapping
    acumem::analysis::context_cache_map_t explicit_mapping;

    // Tells whether the explicit_mapping denotes real thread id:s or
    // virtual thread id:s from 0..n
    bool real_thread_id;

    // If true, use the new hwpf algorithm
    bool new_algorithm;

    // If true, enable low memory optimizations
    bool low_mem;

    // If true, enable verbose output.
    bool verbose;

};

END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;

#endif /* CONF_HH*/
