/// @file analysis/report_generator/report_generator.hh

// Copyright (c) 2011 Acumem AB.  All rights reserved.

#ifndef REPORT_GENERATOR_HH
#define REPORT_GENERATOR_HH

#include "debug_lookup.hh"
#include "analysis/report/report.hh"
#include "analysis/fetch_ratio/windowing.hh"
#include "analysis/thread_mapping/thread_mapping_explicit_context.hh"
#include "library/cpudb/cpudb.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);
BEGIN_NAMESPACE(report_generator);

/// Returns a report, or NULL if an error occurred. Propably throws all kinds
/// of exceptions too.
rpt_report_cptr_t
generate_report(
    feature_context_ptr_t context,

    // Analysis time and command line.
    fnr_real_time_t analysis_time,
    const vector<string> &analysis_argv,

    // Report title.
    const string &title,

    // Sample file.
    const string &sample_file,

    // Extra places to look for binaries, debug info and source files.
    const vector<string> &extra_binaries,
    const vector<string> &debug_directories,
    const vector<string> &source_directories,

    // Debug symbol level.
    debug_symbol_level_t debug_symbol_level,

    // cppfilt binary to use.
    const string &cppfilt,

    // Percentage of total fetches/upgrades/write-backs required for an issue to
    // be considered relevant, 0.0 - 100.0.
    double threshold_percentage,

    // Fetch ratio engine.
    const string &replacement_policy,

    // Cache and line size.
    fnr_cache_size_t cache_size,
    fnr_line_size_t line_size,

    // Number of caches, 0 = one cache per thread
    unsigned int num_caches,

    // Cpu and cache configuration. cpu_manufacturer may be NULL if the cpu was
    // autodetected.
    cpudb_manufacturer_cptr_t cpu_manufacturer,
    cpudb_cpu_cptr_t cpu,
    unsigned int num_cpus,
    cpudb_cache_cptr_t cache,
    unsigned int cache_level, // 1, 2, 3 ...

    // Explicit thread/cache mapping. If real_thread_id is true real thread IDs
    // are used, otherwise our internal thread IDs are used.
    const context_cache_map_t &explicit_mapping,
    bool real_thread_id,

    // Call stack depth.
    fnr_stack_depth_t stack_depth,

    // Windowing engines for non-burst and burst sample files.
    windowing_engine_cptr_t windowing_non_burst,
    windowing_engine_cptr_t windowing_burst,

    // Enabled features. Note that this is not simply the licensed features,
    // e.g., blocking expert should only be true if blocking expert advice is
    // both licensed and requested.
    bool feature_mt_advice,
    bool feature_sw_prefetch,
    bool feature_unlimited_callstack,
    bool feature_blocking_expert,
    bool feature_multisocket,

    // If true, use the new hwpf algorithm
    bool new_hwpf_algorithm,

    // If true, enable low memory optimizations
    bool low_mem,

    // If true, enable verbose output.
    bool verbose,

    // If true, mark the report that autodetecting the current cpu and
    // cache hierarchy failed (like in a virtualized environment)
    bool autodetect_failed
);


// Forward declaration
struct conf_t;

/// Shorthand function if you have a conf struct
rpt_report_cptr_t
generate_report(feature_context_ptr_t context,
		const conf_t &conf,

		// Enabled features. Note that this is not simply the
		// licensed features, e.g., blocking expert should
		// only be true if blocking expert advice is both
		// licensed and requested.
		bool feature_mt_advice,
		bool feature_sw_prefetch,
		bool feature_unlimited_callstack,
		bool feature_blocking_expert,
		bool feature_multisocket);

END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;

#endif /* REPORT_GENERATOR_HH */
