/// @file analysis/structure/access_lookup.cc

// Copyright (c) 2007-2010 Acumem AB.  All rights reserved.

#include <boost/lexical_cast.hpp>
#include <boost/foreach.hpp>

#include "access_lookup.hh"

using namespace std;
using namespace boost;
using namespace acumem::library;

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

// ----- access_lookup_t ------------------------------------------------------

access_lookup_result_cptr_t
access_lookup_t::calculate(const feature_context_ptr_t &context,
			   const parser_result_cptr_t &parser_result,
                           const stack_pc_map_result_cptr_t &stack_pc_map)
{
    access_lookup_result_ptr_t result(new access_lookup_result_t());

    // Collect information per pc.
    BOOST_FOREACH(const parser_burst_t &pb, parser_result->bursts) {
	context->feedback->test_cancel();

	BOOST_FOREACH(const sample_ptr_t &sample, pb.samples_by_pc1) {
	    const fnr_event_sample_t *s(sample.get());

	    access_info_t *pc1_access = &result->pc_access_info[s->begin.pc];
	    if (s->begin.access_size > pc1_access->access_size)
		pc1_access->access_size = s->begin.access_size;
	    pc1_access->access_classes |= get_access_classes(s->begin.access_type);

	    BOOST_FOREACH(const fnr_sample_access_t &access, s->end) {
		access_info_t *pc2_access = &result->pc_access_info[access.pc];
		if (access.access_size > pc2_access->access_size)
		    pc2_access->access_size = access.access_size;
		pc2_access->access_classes |= get_access_classes(access.access_type);
	    }
	}
    }

    // Create information per location.
    BOOST_FOREACH(const trunc_stack_pc_id_map_t::value_type &v,
                  stack_pc_map->trunc_stack_pc_id_map) {
        pc_access_info_map_t::const_iterator pc_it =
            result->pc_access_info.find(v.second.pc);
        // There are locations that only occur as pc1+. We don't have any
        // access type or access size info about those.
        if (pc_it != result->pc_access_info.end())
            result->loc_access_info[v.first] = pc_it->second;
    }

    return result;
}

access_lookup_t::access_lookup_t(
    const feature_context_ptr_t &context,
    const string &sample_file,
    const fnr_stack_depth_t stack_depth)
    : feature_t<access_lookup_result_cptr_t>(context),
      sample_file(sample_file),
      stack_depth(stack_depth)
{
}

string
access_lookup_t::name() const
{
    return "access_lookup_t(" +
	lexical_cast<string>(sample_file) + "," +
        lexical_cast<string>(stack_depth) + ")";
}

access_lookup_result_cptr_t
access_lookup_t::get() const
{
    return get_data();
}

access_lookup_result_cptr_t
access_lookup_t::calculate() const
{
    parser_result_cptr_t parser_result =
        parser_t(context, sample_file).get();

    stack_pc_map_result_cptr_t stack_pc_map =
        stack_pc_map_t(context, sample_file, stack_depth).get();

    return calculate(context, parser_result, stack_pc_map);
}

END_NAMESPACE;
END_NAMESPACE;
