/// @file analysis/structure/access_lookup.hh

// Copyright (c) 2007-2008, 2010 Acumem AB.  All rights reserved.

#ifndef ACCESS_LOOKUP_HH
#define ACCESS_LOOKUP_HH

#include "analysis/call_stack/stack_pc_map.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace acumem::library;
using namespace std;

using boost::shared_ptr;

// ----- access_info_t --------------------------------------------------------

/// Information about the accesses done by a pc.
struct access_info_t {
    access_info_t() : access_classes(FNR_Access_Class_None), access_size(0) {}

    /// Set of all access classes performed by this pc.
    ///
    /// @todo For instructions that do multiple independent memory accesses,
    ///       e.g., "pushl (%eax)" or "movsb", we mix up the access types of
    ///       the accesses.
    fnr_access_classes_t access_classes;

    /// The size of the largest access performed by this pc. Note that this is
    /// the largest part of a line that is accessed, e.g., if the only sampled
    /// access of a pc is a 4 byte access split into a 3 byte access and a 1
    /// byte access by a cache line boundary, the pc gets access size 3.
    fnr_access_size_t access_size;
};

typedef map<fnr_address_t,
		 access_info_t> pc_access_info_map_t;

typedef map<location_t, access_info_t> loc_access_info_map_t;

// ----- access_lookup_result_t -----------------------------------------------

/// Lookup result.
struct access_lookup_result_t {
    /// Access information per pc.
    pc_access_info_map_t pc_access_info;

    /// Access information per location (same info as pc_access_info, but
    /// indexed by location for easy access).
    loc_access_info_map_t loc_access_info;
};

typedef shared_ptr<access_lookup_result_t> access_lookup_result_ptr_t;
typedef shared_ptr<const access_lookup_result_t> access_lookup_result_cptr_t;

// ------ access_lookup_t -----------------------------------------------------

class access_lookup_t : public feature_t<access_lookup_result_cptr_t> {
public:
    static access_lookup_result_cptr_t calculate(
	const feature_context_ptr_t &context,
	const parser_result_cptr_t &parser_result,
        const stack_pc_map_result_cptr_t &stack_pc_map);

    access_lookup_t(const feature_context_ptr_t &context,
                    const string &sample_file,
                    const fnr_stack_depth_t stack_depth);
    access_lookup_result_cptr_t get() const;

protected:
    string name() const;
    access_lookup_result_cptr_t calculate() const;

private:
    const string sample_file;
    const fnr_stack_depth_t stack_depth;
};

END_NAMESPACE;
END_NAMESPACE;

#endif /* ACCESS_LOOKUP_HH */
