/// @file analysis/structure/dep_cycle_search.cc

// Copyright (c) 2007-2009 Acumem AB.  All rights reserved.

#include "dep_cycle_search.hh"
#include "dep_tree_search.hh"

#include <boost/foreach.hpp>

using namespace std;
using namespace boost;
using namespace acumem::library;

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

// ----- dep_cycle_search_t ---------------------------------------------------

dep_cycle_search_t::dep_cycle_search_t(const feature_context_ptr_t &context,
                                       const string &sample_file,
                                       const fnr_stack_depth_t stack_depth,
                                       const fnr_line_size_t line_size,
                                       const fnr_line_size_t page_size,
				       const thread_mapping_cptr_t &thread_mapping,
				       const cpu_model_t &cpu_model,
				       const mapped_sample_type_t sample_type,
                                       const fnr_access_filter_t access_filter,
                                       const location_set_t &start_locations,
                                       const fnr_time_t max_rdist,
                                       const bool backwards)
    : context(context),
      sample_file(sample_file),
      stack_depth(stack_depth),
      line_size(line_size),
      page_size(page_size),
      thread_mapping(thread_mapping),
      cpu_model(cpu_model),
      sample_type(sample_type),
      access_filter(access_filter),
      start_locations(start_locations),
      max_rdist(max_rdist),
      backwards(backwards)
{
}

dep_cycle_search_result_cptr_t
dep_cycle_search_t::get()
{
    dep_cycle_search_result_ptr_t res(new dep_cycle_search_result_t());

    dep_tree_search_result_cptr_t dt1 = dep_tree_search_access_t(
        context, sample_file, stack_depth, line_size, page_size,
	thread_mapping, cpu_model, sample_type, access_filter,
	backwards, true, start_locations, max_rdist).get();

    // For each of the reported locations in this result, locate a
    // path back to start_locations. Continue with different max_rdist
    // in each case, depending on the distance to the instruction of
    // the first search.
    BOOST_FOREACH(const location_time_map_t::value_type &v1,
		  dt1->location_time_map) {
	fnr_time_t remaining_rdist = max_rdist - v1.second;

	dep_tree_search_result_cptr_t dt2 =
            dep_tree_search_location_t(
                context, sample_file, stack_depth, line_size, page_size,
		thread_mapping, cpu_model, sample_type, backwards, v1.first,
		start_locations, remaining_rdist).get();

	if (dt2->location_time_map.size()) {
	    fnr_time_t min_distance = (fnr_time_t) -1;
	    BOOST_FOREACH(const location_time_map_t::value_type &v2,
			  dt2->location_time_map) {
		min_distance = min(min_distance, v1.second + v2.second);
	    }
	    res->push_back(dep_cycle_t(v1.first, min_distance));
	}
    }
    return res;
}

END_NAMESPACE;
END_NAMESPACE;
