/// @file analysis/structure/dep_cycle_search.hh

// Copyright (c) 2007-2008 Acumem AB.  All rights reserved.

#ifndef DEP_CYCLE_SEARCH_HH
#define DEP_CYCLE_SEARCH_HH

#include <vector>
#include <boost/shared_ptr.hpp>
#include "library/lang/common_macros.h"
#include "analysis/thread_mapping/mapped_sample.hh"
#include "analysis/thread_mapping/cpu_model.hh"
#include "analysis/call_stack/stack_pc_map.hh"
#include "analysis/thread_mapping/thread_mapping.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace std;
using namespace acumem::library;

using boost::shared_ptr;

// ----- dep_cycle_search_result_t --------------------------------------------

/// Search result.
struct dep_cycle_t {

    dep_cycle_t(const location_t &location,
		const fnr_time_t cycle_length)
	: location(location), cycle_length(cycle_length)
	{}

    /// The location found by the search.
    location_t location;

    /// The accumulated minimum length of the cycle involving this location
    fnr_time_t cycle_length;
};

typedef vector<dep_cycle_t> dep_cycle_vector_t;
typedef dep_cycle_vector_t dep_cycle_search_result_t; // alias
typedef shared_ptr<dep_cycle_search_result_t> dep_cycle_search_result_ptr_t;
typedef shared_ptr<const dep_cycle_search_result_t>
        dep_cycle_search_result_cptr_t;

// ----- dep_cycle_search_t ---------------------------------------------------

/// Search for a cycle from and back to 'start_locations', involving
/// at least one access of the type specified by the 'access_classes'
/// mask, completing the cycle in less than max_rdist accesses.
class dep_cycle_search_t {
public:
    dep_cycle_search_t(const feature_context_ptr_t &context,
                       const string &sample_file,
                       const fnr_stack_depth_t stack_depth,
                       const fnr_line_size_t line_size,
                       const fnr_line_size_t page_size,
		       const thread_mapping_cptr_t &thread_mapping,
		       const cpu_model_t &cpu_model,
		       const mapped_sample_type_t sample_type,
                       const fnr_access_filter_t access_filter,
                       const location_set_t &start_locations,
                       const fnr_time_t max_rdist,
                       const bool backwards);

    dep_cycle_search_result_cptr_t get();

private:
    const feature_context_ptr_t context;

    /// Sample file.
    const string sample_file;

    /// Call stack depth to use in the analysis.
    const fnr_stack_depth_t stack_depth;

    /// This is the line size that we are interested in.
    const fnr_line_size_t line_size;

    /// Page size, required by sample_mapping_t.
    const fnr_line_size_t page_size;

    /// Thread to cache mappings.
    const thread_mapping_cptr_t thread_mapping;

    /// CPU model
    cpu_model_t cpu_model;

    /// Sample type to use
    const mapped_sample_type_t sample_type;

    const fnr_access_filter_t access_filter;
    const location_set_t start_locations;
    const fnr_time_t max_rdist;
    const bool backwards;
};

END_NAMESPACE;
END_NAMESPACE;

#endif
