/// @file analysis/structure/loop_depgroups.cc

// Copyright (c) 2008-2009 Acumem AB.  All rights reserved.

#include "analysis/structure/loop_depgroups.hh"
#include "analysis/structure/loop_identification.hh"
#include "analysis/structure/dependence_group.hh"
#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace acumem::library;
using namespace boost;
using namespace std;

loop_depgroups_t::loop_depgroups_t(
    const feature_context_ptr_t &context,
    const string &sample_file,
    const fnr_stack_depth_t stack_depth,
    const fnr_line_size_t line_size,
    const fnr_line_size_t page_size,
    const thread_mapping_cptr_t &thread_mapping,
    const cpu_model_t &cpu_model,
    const mapped_sample_type_t sample_type,
    const int limit_factor)
    : feature_t<loop_depgroups_result_cptr_t>(context),
      sample_file(sample_file),
      stack_depth(stack_depth),
      line_size(line_size),
      page_size(page_size),
      thread_mapping(thread_mapping),
      cpu_model(cpu_model),
      sample_type(sample_type),
      limit_factor(limit_factor)
{
}

loop_depgroups_result_cptr_t
loop_depgroups_t::get()
{
    return get_data();
}

string
loop_depgroups_t::name() const
{
    return "loop_depgroups(" +
        sample_file + "," +
        lexical_cast<string>(stack_depth) + "," +
        lexical_cast<string>(line_size) + "," +
        lexical_cast<string>(page_size) + "," +
	thread_mapping->name() + "," +
	cpu_model.name() + "," +
	lexical_cast<string>(sample_type) + "," +
        lexical_cast<string>(limit_factor) + ")";
}

loop_depgroups_result_cptr_t
loop_depgroups_t::calculate() const
{
    loop_depgroups_result_ptr_t result(new loop_depgroups_result_t());

    loop_identification_result_cptr_t loop_result = loop_identification_t(
        context, sample_file, stack_depth).get();

    // Dummy entry for the non-existing loop 0.
    result->loop_depgroups.push_back(depgroup_id_vector_t());

    location_set_t non_loop_pcs;
    stack_pc_map_result_cptr_t stack_pc_map = stack_pc_map_t(
        context, sample_file, stack_depth).get();
    BOOST_FOREACH(trunc_stack_pc_id_map_t::value_type v,
                  stack_pc_map->trunc_stack_pc_id_map)
        non_loop_pcs.insert(v.first);

    BOOST_FOREACH(const loop_ptr_t &loop, loop_result->loops) {
        assert(loop->id == (loop_id_t)result->loop_depgroups.size());

        location_set_t loop_pcs = loop->get_locations(false);
        dependence_group_result_cptr_t loop_depgroups = dependence_group_t(
            context, sample_file, stack_depth, line_size, page_size,
	    thread_mapping, cpu_model, sample_type, limit_factor,
	    loop_pcs).get();
        depgroup_id_vector_t loop_depgroup_ids;
        BOOST_FOREACH(const location_set_t &depgroup,
                      loop_depgroups->pc_sets) {
            loop_depgroup_ids.push_back((depgroup_id_t)result->depgroups.size());
            result->depgroups.push_back(depgroup);
            BOOST_FOREACH(location_t pc , depgroup)
                non_loop_pcs.erase(pc);
        }
        result->loop_depgroups.push_back(loop_depgroup_ids);
    }

    dependence_group_result_cptr_t non_loop_depgroups = dependence_group_t(
        context, sample_file, stack_depth, line_size, page_size,
	thread_mapping, cpu_model, sample_type, limit_factor,
	non_loop_pcs).get();
    BOOST_FOREACH(const location_set_t &depgroup,
                  non_loop_depgroups->pc_sets)
        result->depgroups.push_back(depgroup);

    return result;
}

END_NAMESPACE;
END_NAMESPACE;
