/// @file analysis/structure/loop_depgroups.hh

// Copyright (c) 2008 Acumem AB.  All rights reserved.

#ifndef LOOP_DEPGROUPS_HH
#define LOOP_DEPGROUPS_HH

#include "analysis/call_stack/stack_pc_map.hh"
#include "analysis/thread_mapping/thread_mapping.hh"
#include "analysis/thread_mapping/mapped_sample.hh"
#include "analysis/thread_mapping/cpu_model.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace acumem::library;
using namespace std;

using boost::shared_ptr;

// ----- loop_depgroups_result_t --------------------------------------------

typedef int depgroup_id_t;

typedef vector<depgroup_id_t> depgroup_id_vector_t;

typedef vector<depgroup_id_vector_t> depgroup_id_vector_vector_t;

struct loop_depgroups_result_t {
    /// Vector of dependence groups. Indexed by dependence group id.
    location_set_vector_t depgroups;

    /// A vector containing a vector of indexes into the depgroups vector for
    /// each loop. Indexed by loop id.
    depgroup_id_vector_vector_t loop_depgroups;
};

typedef shared_ptr<loop_depgroups_result_t> loop_depgroups_result_ptr_t;
typedef shared_ptr<const loop_depgroups_result_t> loop_depgroups_result_cptr_t;

// ----- loop_depgroups_t ---------------------------------------------------

/// Module for calculating a set of dependence groups for a pc-set.
class loop_depgroups_t : public feature_t<loop_depgroups_result_cptr_t> {
public:
    loop_depgroups_t(const feature_context_ptr_t &context,
                     const string &sample_file,
                     const fnr_stack_depth_t stack_depth,
                     const fnr_line_size_t line_size,
                     const fnr_line_size_t page_size,
		     const thread_mapping_cptr_t &thread_mapping,
		     const cpu_model_t &cpu_model,
		     const mapped_sample_type_t sample_type,
                     const int limit_factor);
    loop_depgroups_result_cptr_t get();

protected:
    string name() const;
    loop_depgroups_result_cptr_t calculate() const;

private:
    /// Sample file.
    const string sample_file;

    // Call stack depth to use in the analysis.
    const fnr_stack_depth_t stack_depth;

    /// Cache line size.
    const fnr_line_size_t line_size;

    /// Page size, required by sample_mapping_t.
    const fnr_line_size_t page_size;

    /// Thread to cache mappings.
    const thread_mapping_cptr_t thread_mapping;

    /// CPU model
    const cpu_model_t cpu_model;

    /// Sample type to use
    const mapped_sample_type_t sample_type;

    /// Dependency group limit factor.
    const int limit_factor;
};

END_NAMESPACE;
END_NAMESPACE;

#endif
