/// @file analysis/thread_mapping/cpu_model.cc

// Copyright (c) 2008 Acumem AB.  All rights reserved.

#include <boost/lexical_cast.hpp>

#include "cpu_model.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace boost;

cpu_model_t::cpu_model_t()
    : active_pf(FNR_Prefetch_Class_Any),
      coherence_pf(FNR_Prefetch_Class_Any),
      prefetchw_makes_modified(false)
{
}

cpu_model_t::cpu_model_t(
    fnr_prefetch_classes_t active_pf,
    fnr_prefetch_classes_t coherence_pf,
    bool prefetchw_makes_modified)
    : active_pf(active_pf),
      coherence_pf(coherence_pf),
      prefetchw_makes_modified(prefetchw_makes_modified)
{
}

cpu_model_t::cpu_model_t(
    const cpudb_cpu_cptr_t &cpu,
    const cpudb_cache_cptr_t &cache)
    : active_pf(FNR_Prefetch_Class_Unknown),
      coherence_pf(FNR_Prefetch_Class_Unknown),
      prefetchw_makes_modified(cpu->prefetchw_dirty)
{
    // Assume that all prefetches to this level and all levels below
    // are active.
    BOOST_FOREACH(const cpudb_cache_cptr_t &c, cpu->caches) {
	if (c->data)
	    active_pf |= c->active_prefetches;

	if (c == cache)
	    break;
    }

    // All prefetches that are active for the current CPU may cause
    // coherence effects.
    BOOST_FOREACH(const cpudb_cache_cptr_t &c, cpu->caches) {
	if (c->data)
	    coherence_pf |= c->active_prefetches;
    }
}

string
cpu_model_t::name() const
{
    return "cpu_model_t(" +
    	lexical_cast<string>(active_pf) + "," +
	lexical_cast<string>(coherence_pf) + "," +
	lexical_cast<string>(prefetchw_makes_modified) + ")";
}

END_NAMESPACE;
END_NAMESPACE;
