/// @file analysis/thread_mapping/thread_mapping_explicit.cc

// Copyright (c) 2008-2010 Acumem AB.  All rights reserved.

#include "thread_mapping_explicit.hh"

#include <boost/lexical_cast.hpp>

using namespace std;
using namespace boost;
using namespace acumem::library;

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

static void
add_mapping(tm_cache_map_t &cache_map,
            fnr_thread_id_t thread,
            tm_cache_id_t &next_cache,
            tm_cache_id_t num_caches)
{
    tm_cache_map_t::const_iterator cmi = cache_map.find(thread);
    if (cmi == cache_map.end()) {
        cache_map[thread] = next_cache;
	next_cache = (next_cache + 1) % num_caches;
    }
}

thread_mapping_result_cptr_t
thread_mapping_explicit_t::calculate(feature_context_ptr_t context,
				     parser_result_cptr_t parser_result,
				     tm_cache_id_t num_caches,
				     const tm_cache_map_t &explicit_map)
{
    thread_mapping_result_ptr_t result(
	new thread_mapping_result_t(parser_result->bursts.size()));

    // Sanity check.
    BOOST_FOREACH(const tm_cache_map_t::value_type &v, explicit_map)
        assert(v.second < num_caches);

    // Prime the cache_map with the explicit entries.
    tm_cache_map_t cache_map(explicit_map);

    // Map threads without explicit mapping in round robin fashion.
    tm_cache_id_t next_cache(0);
    BOOST_FOREACH(const parser_burst_t &burst, parser_result->bursts) {
	context->feedback->test_cancel();

        BOOST_FOREACH(const sample_ptr_t &sample, burst.samples_by_pc1) {
            add_mapping(cache_map, sample->begin.thread, next_cache, num_caches);
            BOOST_FOREACH(const fnr_sample_access_t &access, sample->end)
                add_mapping(cache_map, access.thread, next_cache, num_caches);
        }
    }

    for (uint32_t tid = 0; tid < parser_result->contexts.size(); tid++)
        add_mapping(cache_map, tid, next_cache, num_caches);

    // Use the resulting cache mapping for all bursts.
    BOOST_FOREACH(thread_mapping_burst_t &burst, result->bursts)
        burst.time_cache_map[0] = cache_map;

    // And set the number of caches.
    result->num_caches = num_caches;

    return result;
}

thread_mapping_explicit_t::thread_mapping_explicit_t(feature_context_ptr_t context,
						     const string &sample_file,
						     const tm_cache_id_t num_caches,
						     const tm_cache_map_t &explicit_mapping)
    : feature_t<thread_mapping_result_cptr_t>(context),
      sample_file(sample_file),
      num_caches(num_caches),
      explicit_mapping(explicit_mapping)
{
}

thread_mapping_result_cptr_t
thread_mapping_explicit_t::get() const
{
    return get_data();
}

string
thread_mapping_explicit_t::name() const
{
    string mapping;
    BOOST_FOREACH(tm_cache_map_t::value_type v, explicit_mapping)
	mapping += "," + lexical_cast<string>(v.first) + ":" +
	lexical_cast<string>(v.second);

    return "thread_mapping_explicit_t(" +
        sample_file + "," +
        lexical_cast<string>(num_caches) +
        mapping + ")";
}

thread_mapping_result_cptr_t
thread_mapping_explicit_t::calculate() const
{
    return calculate(context,
		     parser_t(context, sample_file).get(),
		     num_caches,
		     explicit_mapping);
}

END_NAMESPACE;
END_NAMESPACE;
