/// @file analysis/thread_mapping/thread_mapping_explicit_context.cc

// Copyright (c) 2009 Acumem AB.  All rights reserved.

#include "thread_mapping_explicit_context.hh"
#include "thread_mapping_explicit.hh"

#include <boost/lexical_cast.hpp>

using namespace std;
using namespace boost;
using namespace acumem::library;

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

thread_mapping_result_cptr_t
thread_mapping_explicit_context_t::calculate(thread_mapping_result_cptr_t result)
{
    return result;
}

thread_mapping_explicit_context_t::thread_mapping_explicit_context_t(feature_context_ptr_t context,
						     const string &sample_file,
						     const tm_cache_id_t num_caches,
								     const context_cache_map_t &explicit_mapping,
								     bool real_thread_id)
    : feature_t<thread_mapping_result_cptr_t>(context),
      sample_file(sample_file),
      num_caches(num_caches),
      explicit_mapping(explicit_mapping),
      real_thread_id(real_thread_id)
{
}

thread_mapping_result_cptr_t
thread_mapping_explicit_context_t::get() const
{
    return get_data();
}

string
thread_mapping_explicit_context_t::name() const
{
    std::string mapping;
    BOOST_FOREACH(context_cache_map_t::value_type v, explicit_mapping)
	mapping += "," + lexical_cast<string>(v.first) + ":" +
	lexical_cast<string>(v.second);

    return "thread_mapping_explicit_context_t(" +
        sample_file + "," +
        lexical_cast<string>(num_caches) +
        mapping + ")";
}

thread_mapping_result_cptr_t
thread_mapping_explicit_context_t::calculate() const
{
    parser_result_cptr_t parser_result = parser_t(context, sample_file).get();

    // Convert our context to cache map to a thread to cache map.
    tm_cache_map_t cache_map;

    if (real_thread_id) {
	BOOST_FOREACH(const context_cache_map_t::value_type &v, explicit_mapping) {
	    for (fnr_context_id_vector_t::size_type i = 0; i < parser_result->contexts.size(); i++)
		if (parser_result->contexts[i] == v.first)
		    cache_map[(fnr_context_id_t) i] = v.second;
	}
    } else {
	BOOST_FOREACH(const context_cache_map_t::value_type &v, explicit_mapping)
	    cache_map[v.first] = v.second;
    }

    // Use thread mapping explicit to calculate the mapping.
    thread_mapping_result_cptr_t thread_mapping_result =
        thread_mapping_explicit_t(context, sample_file, num_caches, cache_map).get();

    // Perform "calculation".
    return calculate(thread_mapping_result);
}

END_NAMESPACE;
END_NAMESPACE;
