/// @file analysis/thread_mapping/thread_mapping_explicit_context.hh

// Copyright (c) 2009 Acumem AB.  All rights reserved.

#ifndef THREAD_MAPPING_EXPLICIT_CONTEXT_HH
#define THREAD_MAPPING_EXPLICIT_CONTEXT_HH

#include "analysis/thread_mapping/thread_mapping.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

using namespace std;
using namespace acumem::library;

typedef map<fnr_context_id_t, tm_cache_id_t> context_cache_map_t;

/// Module providing an explicit context to cache mapping. Threads that have no
/// explicit mapping are allocated to the caches in round robin fashion.
class thread_mapping_explicit_context_t
    : public thread_mapping_t,
      public feature_t<thread_mapping_result_cptr_t> {
public:
    static thread_mapping_result_cptr_t calculate(
        thread_mapping_result_cptr_t result);

    thread_mapping_explicit_context_t(feature_context_ptr_t context,
                                      const string &sample_file,
                                      const tm_cache_id_t num_caches,
                                      const context_cache_map_t &explicit_mapping,
				      bool real_thread_id);
    thread_mapping_result_cptr_t get() const;

protected:
    string name() const;
    thread_mapping_result_cptr_t calculate() const;

private:
    const string sample_file;
    const tm_cache_id_t num_caches;
    const context_cache_map_t explicit_mapping;
    const bool real_thread_id;
};

END_NAMESPACE;
END_NAMESPACE;

#endif
