/// @file analysis/thread_mapping/thread_mapping_null.cc

// Copyright (c) 2008-2010 Acumem AB.  All rights reserved.

#include "thread_mapping_null.hh"
#include "analysis/filter/filter_module.hh"

#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>

using namespace std;
using namespace boost;
using namespace acumem::library;

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(analysis);

static void
add_mapping(tm_cache_map_t &cm,
            tm_cache_id_t &current_cache,
            fnr_thread_id_t thread)
{
    bool inserted = cm.insert(make_pair(thread, current_cache)).second;
    if (inserted)
	current_cache++;
}

thread_mapping_result_cptr_t
thread_mapping_null_t::calculate(feature_context_ptr_t context,
				 parser_result_cptr_t parser_result)
{
    thread_mapping_result_ptr_t result(
	new thread_mapping_result_t(parser_result->bursts.size()));
    tm_cache_id_t current_cache(0);


    thread_mapping_burst_vector_t::iterator it_tmb(result->bursts.begin());
    BOOST_FOREACH(const parser_burst_t &burst, parser_result->bursts) {
	context->feedback->test_cancel();

	tm_time_cache_map_t &time_cache_map((it_tmb++)->time_cache_map);
	tm_cache_map_t &cache_map(time_cache_map[0]);

	BOOST_FOREACH(const sample_ptr_t &sample, burst.samples_by_pc1) {
	    add_mapping(cache_map, current_cache, sample->begin.thread);
	    BOOST_FOREACH(const fnr_sample_access_t &access, sample->end)
		add_mapping(cache_map, current_cache, access.thread);
	}

        for (uint32_t tid = 0; tid < parser_result->contexts.size(); tid++)
            add_mapping(cache_map, current_cache, tid);
    }

    result->num_caches = current_cache;

    return result;
}

thread_mapping_null_t::thread_mapping_null_t(feature_context_ptr_t context,
                                             const string &sample_file)
    : feature_t<thread_mapping_result_cptr_t>(context),
      sample_file(sample_file)
{
}

thread_mapping_result_cptr_t
thread_mapping_null_t::get() const
{
    return get_data();
}

string
thread_mapping_null_t::name() const
{
    return "thread_mapping_null_t(" + sample_file + ")";
}

thread_mapping_result_cptr_t
thread_mapping_null_t::calculate() const
{
    return calculate(context, parser_t(context, sample_file).get());
}

END_NAMESPACE;
END_NAMESPACE;
