/** @file library/acquisition/acquisition.h */

/* Copyright (c) 2009-2010 Acumem AB.  All rights reserved. */

#ifndef INCLUDED_ACQUISITION_H
#define INCLUDED_ACQUISITION_H

#include "library/lang/common_macros.h"
#include "library/lang/common_types.h"

BEGIN_EXTERN_C

enum acq_arch_t {
    Acq_I386,
    Acq_X86_64
};

typedef struct {
    int version;
    int arch; /* acq_arch_t */
    int64_t start_time;
    const char *user;
    int argc;
    char **argv;
} acq_begin_t;


typedef struct {
    int64_t end_time;
} acq_end_t;

typedef struct {
    int burst_generation;
    int64_t burst_start_time;
    int64_t burst_sampling_length;
    int64_t burst_cooldown_length;
} acq_burst_begin_t;

typedef struct {
    int burst_generation;
    int64_t burst_end_time;
} acq_burst_end_t;


typedef struct {
    uint64_t pr_vaddr;
    uint64_t pr_size;
    int flag_read;
    int flag_write;
    int flag_exec;
    int flag_shared;
    uint64_t pr_offset;
    uint8_t major;
    uint8_t minor;
    uint64_t inode;
    char *path;
} acq_mapping_entry_t;

typedef struct {
    int n_entries;
    acq_mapping_entry_t **entries;
} acq_mapping_t;


struct acq_handle_t;
typedef struct acq_handle_t acq_handle_t;


acq_handle_t *acq_create(const char *filename);
int acq_close(acq_handle_t *handle);
int acq_fd(acq_handle_t *handle);

int acq_write_begin(acq_handle_t *handle, const acq_begin_t *acq_begin);
int acq_write_end(acq_handle_t *handle, const acq_end_t *acq_end);
int acq_write_burst_begin(acq_handle_t *handle, const acq_burst_begin_t *burst_begin);
int acq_write_burst_end(acq_handle_t *handle, const acq_burst_end_t *burst_end);
int acq_write_mapping(acq_handle_t *handle, const acq_mapping_t *mapping);

END_EXTERN_C

#endif
