/** @file library/am_mem/am_mem.c */

/* Copyright (c) 2006-2008, 2010 Acumem AB.  All rights reserved. */

#include "am_mem.h"

#include <stdio.h>
#include <string.h>

void *
am_malloc(size_t size)
{
    void *ptr = malloc(size);
    if (ptr == NULL) {
        fprintf(stderr, "Error: malloc failed, exiting\n");
        exit(1);
    }
    return ptr;
}

char *
am_strdup(const char *str)
{
    char *ptr = strdup(str);
    if (ptr == NULL) {
        fprintf(stderr, "Error: strdup failed, exiting\n");
        exit(1);
    }
    return ptr;
}


char *
am_strndup(const char *str, size_t n)
{
    const char *end = memchr(str, '\0', n);
    size_t len = end == NULL ? n : end -  str;
    char *ptr = malloc(len + 1);
    if (ptr == NULL) {
        fprintf(stderr, "Error: strndup failed, exiting\n");
        exit(1);
    }
    memcpy(ptr, str, len);
    ptr[len] = '\0';
    return ptr;
}

void *
am_realloc(void *ptr, size_t size)
{
    ptr = realloc(ptr, size);
    if (ptr == NULL && size != 0) {
        fprintf(stderr, "Error: realloc failed, exiting\n");
        exit(1);
    }
    return ptr;
}

void
am_free(void *ptr)
{
    free(ptr);
}
