/// @file library/atf_resampler/access_type.cc

// Copyright (c) 2009 Acumem AB.  All rights reserved.

#include "access_type.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

/// Which access types start which line watchpoint access types, indexed by
/// fnr_access_type_t. */
const access_type_set_t line_start_types[] = {
    0 | ATS_R |                  ATS_PF |            ATS_PF_NTA | ATS_PF_T0 | ATS_PF_T1 | ATS_PF_T2 | ATS_R_NT | ATS_W_NT | ATS_F,
    0 | ATS_R | ATS_W | ATS_RW | ATS_PF | ATS_PF_W | ATS_PF_NTA | ATS_PF_T0 | ATS_PF_T1 | ATS_PF_T2 | ATS_R_NT | ATS_W_NT | ATS_F,
    0 /* FNR_Access_Type_PF, should never happen. */                                                                             ,
    0 | ATS_R | ATS_W | ATS_RW | ATS_PF | ATS_PF_W | ATS_PF_NTA | ATS_PF_T0 | ATS_PF_T1 | ATS_PF_T2 | ATS_R_NT | ATS_W_NT | ATS_F,
    0 | ATS_R |                  ATS_PF |            ATS_PF_NTA | ATS_PF_T0 | ATS_PF_T1 | ATS_PF_T2 | ATS_R_NT | ATS_W_NT | ATS_F,
    0 | ATS_R |                  ATS_PF | ATS_PF_W | ATS_PF_NTA | ATS_PF_T0 | ATS_PF_T1 | ATS_PF_T2 | ATS_R_NT | ATS_W_NT | ATS_F,
    0 | ATS_R |                  ATS_PF |            ATS_PF_NTA | ATS_PF_T0 | ATS_PF_T1 | ATS_PF_T2 | ATS_R_NT | ATS_W_NT | ATS_F,
    0 | ATS_R |                  ATS_PF |            ATS_PF_NTA | ATS_PF_T0 | ATS_PF_T1 | ATS_PF_T2 | ATS_R_NT | ATS_W_NT | ATS_F,
    0 | ATS_R |                  ATS_PF |            ATS_PF_NTA | ATS_PF_T0 | ATS_PF_T1 | ATS_PF_T2 | ATS_R_NT | ATS_W_NT | ATS_F,
    0 | ATS_R |                  ATS_PF |            ATS_PF_NTA | ATS_PF_T0 | ATS_PF_T1 | ATS_PF_T2 | ATS_R_NT | ATS_W_NT | ATS_F,
    0 | ATS_R | ATS_W | ATS_RW | ATS_PF | ATS_PF_W | ATS_PF_NTA | ATS_PF_T0 | ATS_PF_T1 | ATS_PF_T2 | ATS_R_NT | ATS_W_NT | ATS_F,
    0 | ATS_R |                  ATS_PF |            ATS_PF_NTA | ATS_PF_T0 | ATS_PF_T1 | ATS_PF_T2 | ATS_R_NT | ATS_W_NT | ATS_F,
    0 | ATS_R | ATS_W | ATS_RW | ATS_PF | ATS_PF_W | ATS_PF_NTA | ATS_PF_T0 | ATS_PF_T1 | ATS_PF_T2 | ATS_R_NT | ATS_W_NT | ATS_F
};

/// Which access types end which line watchpoint access types, indexed by
/// fnr_access_type_t.
const access_type_set_t line_end_types[] = {
    0 | ATS_R |                  ATS_PF |            ATS_PF_NTA | ATS_PF_T0 | ATS_PF_T1 | ATS_PF_T2 | ATS_R_NT | ATS_W_NT | ATS_F,
    0 | ATS_R | ATS_W | ATS_RW | ATS_PF | ATS_PF_W | ATS_PF_NTA | ATS_PF_T0 | ATS_PF_T1 | ATS_PF_T2 | ATS_R_NT | ATS_W_NT | ATS_F,
    0 /* FNR_Access_Type_PF, should never happen. */                                                                             ,
    0 | ATS_R | ATS_W | ATS_RW | ATS_PF | ATS_PF_W | ATS_PF_NTA | ATS_PF_T0 | ATS_PF_T1 | ATS_PF_T2 | ATS_R_NT | ATS_W_NT | ATS_F,
    0 |                          ATS_PF                                                                                          ,
    0 |                                   ATS_PF_W                                                                               ,
    0 |                                              ATS_PF_NTA                                                                  ,
    0 |                                                           ATS_PF_T0                                                      ,
    0 |                                                                       ATS_PF_T1                                          ,
    0 |                                                                                   ATS_PF_T2                              ,
    0 | ATS_R | ATS_W | ATS_RW | ATS_PF | ATS_PF_W | ATS_PF_NTA | ATS_PF_T0 | ATS_PF_T1 | ATS_PF_T2 | ATS_R_NT | ATS_W_NT | ATS_F,
    0 | ATS_R |                  ATS_PF |            ATS_PF_NTA | ATS_PF_T0 | ATS_PF_T1 | ATS_PF_T2 | ATS_R_NT | ATS_W_NT | ATS_F,
    0 | ATS_R | ATS_W | ATS_RW | ATS_PF | ATS_PF_W | ATS_PF_NTA | ATS_PF_T0 | ATS_PF_T1 | ATS_PF_T2 | ATS_R_NT | ATS_W_NT | ATS_F
};

/// Which access types start which byte watchpoint access types, indexed by
/// fnr_access_type_t.
const access_type_set_t byte_start_types[] = {
    0 | ATS_R |                                                                                       ATS_R_NT | ATS_W_NT        ,
    0 |         ATS_W |                                                                                          ATS_W_NT        ,
    0 /* FNR_Access_Type_PF, should never happen. */                                                                             ,
    0 | ATS_R | ATS_W | ATS_RW |                                                                      ATS_R_NT | ATS_W_NT        ,
    0                                                                                                                            ,
    0                                                                                                                            ,
    0                                                                                                                            ,
    0                                                                                                                            ,
    0                                                                                                                            ,
    0                                                                                                                            ,
    0                                                                                                                            ,
    0 | ATS_R |                                                                                       ATS_R_NT | ATS_W_NT        ,
    0 | ATS_R | ATS_W | ATS_RW |                                                                      ATS_R_NT | ATS_W_NT
};

/// Which access types end which byte watchpoint access types, indexed by
/// fnr_access_type_t.
const access_type_set_t byte_end_types[] = {
    0 | ATS_R |                                                                                       ATS_R_NT | ATS_W_NT        ,
    0 |         ATS_W |                                                                                          ATS_W_NT        ,
    0 /* FNR_Access_Type_PF, should never happen. */                                                                             ,
    0 | ATS_R | ATS_W | ATS_RW |                                                                      ATS_R_NT | ATS_W_NT        ,
    0                                                                                                                            ,
    0                                                                                                                            ,
    0                                                                                                                            ,
    0                                                                                                                            ,
    0                                                                                                                            ,
    0                                                                                                                            ,
    0                                                                                                                            ,
    0 | ATS_R |                                                                                       ATS_R_NT | ATS_W_NT        ,
    0 | ATS_R | ATS_W | ATS_RW |                                                                      ATS_R_NT | ATS_W_NT
};

access_type_set_t
get_access_type_set(fnr_access_type_t access_type)
{
    switch (access_type) {
    case FNR_Access_Type_RD:
        return ATS_R;
    case FNR_Access_Type_WR:
        return ATS_W;
    case FNR_Access_Type_RDWR:
        return ATS_RW;
    case FNR_Access_Type_PF_AMD:
        return ATS_PF;
    case FNR_Access_Type_PF_AMD_W:
        return ATS_PF_W;
    case FNR_Access_Type_PF_Intel_NTA:
        return ATS_PF_NTA;
    case FNR_Access_Type_PF_Intel_T0:
        return ATS_PF_T0;
    case FNR_Access_Type_PF_Intel_T1:
        return ATS_PF_T1;
    case FNR_Access_Type_PF_Intel_T2:
        return ATS_PF_T2;
    case FNR_Access_Type_NTA_RD:
        return ATS_R_NT;
    case FNR_Access_Type_NTA_WR:
        return ATS_W_NT;
    case FNR_Access_Type_CLFLUSH:
        return ATS_F;
    default:
        assert(0);
        while (1);
    }
}

END_NAMESPACE;
END_NAMESPACE;
