/// @file library/atf_resampler/access_type.hh

// Copyright (c) 2009 Acumem AB.  All rights reserved.

#ifndef ACCESS_TYPE_HH
#define ACCESS_TYPE_HH

#include "library/lang/common_macros.h"
#include "library/lang/common_types.h"
#include "library/fenris/fenris.hh"

#include <map>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

using namespace std;

/// Access type set using a bitmap with one bit per access type.
typedef uint16_t access_type_set_t;

/// Values of access_type_set_t.
#define ATS_R      (1 << FNR_Access_Type_RD)
#define ATS_W      (1 << FNR_Access_Type_WR)
#define ATS_RW     (1 << FNR_Access_Type_RDWR)
#define ATS_PF     (1 << FNR_Access_Type_PF_AMD)
#define ATS_PF_W   (1 << FNR_Access_Type_PF_AMD_W)
#define ATS_PF_NTA (1 << FNR_Access_Type_PF_Intel_NTA)
#define ATS_PF_T0  (1 << FNR_Access_Type_PF_Intel_T0)
#define ATS_PF_T1  (1 << FNR_Access_Type_PF_Intel_T1)
#define ATS_PF_T2  (1 << FNR_Access_Type_PF_Intel_T2)
#define ATS_R_NT   (1 << FNR_Access_Type_NTA_RD)
#define ATS_W_NT   (1 << FNR_Access_Type_NTA_WR)
#define ATS_F      (1 << FNR_Access_Type_CLFLUSH)

/// Access types that do not actually read or write any data in the line.
#define ATS_CLASS_NO_DATA \
    (ATS_PF | ATS_PF_W | ATS_PF_NTA | ATS_PF_T0 | ATS_PF_T1 | ATS_PF_T2 | ATS_F)

/// Access types that cause a flush of the cache line in the cache where they
/// are executed.
#define ATS_CLASS_FLUSH \
    (ATS_F)

/// Access types that maybe cause an upgrade if the line is in shared state.
#define ATS_CLASS_MAYBE_UPGRADE \
    (ATS_W | ATS_RW | ATS_PF_W)

/// Access types that always cause an upgrade if the line is in shared state.
#define ATS_CLASS_ALWAYS_UPGRADE \
    (ATS_W | ATS_RW)

/// Access types that always cause a downgrade of the line if it is in
/// exclusive or modified state in another cache.
#define ATS_CLASS_ALWAYS_DOWNGRADE \
    (ATS_R | ATS_R_NT)

/// Access types that maybe cause an invalidation of the line if it is not
/// already invalid in another cache.
#define ATS_CLASS_MAYBE_INVALIDATION \
    (ATS_W | ATS_RW | ATS_PF_W | ATS_W_NT | ATS_F)

/// Access types that always cause an invalidation of the line if it is not
/// already invalid in another cache.
#define ATS_CLASS_ALWAYS_INVALIDATION \
    (ATS_W | ATS_RW | ATS_W_NT | ATS_F)



extern const access_type_set_t line_start_types[];
extern const access_type_set_t line_end_types[];
extern const access_type_set_t byte_start_types[];
extern const access_type_set_t byte_end_types[];


/// Seen access types by thread id.
typedef map<fnr_thread_id_t, access_type_set_t> thread_access_type_set_t;

/// Return the access type set corresponding to the fenris access type.
access_type_set_t get_access_type_set(fnr_access_type_t access_type);



END_NAMESPACE;
END_NAMESPACE;

#endif
