/// @file library/atf_resampler/debug_log.hh

// Copyright (c) 2009 Acumem AB.  All rights reserved.

#ifndef INCLUDED_DEBUG_LOG
#define INCLUDED_DEBUG_LOG

#include "library/lang/common_macros.h"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

// Enable to get debug log printouts. The printouts are identical to the debug
// log printouts from the sampler for easier debugging.
//#define DEBUG_LOG


#if defined(DEBUG_LOG)

static inline void
debug_log_mem(const fnr_sample_access_t *access,
              unsigned long long local_macnt,
              const char *event_type)
{
    static const char * const access_type_names[] = {
        "rd", "wr", "pf", "rdwr", "pf_amd", "pf_amd_w", "pf_intel_nta",
        "pf_intel_t0", "pf_intel_t1", "pf_intel_t2", "clflush", "rd_nt",
        "wr_nt"};

    printf("%5u%12llu%12llu, %s %u @ %lx: %s\n",
           (unsigned int)access->thread, local_macnt, (long long)access->time,
           access_type_names[access->access_type], access->access_size,
           access->addr.virtual_address, event_type);
}

static inline void
debug_log_wp(unsigned int tid, unsigned int group_id, unsigned int wp_id, const char *event_type,
             unsigned long start, unsigned int length)
{
    printf("%7u: id %u:%u: %s %u @ %lx\n",
           tid, group_id, wp_id, event_type, length, start);
}

static inline void
debug_log_kill(unsigned int group_id, unsigned int wp_id)
{
    printf("kill: id %u:%u\n", group_id, wp_id);
}

#define DEBUG_LOG_MEM(access, local_macnt, event_type) \
    debug_log_mem(access, local_macnt, event_type)
#define DEBUG_LOG_WP(tid, group_id, wp_id, event_type, start, length)   \
    debug_log_wp(tid, group_id, wp_id, event_type, start, length)
#define DEBUG_LOG_KILL(group_id, wp_id)          \
    debug_log_kill(group_id, wp_id)
#else // !defined(DEBUG_LOG)
#define DEBUG_LOG_MEM(access, local_macnt, event_type)
#define DEBUG_LOG_WP(tid, group_id, wp_id, event_type, start, length)
#define DEBUG_LOG_KILL(group_id, wp_id)
#endif // defined(DEBUG_LOG)


END_NAMESPACE;
END_NAMESPACE;

#endif
