/// @file library/atf_resampler/exceptions.hh

// Copyright (c) 2009 Acumem AB.  All rights reserved.

#ifndef EXCEPTIONS_HH
#define EXCEPTIONS_HH

#include "library/lang/common_macros.h"
#include "library/lang/common_types.h"
#include "library/fenris/flow.hh"
#include "library/warning/warning.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

using namespace std;

class resampler_exception_t
    : public flow_exception_t
{
public:
    resampler_exception_t(const string &what_)
	: flow_exception_t(what_)
	{}
};

class resampler_not_enough_samples_warning_t
    : public warning_base_t
{
public:
    resampler_not_enough_samples_warning_t(int num_samples,
					   int min_samples,
					   int desired_samples,
					   fnr_time_t max_sample_period,
					   fnr_time_t desired_sample_period)
	: num_samples(num_samples),
	  min_samples(min_samples),
	  desired_samples(desired_samples),
	  max_sample_period(max_sample_period),
	  desired_sample_period(desired_sample_period)
	{}
    string what() const;

    /// Number of samples found, for the line size with the fewest
    /// samples.
    int num_samples;
    /// Minimum number of samples for reliable result
    int min_samples;
    /// Desired number of samples
    int desired_samples;
    /// Maximum sample period to get the reliale results.
    fnr_time_t max_sample_period;
    /// Desired sample period to get the reliale results.
    fnr_time_t desired_sample_period;
};


class resampler_not_enough_bursts_warning_t
    : public warning_base_t
{
public:
    resampler_not_enough_bursts_warning_t(int num_samples,
                                          int min_samples,
                                          int desired_samples)
	: num_samples(num_samples),
	  min_samples(min_samples),
	  desired_samples(desired_samples)
	{}
    string what() const;

    /// Number of samples found, for the line size with the fewest
    /// samples.
    int num_samples;
    /// Minimum number of samples for reliable result
    int min_samples;
    /// Desired number of samples
    int desired_samples;
};

class resampler_premature_eof_t
    : public warning_base_t
{
public:
    string what() const {
	return "Sampler output contains a premature end of file. "
	    "Sampling may have terminated prematurely.";
    }
};


END_NAMESPACE;
END_NAMESPACE;

#endif
