/// @file library/atf_resampler/page_watchpoint.cc

// Copyright (c) 2009 Acumem AB.  All rights reserved.

#include "page_watchpoint.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

int
page_watchpoint_t::apply_access_impl(const fnr_sample_access_t *access,
				     stack_t *stack,
				     fnr_line_size_t line_size,
				     bool must_be_last)
{
    int ret = 0x0;
    if (!terminated && !must_be_last) {

	// Log the first access for each thread and each access type

	access_type_set_t access_type_set =
	    get_access_type_set(access->access_type);
        if (!(seen_access_types[access->thread.id] & access_type_set)) {
            seen_access_types[access->thread.id] |= access_type_set;
            ret |= 0x1;
        }

	// Mark the watchpoint as finished as soon as we see a regular
	// access.

        if (access->thread == tid &&
            (!(access_type_set & ATS_CLASS_NO_DATA) ||
             access->access_type == start_access_type)) {
            terminated = true;
            return 0x3;
        }
    }

    return ret;
}

page_watchpoint_t::page_watchpoint_t(flow_atf_resampler_t *resampler,
				     thread_t *thread,
				     uint64_t wp_addr,
				     wp_group_t *wp_group,
				     const fnr_sample_access_t *access)
    : watchpoint_t(resampler, thread, wp_addr, PAGE_SAMPLE_SIZE_LOG2,
		   wp_group, access),
      start_access_type(access->access_type)
{
}

END_NAMESPACE;
END_NAMESPACE;

