/// @file library/atf_resampler/stack.hh

// Copyright (c) 2009 Acumem AB.  All rights reserved.

#ifndef STACK_HH
#define STACK_HH

#include "library/lang/common_macros.h"
#include "library/lang/common_types.h"
#include "library/fenris/fenris.hh"

#include <map>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

using namespace std;

/// A call stack.
struct stack_t {
    /// Allocates memory for a stack of the specified depth. Sets the depth
    /// of the stack to 0 so that it can be filled in by push_stack.
    stack_t(int depth);

    /// Returns true if the stacks are equal. Does not look at the stack id
    /// or next pointer.
    inline bool operator==(const stack_t &rhs);

    /// Pushes a pc onto a stack, and returns an updated hash value.
    inline uint64_t push(fnr_address_t pc, uint64_t hash) {
	ret_addrs.push_back(pc);
	return ((hash >> 3) | (hash << 61)) ^ pc.virtual_address;
    }

    /// Stack id, or 0 if the stack has not been assigned an id.
    fnr_stack_id_t id;

    /// Next stack in the watchpoint hash table conflict list.
    stack_t *next;

    /// Return addresses on the call stack, outermost first.
    fnr_address_vector_t ret_addrs;
};

inline bool
stack_t::operator==(const stack_t &rhs)
{
    if (ret_addrs.size() != rhs.ret_addrs.size())
	return false;

    for (unsigned int i = 0; i < ret_addrs.size(); i++) {
	if (ret_addrs[i].virtual_address !=
	    rhs.ret_addrs[i].virtual_address)
	    return false;
    }

    return true;
}

/// Vector of call stacks.
typedef vector<stack_t *> stack_vector_t;

/// The last access we record consist of a access and its corresponding stack.
typedef pair<fnr_sample_access_t, stack_t *> access_stack_t;

/// The last access of each type from a specific thread.
typedef map<fnr_access_type_t, access_stack_t> thread_last_access_t;

/// The last access of each type from each thread.
typedef map<fnr_thread_id_t, thread_last_access_t> last_access_t;

/// Map between access times and stacks, namely which stack corresponds to what access
typedef map<fnr_time_t, stack_t *> time_stack_map_t;

END_NAMESPACE;
END_NAMESPACE;

#endif
