/// @file library/atf_resampler/thread.hh

// Copyright (c) 2009-2010 Acumem AB.  All rights reserved.

#ifndef INCLUDED_THREAD_HH
#define INCLUDED_THREAD_HH

#include "library/lang/common_macros.h"
#include "library/lang/common_types.h"
#include "library/fenris/fenris.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

using namespace std;

/// Thread structure.
class thread_t {
public:
    thread_t(fnr_context_id_t context_id,
	     fnr_thread_id_t thread_id)
	: context_id(context_id),
	  thread_id(thread_id),
	  local_time(0),
          greatest_local_time(0),
	  waiting_for_pc_1_plus(NULL),
	  last_local_begin_time(0),
	  stack(NULL),
	  last_stack(NULL),
	  effective_sample_period(0.) {}

    /// Context id - original, possibly recurring thread id
    fnr_context_id_t context_id;

    /// Thread id - that is our new artificial non-recurring id
    fnr_thread_id_t thread_id;

    /// The time of the last ATF event for this thread.
    fnr_time_t local_time;

    /// The greatest local time seen in any ATF event for this thread.
    fnr_time_t greatest_local_time;

    /// Watchpoint group waiting for pc1+ access, or NULL if none.
    wp_group_t *waiting_for_pc_1_plus;

    /// Last local sample begin access time of the thread.
    fnr_time_t last_local_begin_time;

    /// Current call stack of the thread, NULL if no stack is available.
    stack_t *stack;

    /// Last relative call stack of the thread, NULL if no stack is available.
    stack_t *last_stack;

    /// Effective sample period of the next sample begun by the thread.
    double effective_sample_period;
};


END_NAMESPACE;
END_NAMESPACE;

#endif
