/// @file library/atf_resampler/wp_group.hh

// Copyright (c) 2009 Acumem AB.  All rights reserved.

#ifndef WP_GROUP_HH
#define WP_GROUP_HH

// Need definition of access_type_set_t
#include "access_type.hh"

#include "watchpoint.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

using namespace std;

class flow_atf_resampler_t;
class thread_t;
struct fnr_sample_access_t;

/// Representation of an access that started a sample. References all
/// watchpoints that were started by the access so that they can be removed
/// if the access is removed during decimation.
class wp_group_t {
public:
    /// Create a sampled access, and also create the associated watchpoints
    /// and insert them into the data structures.
    wp_group_t(flow_atf_resampler_t *resampler,
	       thread_t *thread,
	       const fnr_sample_access_t *access);

    /// Delete the sampled access, and also remove the associated
    /// watchpoints from the data structures and delete them.
    ~wp_group_t();

    /// Sets flags on all samples in the group
    void set_sample_flag(fnr_sample_flags_t flags);

    /// Add a pc1+ access.
    void pc_1_plus(
	thread_t *thread,
	const fnr_sample_access_t *access);

    /// Returns true of the access has any unterminated watchpoints.
    bool has_live_wps();

    /// Mark this group as complete, except for missing a pc_1_plus event.
    void mark_complete_lacking_pc_1_plus() {
	complete_lacking_pc_1_plus = true;
    }

    ///
    bool is_complete_lacking_pc_1_plus() const {
	return complete_lacking_pc_1_plus;
    }

     /// The resampler.
    flow_atf_resampler_t *resampler;

    /// Watchpoint group id.
    fnr_sample_count_t id;

    /// Access type of the start acess.
    access_type_set_t start_access_type;

    /// Keep track of if the access has gotten a pc1+ access yet.
    bool has_pc_1_plus;

    /// Byte watchpoints.
    watchpoint_vector_t byte_wps;

    /// Maps line sizes to vectors of watchpoints of that size.
    watchpoint_vector_map_t line_wps;

    /// Page watchpoints.
    watchpoint_vector_t page_wps;

    /// Effective sample period. Starts out as the sample period of the
    /// thread when the access was done, and is then updated by each
    /// decimation pass.
    double effective_sample_period;

    /// True if the access has been selected for decimation.
    bool decimated;

    fnr_time_t last_sampler_time;

    // True if we have received all pc_2 but no pc_1_plus.
    bool complete_lacking_pc_1_plus;
};


END_NAMESPACE;
END_NAMESPACE;

#endif
