/// @file library/cache_sim/cachesim_rand.cc

// Copyright (c) 2006-2008, 2010 Acumem AB.  All rights reserved.

#include "cachesim.hh"

#include "library/lang/compat.h"
#include "library/random_lcg/random_lcg.h"

#include <iostream>
#include <iomanip>
#include <boost/format.hpp>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

using namespace std;
using namespace boost;

void
cachesim_rand_t::init()
{
    random_lcg = random_lcg_new((uint32_t) seed);

    uint32_t indexes_ = (uint32_t) indexes;

    rv.reserve(indexes_);
    for (uint32_t i = 0; i < indexes_; i++) {
	fnr_address_vector_t rv1;
	rv.push_back(rv1);
	fnr_address_vector_t &rv2 = rv[i];

	cachesim_line_map_t &m = v[i];
	cachesim_line_map_t::const_iterator map_iter;
	cachesim_line_map_t::const_iterator map_end;

	map_iter = m.begin();
	map_end = m.end();

	for (; map_iter != map_end; map_iter++) {
	    rv2.push_back(map_iter->first);
	}
    }
}

cachesim_rand_t::cachesim_rand_t(fnr_cache_size_t size,
                                 fnr_line_size_t line_size,
                                 fnr_line_count_t assoc,
                                 bool calc_footprint,
                                 bool calc_spatial_usage,
                                 string name,
                                 uint64_t seed)
    : cachesim_base_t(
        size, line_size, assoc, calc_footprint, calc_spatial_usage, name),
      seed(seed)
{
    init();
}

cachesim_rand_t::cachesim_rand_t(fnr_cache_size_t size,
                                 fnr_line_size_t line_size,
                                 fnr_line_count_t assoc,
                                 bool calc_footprint,
                                 bool calc_spatial_usage,
                                 uint64_t seed)
    : cachesim_base_t(
        size, line_size, assoc, calc_footprint, calc_spatial_usage),
      seed(seed)
{
    init();
}

cachesim_rand_t::~cachesim_rand_t()
{
    random_lcg_delete(random_lcg);
}

void
cachesim_rand_t::dbgpr()
{
    cachesim_base_t::dbgpr();

    cout << "index random-vector" << endl;
    uint32_t indexes_ = (uint32_t) indexes;

    for (uint32_t i = 0; i<indexes_; i++) {
	fnr_address_vector_t &rand_vector = rv[i];
	fnr_address_vector_t::const_iterator
	    vector_iter = rand_vector.begin(),
	    vector_end = rand_vector.end();

	cout << format("%1%") % io::group(setfill(' '), dec, setw(4), i);
	for (; vector_iter != vector_end; vector_iter++) {
	    cout << format("  %1%:%2%")
		% io::group(setfill('0'), hex, setw(4), 0 /*vector_iter->tag*/)
		% io::group(setfill('0'), hex, setw(8), vector_iter->virtual_address);
	}
	cout << endl;
    }
}

fnr_address_t
cachesim_rand_t::get_victim(const fnr_address_t &memop, fnr_line_count_t index)
{
    fnr_address_vector_t &index_rv = rv[(uint32_t) index];
    fnr_line_count_t victim;

    cachesim_line_map_t &m = v[(uint32_t) index];
    if (m.find(memop) != m.end()) {
        // TODO: Limitation in the current random replacement model; if there
        // is an invalid line with the same tag as the line that caused the
        // replacement we must evict that line. If we don't v and rv get out of
        // sync.
        for (fnr_address_vector_t::size_type
                 i = 0; i < index_rv.size(); i++)
            if (index_rv.at(i) == memop) {
                victim = i;
                goto found;
            }
        assert(0);
        while (1); /* Avoid warning on Solaris. */
    found:
        ;
    }
    else
        /// @todo This isn't really statistically correct...
        victim = random_lcg_next(random_lcg) % conf.assoc;

    victim_cache = &index_rv[(uint32_t) victim];

    return index_rv[(uint32_t) victim];
}

void
cachesim_rand_t::replacement_update_after_miss(const fnr_address_t &memop, fnr_line_count_t index)
{
    *victim_cache = memop;
}

void
cachesim_rand_t::replacement_update_after_hit(const fnr_address_t &memop, fnr_line_count_t index)
{
    // do nothing
}

END_NAMESPACE;
END_NAMESPACE;
