/// @file library/codec/buffer.cc

// Copyright (c) 2006-2010 Acumem AB.  All rights reserved.

#include "buffer.hh"
#include "library/am_mem/am_mem.h"

#include <string.h>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

codec_buffer_t::codec_buffer_t()
    : buffer(NULL),
      allocated_bytes(0),
      written_bytes(0),
      read_bytes(0),
      assert_read_bytes(0)
{
}

codec_buffer_t::~codec_buffer_t()
{
    am_free(buffer);
}

size_t
codec_buffer_t::size() const
{
    return written_bytes - read_bytes;
}

void
codec_buffer_t::write(const char *data, size_t size)
{
    if (allocated_bytes - written_bytes < size) {
        // We need more space. Compact the data to the beginning of the buffer.
        memmove(buffer, buffer + read_bytes, written_bytes - read_bytes);
        written_bytes -= read_bytes;
        assert_read_bytes -= read_bytes;
        read_bytes = 0;

        // Allocate more memory so that at most half of the buffer is in use.
        size_t used = written_bytes + size;
        if (used * 2 > allocated_bytes) {
            allocated_bytes = 1024;
            while (used * 2 > allocated_bytes)
                allocated_bytes *= 2;
            buffer = (char *)am_realloc(buffer, allocated_bytes);
        }
    }

    memcpy(buffer + written_bytes, data, size);
    written_bytes += size;
}

const char *
codec_buffer_t::read(size_t size)
{
    if (size > written_bytes - read_bytes)
        throw codec_exception_t("Read beyond end of buffer.", true);

    const char *ret = buffer + read_bytes;
    read_bytes += size;
    return ret;
}

void
codec_buffer_t::assert_read(size_t size)
{
    if (read_bytes != assert_read_bytes)
        throw codec_exception_t("Unexpected number of bytes read.", true);

    assert_read_bytes = read_bytes + size;
}

END_NAMESPACE;
END_NAMESPACE;
