/// @file library/codec/buffer.hh

// Copyright (c) 2006, 2008, 2010 Acumem AB.  All rights reserved.

#ifndef BUFFER_HH
#define BUFFER_HH

#include "library/stream/stream.hh"

#include <stdexcept>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

using namespace std;

class codec_exception_t : public std::runtime_error {
public:
    codec_exception_t(const string &what,
                      bool error)
        : std::runtime_error(what),
          _error(error)
        {
        }

    ~codec_exception_t() throw()
        {
        }

    bool error() {
        return _error;
    }

private:
    const bool _error;
};

class codec_buffer_t {
public:
    codec_buffer_t();
    ~codec_buffer_t();

    /// Number of bytes in the buffer.
    size_t size() const;

    /// Append data to the buffer.
    void write(const char *data, size_t size);

    /// Returns a pointer to the size first bytes of data and removes them from
    /// the buffer. The pointer is only valid until the next method call to the
    /// buffer.
    ///
    /// Throws codec_exception_t if there is not enough data to satisfy the
    /// read.
    const char *read(size_t size);

    /// Assert that exactly size bytes will be read before the next time
    /// assert_read() is called.
    ///
    /// Throws codec_exception_t if the expected number of bytes has not been
    /// read since the last call.
    void assert_read(size_t size);

private:
    char *buffer;
    size_t allocated_bytes;
    size_t written_bytes;
    size_t read_bytes;
    size_t assert_read_bytes;
};

END_NAMESPACE;
END_NAMESPACE;

#endif /* BUFFER_HH */
