/// @file library/cpuid/cpuid_amd.hh

// Copyright (c) 2008-2009 Acumem AB.  All rights reserved.

#ifndef CPUID_AMD_HH
#define CPUID_AMD_HH

#include "library/cpuid/cpuid.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

/// Query AMD processors for their cpu information and cache information
class cpuid_cpu_amd_t
    : public cpuid_cpu_t
{
public:
    static const char VENDOR_ID[];

    cpuid_cpu_amd_t(warning_consumer_ptr_t warning_consumer);

    unsigned int get_thread_bits() const;
    unsigned int get_core_bits() const;

protected:
    void collect_apic_ids_impl();

    cpuid_cpu_info_cptr_t query_cpu_info() const;
    cpuid_cache_info_cptr_t query_cache_info() const;

private:
    unsigned int get_max_cores() const;

    void detect_cores();
    void add_l1_cache(int reg, bool data, bool instruction);
    void add_l2_cache(int reg);
    void add_l3_cache(int reg);

    cpuid_cache_level_t get_l1_cache(
        const int reg,
        const cpuid_apic_id_t owner_apic_id,
        const unsigned int apic_mask,
        const bool data,
        const bool instructions) const;
    cpuid_cache_level_t get_l2_cache(
        const int reg,
        const cpuid_apic_id_t owner_apic_id,
        const int apic_mask) const;
    cpuid_cache_level_t get_l3_cache(
        const int reg,
        const cpuid_apic_id_t owner_apic_id,
        const unsigned int apic_mask) const;
};

END_NAMESPACE;
END_NAMESPACE;

#endif
