/// @file library/cpuid/cpuid_intel.hh

// Copyright (c) 2008-2009 Acumem AB.  All rights reserved.

#ifndef CPUID_INTEL_HH
#define CPUID_INTEL_HH

#include "library/cpuid/cpuid.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

/// Query Intel processors for their cpu information and cache information
class cpuid_cpu_intel_t
    : public cpuid_cpu_t
{
public:
    static const char VENDOR_ID[];

    cpuid_cpu_intel_t(warning_consumer_ptr_t warning_consumer);

    unsigned int get_thread_bits() const;
    unsigned int get_core_bits() const;

protected:
    void collect_apic_ids_impl();

    cpuid_cpu_info_cptr_t query_cpu_info() const;
    cpuid_cache_info_cptr_t query_cache_info() const;

private:
    cpuid_cache_info_ptr_t query_cache_info_legacy(
	const cpuid_apic_id_t owner_apic_id) const;
    cpuid_cache_info_ptr_t query_cache_info_deterministic(
	const cpuid_apic_id_t owner_apic_id) const;

    void add_legacy_cache(cpuid_cache_vector_t &caches,
                          const cpuid_apic_id_t owner_apic_id,
                          const int cache_type) const;
    void add_legacy_caches(cpuid_cache_vector_t &caches,
                           const cpuid_apic_id_t owner_apic_id,
                           const int reg,
                           const int max_caches) const;

    bool supports_x2apic;
};

END_NAMESPACE;
END_NAMESPACE;

#endif
