/// @file library/cpuid/cpuid_os.hh

// Copyright (c) 2008-2009 Acumem AB.  All rights reserved.

#ifndef CPUID_OS_HH
#define CPUID_OS_HH

#include "library/lang/common_macros.h"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

/// Registers passed to (only eax and ecx) and from the cpuid instruction.
struct cpuid_regs_t {
    cpuid_regs_t()
	: eax(0),
          ebx(0),
          ecx(0),
          edx(0)
        {}

    cpuid_regs_t(unsigned int eax,
                 unsigned int ebx,
                 unsigned int ecx,
                 unsigned int edx)
	: eax(eax),
          ebx(ebx),
          ecx(ecx),
          edx(edx)
        {}

    unsigned int eax;
    unsigned int ebx;
    unsigned int ecx;
    unsigned int edx;
};

/// Operating system dependent functions to query a particular core about its
/// processor / cache information
struct cpuid_os_t {

    /// Get the maximum number of CPUs that may be installed on this system.
    static int get_cpu_max();

    /// Check if a specific CPU is installed.
    static bool cpu_exists(unsigned int cpu_nr);

    /// Check if a specific CPU is installed and online.
    static bool cpu_online(unsigned int cpu_nr);

    /// Execute CPUID on the current CPU.
    static void cpuid(cpuid_regs_t &regs);

    /// Execute CPUID on the specified CPU.
    static void cpuid_cpu(const unsigned int cpu_nr, cpuid_regs_t &regs);
};

END_NAMESPACE;
END_NAMESPACE;

#endif
