/// @file library/exe_path/exe_path.hh

// Copyright (c) 2010 Acumem AB.  All rights reserved.

#include "library/lang/common_macros.h"

#include <boost/filesystem.hpp>
#include <stdio.h>

#ifndef EXE_PATH_HH
#define EXE_PATH_HH

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(exe_path);

/// Return the absolute path to the current executable. Throws
/// boost::system::system_error if there is a problem
boost::filesystem::path get_executable_filename();

/// Return the canonical installation directory. Throws
/// boost::system::system_error if there is a problem
boost::filesystem::path get_installation_directory();

/// Return the canonical common directory for license files
boost::filesystem::path get_app_license_file_directory();

/// Return the canonical user directory for license files
boost::filesystem::path get_user_license_file_directory();

/// Return the absolute path to the temp directory, taking specific
/// environment variables and system defaults into account
boost::filesystem::path get_temp_path();

/// Open a file
FILE *fopen(const boost::filesystem::path &path, const std::string &mode);

/// Create a file, and return false if it already existed. Throw
/// exception if there is any other error.
bool create_file_fail_if_exists(const boost::filesystem::path &path);

END_NAMESPACE;
END_NAMESPACE;

#endif /* EXE_PATH_HH */
