/// @file library/fenris/fenris.cc

// Copyright (c) 2006-2010 Acumem AB.  All rights reserved.

#include <stdio.h>

#include <boost/format.hpp>
#include <boost/foreach.hpp>

#include "fenris.hh"

using namespace std;
using namespace boost;

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

using namespace std;

const char *
get_access_type_str(fnr_access_type_t access_type)
{
    switch (access_type) {
    case FNR_Access_Type_RD:
        return "RD";
    case FNR_Access_Type_WR:
        return "WR";
    case FNR_Access_Type_PF:
        return "PF";
    case FNR_Access_Type_RDWR:
        return "RDWR";
    case FNR_Access_Type_PF_AMD:
        return "PF_AMD";
    case FNR_Access_Type_PF_AMD_W:
        return "PF_AMD_W";
    case FNR_Access_Type_PF_Intel_NTA:
        return "PF_Intel_NTA";
    case FNR_Access_Type_PF_Intel_T0:
        return "PF_Intel_T0";
    case FNR_Access_Type_PF_Intel_T1:
        return "PF_Intel_T1";
    case FNR_Access_Type_PF_Intel_T2:
        return "PF_Intel_T2";
    case FNR_Access_Type_CLFLUSH:
        return "CLFLUSH";
    case FNR_Access_Type_NTA_RD:
        return "NTA_RD";
    case FNR_Access_Type_NTA_WR:
        return "NTA_WR";
    default:
        assert(0);
        while(1);
    }
}

ostream &
fnr_event_read_t::stream_put(ostream &os) const
{
    os << "[event-read]     pc<" << pad(pc)
       << ">, addr<" << pad(address)
       << ">, thread_id: " << thread_id
       << ", size: " << size;

    return os;
}

bool
fnr_event_read_t::operator==(const fnr_event_t &rhs) const
{
    if (rhs.get_kind() != FNR_Event_Read)
	return false;

    const fnr_event_read_t &read =
	static_cast<const fnr_event_read_t &>(rhs);

    return
	pc == read.pc &&
	address == read.address &&
	thread_id == read.thread_id &&
	size == read.size;
}

ostream &
fnr_event_write_t::stream_put(ostream &os) const
{
    os << "[event-write]    pc<" << pad(pc)
       << ">, addr<" << pad(address)
       << ">, thread_id: " << thread_id
       << ", size: " << size;

    return os;
}

bool
fnr_event_write_t::operator==(const fnr_event_t &rhs) const
{
    if (rhs.get_kind() != FNR_Event_Write)
	return false;

    const fnr_event_write_t &write =
	static_cast<const fnr_event_write_t &>(rhs);

    return
	pc == write.pc &&
	address == write.address &&
	thread_id == write.thread_id &&
	size == write.size;
}

ostream &
fnr_event_prefetch_t::stream_put(ostream &os) const
{
    os << "[event-prefetch] pc<" << pad(pc)
       << ">, addr<" << pad(address)
       << ">, thread_id: " << thread_id
       << ", size: " << size;

    return os;
}

bool
fnr_event_prefetch_t::operator==(const fnr_event_t &rhs) const
{
    if (rhs.get_kind() != FNR_Event_Prefetch)
	return false;

    const fnr_event_prefetch_t &prefetch =
	static_cast<const fnr_event_prefetch_t &>(rhs);

    return
	pc == prefetch.pc &&
	address == prefetch.address &&
	thread_id == prefetch.thread_id &&
	size == prefetch.size;
}

ostream &
fnr_event_sample_begin_t::stream_put(ostream &os) const
{
    os << "[event-smpl-bgn] pc<" << pad(pc)
       << ">, pc+1<" << pad(pc_plus_one)
       << ">, addr<" << pad(address)
       << ">, thread_id: " << thread_id
       << ", sample_id: " << sample_id
       << ", time: " << time
       << ", kind: " << get_access_type_str(kind)
       << ", access_size: " << access_size
       << ", line_size: " << line_size;

    return os;
}

bool
fnr_event_sample_begin_t::operator==(const fnr_event_t &rhs) const
{
    if (rhs.get_kind() != FNR_Event_Sample_Begin)
	return false;

    const fnr_event_sample_begin_t &sample_begin =
	static_cast<const fnr_event_sample_begin_t &>(rhs);

    return
	time == sample_begin.time &&
	thread_id == sample_begin.thread_id &&
	sample_id == sample_begin.sample_id &&
	pc == sample_begin.pc &&
	pc_plus_one == sample_begin.pc_plus_one &&
	address == sample_begin.address &&
	kind == sample_begin.kind &&
	access_size == sample_begin.access_size &&
	line_size == sample_begin.line_size;
}

ostream &
fnr_event_sample_end_t::stream_put(ostream &os) const
{
    os << "[event-smpl-end] pc<" << pad(pc)
       << ">, addr<" << pad(address)
       << ">, thread_id: " << thread_id
       << ", sample_id: " << sample_id
       << ", time: " << time
       << ", kind:" << get_access_type_str(kind)
       << ", access_size: " << access_size
       << ", rdist: " << rdist;

    return os;
}

bool
fnr_event_sample_end_t::operator==(const fnr_event_t &rhs) const
{
    if (rhs.get_kind() != FNR_Event_Sample_End)
	return false;

    const fnr_event_sample_end_t &sample_end =
	static_cast<const fnr_event_sample_end_t &>(rhs);

    return
	time == sample_end.time &&
	thread_id == sample_end.thread_id &&
	sample_id == sample_end.sample_id &&
	pc == sample_end.pc &&
	address == sample_end.address &&
	kind == sample_end.kind &&
	access_size == sample_end.access_size &&
	rdist == sample_end.rdist;
}

ostream &
fnr_event_sample_period_t::stream_put(ostream &os) const
{
    os << "[event-smpl-rt]  sample_rate: " << sample_period;

    return os;
}

bool
fnr_event_sample_period_t::operator==(const fnr_event_t &rhs) const
{
    if (rhs.get_kind() != FNR_Event_Sample_Period)
	return false;

    const fnr_event_sample_period_t &rhs_sp =
	static_cast<const fnr_event_sample_period_t &>(rhs);

    return
	sample_period == rhs_sp.sample_period;
}

ostream &
fnr_event_acquisition_begin_t::stream_put(ostream &os) const
{
    os << "[acq-begin]      time: " << time
       << ", user: " << user
       << ", argv:";

    BOOST_FOREACH(const string &a, argv) {
	os << " " << a;
    }

    os << ", line sizes: ";

    BOOST_FOREACH(fnr_line_size_log2_t ls2, line_sizes_log2) {
	os << " " << line_size_from_log2(ls2);
    }

    return os;
}

bool
fnr_event_acquisition_begin_t::operator==(const fnr_event_t &rhs) const
{
    if (rhs.get_kind() != FNR_Event_Acquisition_Begin)
	return false;

    const fnr_event_acquisition_begin_t &acquisition_begin =
	static_cast<const fnr_event_acquisition_begin_t &>(rhs);

    return
	time == acquisition_begin.time &&
	architecture == acquisition_begin.architecture &&
	argv == acquisition_begin.argv &&
	user == acquisition_begin.user &&
	line_sizes_log2 == acquisition_begin.line_sizes_log2;
}

ostream &
fnr_event_acquisition_end_t::stream_put(ostream &os) const
{
    os << "[acq-end]        time: " << time;

    return os;
}

bool
fnr_event_acquisition_end_t::operator==(const fnr_event_t &rhs) const
{
    if (rhs.get_kind() != FNR_Event_Acquisition_End)
	return false;

    const fnr_event_acquisition_end_t &acquisition_end =
	static_cast<const fnr_event_acquisition_end_t &>(rhs);

    return
	time == acquisition_end.time;
}

ostream &
fnr_event_mmap_t::stream_put(ostream &os) const
{
    os << "[mmap]           address: " << format("0x%016x") % address
       << ", size: " << size
       << ", attr: " << attr
       << ", device: " << device
       << ", inode: " << inode
       << ", time: " << global_time
       << ", offset: " << format("0x%016x") % offset
       << ", filename: " << filename;

    return os;
}

bool
fnr_event_mmap_t::operator==(const fnr_event_t &rhs) const
{
    if (rhs.get_kind() != FNR_Event_Mmap)
	return false;

    const fnr_event_mmap_t &mmap =
	static_cast<const fnr_event_mmap_t &>(rhs);

    return
	address == mmap.address &&
	size == mmap.size &&
	attr == mmap.attr &&
	device == mmap.device &&
	inode == mmap.inode &&
	filename == mmap.filename &&
	offset == mmap.offset &&
	global_time == mmap.global_time;
}

ostream &
fnr_event_munmap_t::stream_put(ostream &os) const
{
    os << "[munmap]         address: " << format("0x%016x") % address
       << ", size: " << size
       << ", time: " << global_time;

    return os;
}

bool
fnr_event_munmap_t::operator==(const fnr_event_t &rhs) const
{
    if (rhs.get_kind() != FNR_Event_Munmap)
	return false;

    const fnr_event_munmap_t &munmap =
	static_cast<const fnr_event_munmap_t &>(rhs);

    return
	address == munmap.address &&
	size == munmap.size &&
	global_time == munmap.global_time;
}

ostream &
fnr_event_stack_t::stream_put(ostream &os) const
{
    os << "[stack]          id: " << id
       << ", return addresses:";

    BOOST_FOREACH(const fnr_address_t &a, ret_addrs) {
	os << " " << a;
    }

    return os;
}

bool
fnr_event_stack_t::operator==(const fnr_event_t &rhs) const
{
    if (rhs.get_kind() != FNR_Event_Stack)
	return false;

    const fnr_event_stack_t &stack =
	static_cast<const fnr_event_stack_t &>(rhs);

    return
	id == stack.id &&
	ret_addrs == stack.ret_addrs;
}

ostream &
fnr_event_time_mapping_t::stream_put(ostream &os) const
{
    os << "[time_mapping]" << endl;
    BOOST_FOREACH(const time_event_t &e, time_events)
        os << "  time " << e.global_time << ", tid " << e.tid << endl;

    return os;
}

bool
fnr_event_time_mapping_t::operator==(const fnr_event_t &rhs) const
{
    if (rhs.get_kind() != FNR_Event_Time_Mapping)
	return false;

    const fnr_event_time_mapping_t *that =
	&static_cast<const fnr_event_time_mapping_t &>(rhs);

    if (this->time_events.size() != that->time_events.size())
        return false;

    time_event_vector_t::size_type num = this->time_events.size();
    for (time_event_vector_t::size_type i = 0; i < num; i++) {
        const time_event_t *this_e = &this->time_events[i];
        const time_event_t *that_e = &that->time_events[i];
        if (this_e->global_time != that_e->global_time ||
            this_e->tid != that_e->tid)
            return false;
    }

    return true;
}

ostream &
fnr_event_local_time_t::stream_put(ostream &os) const
{
    os << "[local_time]     tid: " << this->tid
       << ", global time: " << this->global_time
       << ", local time: " << this->local_time << endl;

    return os;
}

bool
fnr_event_local_time_t::operator==(const fnr_event_t &rhs) const
{
    if (rhs.get_kind() != FNR_Event_Local_Time)
	return false;

    const fnr_event_local_time_t *that =
	&static_cast<const fnr_event_local_time_t &>(rhs);

    return
        this->tid == that->tid &&
        this->global_time == that->global_time &&
        this->local_time == that->local_time;
}

bool
fnr_sample_access_t::is_read(bool include_pf) const
{
    fnr_access_classes_t access_class = get_access_classes(access_type);
    return access_class & FNR_Access_Class_RD &&
	(include_pf || !(access_class & FNR_Access_Class_PF));
}

bool
fnr_sample_access_t::is_write(bool include_pf) const
{
    fnr_access_classes_t access_class = get_access_classes(access_type);
    return access_class & FNR_Access_Class_WR &&
	(include_pf || !(access_class & FNR_Access_Class_PF));
}

bool
fnr_sample_access_t::is_prefetch() const
{
    fnr_access_classes_t access_class = get_access_classes(access_type);
    return (access_class & FNR_Access_Class_PF) != 0;
}

bool
fnr_sample_access_t::is_proper() const
{
    fnr_access_classes_t access_class = get_access_classes(access_type);
    return !(access_class & FNR_Access_Classes_Pseudo);
}

ostream &
fnr_event_sample_t::stream_put(ostream &os) const
{
    os << "[sample] line_size: " << line_size_from_log2(line_size_log2) << endl
       << "  begin: [" << begin << "]" << endl
       << "  pc1+: [pc " << pad(pc_1_plus) << ", stack: " << pc_1_plus_stack << "]" << endl
       << "  flags: " << (int)flags
       << (flags & FNR_Sample_Flag_Killed ? " Killed " : "")
       << endl;

    BOOST_FOREACH(const fnr_sample_access_t &e, end)
	os << "  end: [" << e << "]" << endl;
    return os;
}

bool
fnr_event_sample_t::operator==(const fnr_event_t &rhs) const
{
    if (rhs.get_kind() != FNR_Event_Sample)
	return false;

    const fnr_event_sample_t &sample =
	static_cast<const fnr_event_sample_t &>(rhs);

    return
	begin == sample.begin &&
	end == sample.end &&
	pc_1_plus == sample.pc_1_plus &&
	pc_1_plus_stack == sample.pc_1_plus_stack &&
	line_size_log2 == sample.line_size_log2 &&
	flags == sample.flags;
}

ostream &operator<<(ostream &os, const fnr_event_t &event)
{
    return event.stream_put(os);
}

ostream &operator<<(ostream &os, const fnr_sample_access_t &access)
{
    os << "time "  << access.time
       << ", thread: " << access.thread
       << ", pc " << pad(access.pc)
       << ", stack "  << access.stack
       << ", addr " << pad(access.addr)
       << ", size " << access.access_size
       << ", type " << get_access_type_str(access.access_type);

    return os;
}

ostream &
fnr_event_burst_begin_t::stream_put(ostream &os) const
{
    os << "[burst-begin]    real_time: " << real_time
       << ", time: " << time;

    return os;
}

bool
fnr_event_burst_begin_t::operator==(const fnr_event_t &_rhs) const
{
    if (_rhs.get_kind() != FNR_Event_Burst_Begin)
	return false;

    const fnr_event_burst_begin_t &rhs =
	static_cast<const fnr_event_burst_begin_t &>(_rhs);

    return
	real_time == rhs.real_time &&
	time == rhs.time;
}

ostream &
fnr_event_burst_end_t::stream_put(ostream &os) const
{
    os << "[burst-end]      real_time: " << real_time
       << ", time: " << time;

    return os;
}

bool
fnr_event_burst_end_t::operator==(const fnr_event_t &_rhs) const
{
    if (_rhs.get_kind() != FNR_Event_Burst_End)
	return false;

    const fnr_event_burst_end_t &rhs =
	static_cast<const fnr_event_burst_end_t &>(_rhs);

    return
	real_time == rhs.real_time &&
	time == rhs.time;
}

ostream &
fnr_event_contexts_t::stream_put(ostream &os) const
{
    os << "[contexts]      contexts:";

    BOOST_FOREACH(const fnr_thread_id_t &t, contexts)
	os << " " << t;

    return os;
}

bool
fnr_event_contexts_t::operator==(const fnr_event_t &_rhs) const
{
    if (_rhs.get_kind() != FNR_Event_Contexts)
	return false;

    const fnr_event_contexts_t &rhs =
	static_cast<const fnr_event_contexts_t &>(_rhs);

    return rhs.contexts == contexts;
}

END_NAMESPACE;
END_NAMESPACE;
