/// @file library/fenris/flow.cc

// Copyright (c) 2008-2010 Acumem AB.  All rights reserved.

#include "flow.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

// ----------------------------------------------------------------------

flow_status_t
fenris_flow_by_type_t::process_event(const fnr_event_t &event)
{
    switch (event.get_kind()) {
    case FNR_Event_Read:
	return read(static_cast<const fnr_event_read_t &>(event));
    case FNR_Event_Write:
	return write(static_cast<const fnr_event_write_t &>(event));
    case FNR_Event_Prefetch:
	return prefetch(static_cast<const fnr_event_prefetch_t &>(event));
    case FNR_Event_Sample_Begin:
	return sample_begin(
	    static_cast<const fnr_event_sample_begin_t &>(event));
    case FNR_Event_Sample_End:
	return sample_end(
	    static_cast<const fnr_event_sample_end_t &>(event));
    case FNR_Event_Sample_Period:
	return sample_period(
	    static_cast<const fnr_event_sample_period_t &>(event));
    case FNR_Event_Acquisition_Begin:
	return acquisition_begin(
	    static_cast<const fnr_event_acquisition_begin_t &>(event));
    case FNR_Event_Acquisition_End:
	return acquisition_end(
	    static_cast<const fnr_event_acquisition_end_t &>(event));
    case FNR_Event_Mmap:
	return mmap(static_cast<const fnr_event_mmap_t &>(event));
    case FNR_Event_Munmap:
	return munmap(static_cast<const fnr_event_munmap_t &>(event));
    case FNR_Event_Stack:
	return stack(static_cast<const fnr_event_stack_t &>(event));
    case FNR_Event_Sample:
	return sample(static_cast<const fnr_event_sample_t &>(event));
    case FNR_Event_Time_Mapping:
        return time_mapping(static_cast<const fnr_event_time_mapping_t &>(event));
    case FNR_Event_Local_Time:
        return local_time(static_cast<const fnr_event_local_time_t &>(event));
    case FNR_Event_Burst_Begin:
	return burst_begin(static_cast<const fnr_event_burst_begin_t &>(event));
    case FNR_Event_Burst_End:
	return burst_end(static_cast<const fnr_event_burst_end_t &>(event));
    case FNR_Event_Contexts:
	return contexts(static_cast<const fnr_event_contexts_t &>(event));
    }

    return FLOW_OK;
}

// ----------------------------------------------------------------------

fenris_flow_filter_t::fenris_flow_filter_t(fenris_flow_t *next)
    : next(next)
{
}

fenris_flow_filter_t::~fenris_flow_filter_t()
{
}

flow_status_t
fenris_flow_filter_t::flow_init(fnr_features_t features)
{
    if (next)
	return next->flow_init(features);
    else
	throw flow_exception_t("Flow chain broken.");
}

flow_status_t
fenris_flow_filter_t::flow_to_next(const fnr_event_t &event)
{
    if (next)
	return next->process_event(event);
    else
	throw flow_exception_t("Flow chain broken.");
}


END_NAMESPACE;
END_NAMESPACE;
