/// @file library/fenris/flow_file_reader.hh

// Copyright (c) 2007-2009 Acumem AB.  All rights reserved.

#ifndef FLOW_FILE_READER_HH
#define FLOW_FILE_READER_HH

#include "codec_binary.hh"
#include "library/stream/stream_file.hh"
#include "library/stream/stream_container.hh"

#include <boost/filesystem.hpp>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

/** Fenris file reader. Reads a fenris file and dispatches the events to a
    fenris event flow. */
class flow_file_reader_t
{
public:
    /**
     * Initializes a flow file reader.
     *
     * \param filename input file or empty for stdin.
     * \param flow_next pointer to the next flow node.
     * \throw flow_exception_t On flow initialization error.
     * \throw stream_exception_t On stream initialization error.
     */
    flow_file_reader_t(
	const boost::filesystem::path &filename,
	fenris_flow_t *flow_next,
        warning_consumer_ptr_t warning_consumer);

    /**
     * Initializes a flow file reader.
     *
     * \param f input file stream. Will not be closed by this class
     * \param flow_next pointer to the next flow node.
     * \throw flow_exception_t On flow initialization error.
     * \throw stream_exception_t On stream initialization error.
     */
    flow_file_reader_t(
        FILE *f,
        fenris_flow_t *flow_next,
        warning_consumer_ptr_t warning_consumer);

    ~flow_file_reader_t();

    /**
     * Get a reference to the header in the file reader.
     * Version will be 0 for legacy files without a header.
     */
    const stream_container_header_t &get_header();

    /**
     * Starts streaming the file through the decoding process and
     * eventually the callback supplied by the user in the constructor.
     *
     * \return STREAM_ERROR on an error, or STREAM_ABORT or STREAM_OK on
     *         a successful read.
     */
    stream_status_t dispatch();

private:
    fnr_decoder_binary_t decoder;
    stream_container_reader_t container_stream;
    stream_file_reader_t file_stream;
};

END_NAMESPACE;
END_NAMESPACE;

#endif
