/// @file library/fenris/flow_file_writer.cc

// Copyright (c) 2006-2010 Acumem AB.  All rights reserved.

#include "flow_file_writer.hh"

#include "library/am_mem/am_mem.h"

#include <string.h>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

namespace fs = boost::filesystem;

flow_file_writer_t::flow_file_writer_t(
    const fs::path &filename,
    const stream_container_header_t &header,
    warning_consumer_ptr_t warning_consumer)
    : file_stream(filename),
      container_stream(header, &file_stream, warning_consumer),
      encoder(&container_stream, warning_consumer)
{
    /* The file writer holds a private chain of stream modules and an encoder:
     *   encoder_binary -> stream_container -> stream_file_writer
     */
}

flow_file_writer_t::flow_file_writer_t(
    FILE *file,
    const stream_container_header_t &header,
    warning_consumer_ptr_t warning_consumer)
    : file_stream(file),
      container_stream(header, &file_stream, warning_consumer),
      encoder(&container_stream, warning_consumer)
{
    /* The file writer holds a private chain of stream modules and an encoder:
     *   encoder_binary -> stream_container -> stream_file_writer
     */
}

flow_file_writer_t::~flow_file_writer_t()
{
}

flow_status_t
flow_file_writer_t::flow_init(fnr_features_t features)
{
    return encoder.flow_init(features);
}

flow_status_t
flow_file_writer_t::process_event(const fnr_event_t &event)
{
    return encoder.process_event(event);
}

END_NAMESPACE;
END_NAMESPACE;
