/// @file library/fenris/flow_file_writer.hh

// Copyright (c) 2007-2009 Acumem AB.  All rights reserved.

#ifndef FLOW_FILE_WRITER_HH
#define FLOW_FILE_WRITER_HH

#include "codec_binary.hh"
#include "library/stream/stream_file.hh"
#include "library/stream/stream_container.hh"

#include <boost/filesystem.hpp>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

/**
 * Fenris file writer. Processes events from a fenris event flow and writes
 * them to a fenris file.
 */
class flow_file_writer_t
    : public fenris_flow_t
{
public:
    /**
     * Initializes a flow file writer.
     *
     * \param filename input file or empty for stdout.
     * \param header container header to write to the file
     * \throw flow_exception_t On flow initialization error.
     * \throw stream_exception_t On stream initialization error.
     */
    flow_file_writer_t(
	const boost::filesystem::path &filename,
        const stream_container_header_t &header,
        warning_consumer_ptr_t warning_consumer);

    /**
     * Initializes a flow file writer.
     *
     * \param file a file handle managed outside this class.
     * \param header container header to write to the file
     * \throw flow_exception_t On flow initialization error.
     * \throw stream_exception_t On stream initialization error.
     */
    flow_file_writer_t(
        FILE *file,
        const stream_container_header_t &header,
        warning_consumer_ptr_t warning_consumer);

    ~flow_file_writer_t();

    flow_status_t flow_init(fnr_features_t features);
    flow_status_t process_event(const fnr_event_t &event);

private:
    stream_file_writer_t file_stream;
    stream_container_writer_t container_stream;
    fnr_encoder_binary_t encoder;
};

END_NAMESPACE;
END_NAMESPACE;


#endif
