/// @file library/license/license.cc

// Copyright (c) 2006-2010 Acumem AB.  All rights reserved.

#ifdef PRODUCT
#undef PRODUCT
#endif
#define PRODUCT NO_PRODUCT
#include "product.h"

#include "license.hh"
#include "library/version/version.h"
#include "library/exe_path/exe_path.hh"

#if 0
extern "C" {
#include <lmclient.h>
#include <lm_attr.h>
}
#else
struct lm_err_info {};
#endif

#include <iostream>
#include <boost/lexical_cast.hpp>
#include <boost/foreach.hpp>
#include <boost/algorithm/string.hpp>

BEGIN_NAMESPACE(acumem)
BEGIN_NAMESPACE(license)

using namespace std;
using namespace boost;
using namespace boost::filesystem;
using namespace acumem::library;

// ----------------------------------------------------------------------

#if 0
VENDORCODE code;
#endif

// ----------------------------------------------------------------------

template <char c0, char c1, char c2 = 0, char c3 = 0, char c4 = 0,
          char c5 = 0, char c6 = 0, char c7 = 0, char c8 = 0,
	  char c9 = 0, char c10 = 0>
class obfuscated_string_t {
private:
    char text[12];

public:
    operator char *()
	{
	    text[6] = c6;
	    text[0] = c0;
	    text[2] = c2;
	    text[1] = c1;
	    text[11] = '\0';
	    text[9] = c9;
	    text[3] = c3;
	    text[8] = c8;
	    text[4] = c4;
	    text[7] = c7;
	    text[5] = c5;
	    text[10] = c10;

	    return text;
	}
};

// ----------------------------------------------------------------------

/// FlexNet Features
static obfuscated_string_t<'t','s','u','s','e','r'> tsuser;
static obfuscated_string_t<'t','s','v','i','e','w'> tsview;
static obfuscated_string_t<'t','s','t','o','o','l'> tstool;

/// Allow software prefetch analysis.
static obfuscated_string_t<'s','p'> LIC_FEATURE_SW_PREFETCH;

/// Allow unlimited analysis callstack depth. Without it the call stack depth
/// is fixed at 0.
static obfuscated_string_t<'u','c'> LIC_FEATURE_UNLIMITED_CALLSTACK;

/// Enable expert blocking and fusion advice.
static obfuscated_string_t<'b','l'> LIC_FEATURE_BLOCKING_EXPERT;

/// Allow attaching to running process.
static obfuscated_string_t<'a','t'> LIC_FEATURE_ATTACH;

/// Allow sampling of unlimited numbers of threads. Without it only sampling of
/// 1 thread is allowed (up to 3 threads were allowed until the SS/TS source
/// code unification).
static obfuscated_string_t<'u','t'> LIC_FEATURE_UNLIMITED_THREADS;

/// Allow use of API to control sampling from the sampled process.
static obfuscated_string_t<'a','p'> LIC_FEATURE_API;

/// Allow burst sampling.
static obfuscated_string_t<'b','u'> LIC_FEATURE_BURST;

/// Analysis for MT statistics and issues. Used to also be required for MT
/// sampling, but after the SS/TS source code unification MT sampling is always
/// used so this not is not checked any more.
static obfuscated_string_t<'m','t'> LIC_FEATURE_MT;

/// Allow multi-socket analysis.
static obfuscated_string_t<'m','s'> LIC_FEATURE_MULTISOCKET;

/// Allow preparing static reports
static obfuscated_string_t<'s','r'> LIC_FEATURE_STATIC_REPORT;

/// Allow command line tools for sampling and reporting.
static obfuscated_string_t<'c','l'> LIC_FEATURE_CMD_LINE;

/// Allow thread binding analysis.
static obfuscated_string_t<'t','b'> LIC_FEATURE_THREAD_BINDING;

/// Introduce a required manifest hash
static obfuscated_string_t<'h','a','s','h'> LIC_FEATURE_MANIFEST_HASH;

/// Shorthand codes
static obfuscated_string_t<'s','s'> LIC_PRODUCT_SLOWSPOTTER;
static obfuscated_string_t<'s','p','o','t','l','i','t','e'> LIC_PRODUCT_SPOTLITE;
static obfuscated_string_t<'t','s'> LIC_PRODUCT_THREADSPOTTER;
static obfuscated_string_t<'f','r','e','j','a'> LIC_PRODUCT_FREJA_INTERNAL;


#define NUM_FEATURES 12

struct shorthand_t {
    char *product;
    char *options[NUM_FEATURES + 1];
};

shorthand_t shorthand[] =
{
    {LIC_PRODUCT_SLOWSPOTTER,
     {LIC_FEATURE_SW_PREFETCH,
      LIC_FEATURE_UNLIMITED_CALLSTACK,
      LIC_FEATURE_ATTACH,
      LIC_FEATURE_API,
      LIC_FEATURE_BURST,
      LIC_FEATURE_CMD_LINE,
      NULL}},

    {LIC_PRODUCT_THREADSPOTTER,
     {LIC_FEATURE_SW_PREFETCH,
      LIC_FEATURE_UNLIMITED_CALLSTACK,
      LIC_FEATURE_ATTACH,
      LIC_FEATURE_UNLIMITED_THREADS,
      LIC_FEATURE_API,
      LIC_FEATURE_BURST,
      LIC_FEATURE_MT,
      LIC_FEATURE_MULTISOCKET,
      LIC_FEATURE_CMD_LINE,
      NULL}},

    {LIC_PRODUCT_SPOTLITE, // This is the new "spotlite",
			   // threadspotter without cmdline and attach
     {LIC_FEATURE_SW_PREFETCH,
      LIC_FEATURE_UNLIMITED_CALLSTACK,
      LIC_FEATURE_UNLIMITED_THREADS,
      LIC_FEATURE_API,
      LIC_FEATURE_BURST,
      LIC_FEATURE_MT,
      LIC_FEATURE_MULTISOCKET,
      NULL}},

    {LIC_PRODUCT_FREJA_INTERNAL,
     {LIC_FEATURE_SW_PREFETCH,
      LIC_FEATURE_UNLIMITED_CALLSTACK,
      LIC_FEATURE_BLOCKING_EXPERT,
      LIC_FEATURE_ATTACH,
      LIC_FEATURE_UNLIMITED_THREADS,
      LIC_FEATURE_API,
      LIC_FEATURE_BURST,
      LIC_FEATURE_MT,
      LIC_FEATURE_MULTISOCKET,
      LIC_FEATURE_STATIC_REPORT,
      LIC_FEATURE_CMD_LINE,
      LIC_FEATURE_THREAD_BINDING,
      NULL}},
};

// ----------------------------------------------------------------------

error_info_t::error_info_t(const char *msg, const struct lm_err_info *err_info)
    : msg(msg)
{
#if 0
    maj_errno = err_info->maj_errno;
    min_errno = err_info->min_errno;
    sys_errno = err_info->sys_errno;
	if (err_info->feature)
		feature = err_info->feature;

    LM_CHAR_PTR_PTR f = err_info->lic_files;
    while (f && *f != 0)
	lic_files.push_back(string(*f++));

	if (err_info->context)
		context = err_info->context;
#endif
}

error_info_t::error_info_t(int maj_errno, const string &context)
    : maj_errno(maj_errno), context(context)
{
    min_errno = 0;
    sys_errno = 0;
    feature = "";
    msg = "Error: " + boost::lexical_cast<string>(maj_errno) + " " + context;
}

// ----------------------------------------------------------------------

// This may throw license_exception and boost::system::system_error
flexnet_wrapper_t::flexnet_wrapper_t()
{
#if 0
    using namespace acumem::exe_path;

    lm_handle *h = 0;
    int status;

    reconnecting = false;

    status = lc_new_job(h, lc_new_job_arg2, &code, &h);
    if (status != 0)
	throw license_exception_t(status, "new job");

    job_ptr_t j(h, lc_free_job);

#define CHECK_STATUS(status) do { if (status) throw license_exception_t(lc_errstring(h), lc_err_info(h));} while(0)

#ifdef WIN32
    string delim(";");
#else
    string delim(":");
#endif

    // Default license location:
    // $(HOME)/.threadspotter/license : installdir/license : @localhost
    // %APPDATA%/ThreadSpotter/license : installdir/license : @localhost
    string default_license_path;

    path user_dir = get_user_license_file_directory();
    if (!user_dir.empty())
	default_license_path += user_dir.string() + delim;

	try {
		path app_dir = get_app_license_file_directory();
		if (!app_dir.empty())
			default_license_path += app_dir.string() + delim;
	} catch (const boost::system::system_error &e) {
		// if registry path not found/installed, try running without it
		// ENOENT - 2 
	    if (e.code() != boost::system::error_code(boost::system::errc::no_such_file_or_directory, boost::system::system_category()))
			throw;
	}
    default_license_path += "@localhost";

    status = lc_set_attr(h, LM_A_LICENSE_DEFAULT,
			 (LM_A_VAL_TYPE) default_license_path.c_str());
    CHECK_STATUS(status);

    // Provide long error messages, 1 is default
    status = lc_set_attr(h, LM_A_LONG_ERRMSG, (LM_A_VAL_TYPE) 1);
    CHECK_STATUS(status);

    // Disable the error dialog
    status = lc_set_attr(h, LM_A_PERROR_MSGBOX, (LM_A_VAL_TYPE) 0);
    CHECK_STATUS(status);

    // Disable the dialog prompting for a license file
    status = lc_set_attr(h, LM_A_PROMPT_FOR_FILE, (LM_A_VAL_TYPE) 0);
    CHECK_STATUS(status);

    // Automatic heartbeat interval, 30 seconds
    status = lc_set_attr(h, LM_A_CHECK_INTERVAL, (LM_A_VAL_TYPE) 30);
    CHECK_STATUS(status);

    // Set retry to something low
    status = lc_set_attr(h, LM_A_RETRY_INTERVAL, (LM_A_VAL_TYPE) 30);
    CHECK_STATUS(status);
    status = lc_set_attr(h, LM_A_RETRY_COUNT, (LM_A_VAL_TYPE) -1);
    CHECK_STATUS(status);

    // 0 - don't update the registry VENDOR_LICENSE_FILE setting
    status = lc_set_attr(h, LM_A_CKOUT_INSTALL_LIC, (LM_A_VAL_TYPE) 0);
    CHECK_STATUS(status);

    // Callback context data, a pointer to 'this'
    status = lc_set_attr(h, LM_A_VENDOR_CALLBACK_DATA, (LM_A_VAL_TYPE) this);
    CHECK_STATUS(status);

    // Callbacks to notify about loss of contact with the license server
    status = lc_set_attr(h, LM_A_USER_RECONNECT_EX,
			 (LM_A_VAL_TYPE) flexnet_wrapper_t::private_reconnect_call);
    status = lc_set_attr(h, LM_A_USER_RECONNECT_DONE_EX,
			 (LM_A_VAL_TYPE) flexnet_wrapper_t::private_reconnect_done_call);
    CHECK_STATUS(status);
#undef CHECK_STATUS
    job = j;
#endif
}

token_ptr_t
flexnet_wrapper_t::dry_run_checkout(char *feature, char *version)
{
#if 0
    int result = lc_checkout(job.get(),
			     feature,
			     version,
			     1,
			     LM_CO_NOWAIT | LM_CO_LOCALTEST,
			     &code,
			     LM_DUP_NONE);
    switch (result) {
    case 0:
	return token_ptr_t(new token_t(job, feature, true /* dry_run */));

    case LM_MAXUSERS:
    case LM_USERSQUEUED:	
    default:
      {
	string err(lc_errstring(job.get()));
	throw license_exception_t(err.c_str(), lc_err_info(job.get()));
      }
    }

    return token_ptr_t();
#else
	return token_ptr_t(new token_t(job, feature, true /* dry_run */));
#endif
}

token_ptr_t
flexnet_wrapper_t::checkout(char *feature, char *version, int num)
{

#if 0
     // The description of the flag is a little bit unclear in the
     // documentation. LM_CO_WAIT will block until a license is
     // available. LM_CO_NOWAIT will get a license if there is one,
     // but return an error code otherwise. Then there is LM_CO_QUEUE
     // that we can use to interleave waiting with some other
     // checking.

    int result = lc_checkout(job.get(),
			     feature,
			     version,
			     num,
			     LM_CO_NOWAIT,
			     &code,
			     LM_DUP_NONE);
    switch (result) {
    case 0:
	return token_ptr_t(new token_t(job, feature, false /* no dry_run */));

    case LM_MAXUSERS:
    case LM_USERSQUEUED:
    default:
	string err(lc_errstring(job.get()));
	throw license_exception_t(err.c_str(), lc_err_info(job.get()));
    }

    return token_ptr_t();
#else
    return token_ptr_t(new token_t(job, feature, false /* no dry_run */));
#endif
}

bool
flexnet_wrapper_t::is_reconnecting() const
{
    return reconnecting;
}

void
flexnet_wrapper_t::reconnect_call(const char *feature,
				  int pass,
				  int total_attempts,
				  int interval)
{
    reconnecting = true;
}

void
flexnet_wrapper_t::reconnect_done_call(const char *feature, int tries,
				       int total_attempts, int interval)
{
    reconnecting = false;
}

void
flexnet_wrapper_t::private_reconnect_call(lm_handle *lmjob,
					  char *feature,
					  int pass,
					  int total_attempts,
					  int interval,
					  void *context)
{
    flexnet_wrapper_t *self = (flexnet_wrapper_t *) context;
    self->reconnect_call(feature, pass, total_attempts, interval);
}

void
flexnet_wrapper_t::private_reconnect_done_call(lm_handle *lmjob,
					       char *feature,
					       int tries,
					       int total_attempts,
					       int interval,
					       void *context)
{
    flexnet_wrapper_t *self = (flexnet_wrapper_t *) context;
    self->reconnect_done_call(feature, tries, total_attempts, interval);
}

// ----------------------------------------------------------------------

token_t::token_t(job_ptr_t job, const char *feature, bool dry_run)
    : dry_run(dry_run),
      released(false),
      feature(feature),
      job(job)
{
#if 0
    CONFIG *conf;

    if (dry_run) {
	conf = lc_test_conf(job.get());
    } else {
	LM_CHAR_PTR f = const_cast<LM_CHAR_PTR>(feature);
	conf = lc_auth_data(job.get(), f);
    }

    if (conf && conf->lc_vendor_def != 0) {
	vendor_string = conf->lc_vendor_def;
	date = conf->date;
    }
#else
    vendor_string = "ParaTools, Inc.";
    date = "00:00:00 00/00/00";
#endif
}

token_t::~token_t()
{
    release();
}

void
token_t::release()
{
#if 0
    // XXX Make threadsafe.
    if (!dry_run && !released) {
        LM_CHAR_PTR f = const_cast<LM_CHAR_PTR>(feature.c_str());
	lc_checkin(job.get(), f, true /* keep */);
	released = true;
    }
#else
    released = true;
#endif
}

string
token_t::get_vendor_string() const
{
    return vendor_string;
}

string
token_t::get_date() const
{
    return date;
}

// ----------------------------------------------------------------------

static void
add_option(set<string> &option_set, const string &str)
{
    option_set.insert(str);
}

static void
process_option(set<string> &remove, set<string> &add, string option)
{
    if (option.size() == 0)
	return;

    bool minus = false;
    if (option[0] == '-') {
	if (option.size() == 1)
	    return;
	minus = true;
	option = option.substr(1, option.size()-1);
    }

    // if this option is a shorthand, process all corresponding tokens
    int size = sizeof(shorthand)/sizeof(shorthand[0]);
    for (int index = 0; index < size; index++)
    {
	if (option == shorthand[index].product) {
	    char **opt = &shorthand[index].options[0];
	    while (*opt) {
		add_option(minus ? remove : add, string(*opt));
		opt++;
	    }
	    return;
	}
    }

    add_option(minus ? remove : add, option);
}

void
feature_set_t::read_vendor_string(token_ptr_t token)
{
    if (!token)
	return;

    string vendor_string = token->get_vendor_string();
    string::const_iterator i = vendor_string.begin(), e = vendor_string.end();
    string::const_iterator last = i;

    features.clear();
    set<string> remove;

    while (i != e) {
	if (isspace(*i) || *i == ',') {
	    if (last != i) {
		process_option(remove, features, string(last, i));
		last = i+1;
	    }
	}
	i++;
    }

    if (last != i) {
	process_option(remove, features, string(last, i));
    }

    BOOST_FOREACH(string r, remove)
	features.erase(r);
}

bool
feature_set_t::sw_prefetch() const
{
    return has_feature(LIC_FEATURE_SW_PREFETCH);
}

bool
feature_set_t::unlimited_callstack() const
{
    return has_feature(LIC_FEATURE_UNLIMITED_CALLSTACK);
}

bool feature_set_t::blocking_expert() const
{
    return has_feature(LIC_FEATURE_BLOCKING_EXPERT);
}

bool
feature_set_t::attach() const
{
    return has_feature(LIC_FEATURE_ATTACH);
}

bool
feature_set_t::unlimited_threads() const
{
    return has_feature(LIC_FEATURE_UNLIMITED_THREADS);
}

bool
feature_set_t::api() const
{
    return has_feature(LIC_FEATURE_API);
}

bool
feature_set_t::burst() const
{
    return has_feature(LIC_FEATURE_BURST);
}

bool
feature_set_t::mt() const
{
    return has_feature(LIC_FEATURE_MT);
}

bool
feature_set_t::multisocket() const
{
    return has_feature(LIC_FEATURE_MULTISOCKET);
}

bool
feature_set_t::static_report() const
{
    return has_feature(LIC_FEATURE_STATIC_REPORT);
}

bool
feature_set_t::cmd_line() const
{
    return has_feature(LIC_FEATURE_CMD_LINE);
}

bool
feature_set_t::thread_binding() const
{
    return has_feature(LIC_FEATURE_THREAD_BINDING);
}

bool
feature_set_t::require_manifest_check() const
{
    /* SSS : 
    return has_feature(LIC_FEATURE_MANIFEST_HASH);
    */
    return false;
}

bool
feature_set_t::check_manifest_hash(const std::vector<unsigned char> &hash) const
{
    string test(LIC_FEATURE_MANIFEST_HASH);
    test += "=";
    static const char *hex="0123456789abcdef";
    BOOST_FOREACH(unsigned char ch, hash) {
	test += hex[ch >> 4];
	test += hex[ch & 0xf];
    }

    return has_feature(test.c_str());
}

bool
feature_set_t::has_feature(const char *subfeature) const
{
    /* SSS: */
    return true;
    /* SSS */

    if (features.find(string(subfeature)) != features.end()) {
	return true;
    } else {

	string param_option(subfeature);
	param_option += "=";

	set<string>::const_iterator i = features.lower_bound(param_option);
	if (i != features.end() &&
	    i->substr(0, param_option.size()) == param_option) {
	    return true;
	}

	return false;
    }
}

// ----------------------------------------------------------------------

license_manager_t::license_manager_t(const warning_consumer_ptr_t &warning)
    : warning(warning)
{
}

feature_set_ptr_t
license_manager_t::dry_run()
{
    char version[10];
    get_compile_time(version);

    // If there already is a proper license, then the dry run is unnecessary
    token_ptr_t user_token = tokens[user_kind].lock();
    token_ptr_t tool_token = tokens[tool_kind].lock();
    if (user_token && tool_token)
	return feature_set_ptr_t(new feature_set_t(shared_from_this(),
						   user_token, tool_token));

    // Else, if there is an old dry_run kind already checked out, reuse it
    tool_token = tokens[dry_run_kind].lock();
    if (tool_token)
	return feature_set_ptr_t(new feature_set_t(shared_from_this(),
						   token_ptr_t(), tool_token));

    // throws if no license can be found
    tool_token = flexnet_wrapper.dry_run_checkout(tstool, version);
    
    tokens[dry_run_kind] = tool_token;
    
    if (remaining_days() < LIC_WARN_PERIOD)
	warning->info("License expires: " + tool_token->get_date());
    
    return feature_set_ptr_t(new feature_set_t(shared_from_this(),
					       token_ptr_t(), tool_token));
}

feature_set_ptr_t
license_manager_t::acquire(license_kind_t app_kind)
{
    char version[10];
    get_compile_time(version);

    // If we had a dry_run token, forget about it
    tokens.erase(dry_run_kind);

    // If we already have the requested token, and it is still valid,
    // all is fine, return its set.
    token_ptr_t app_token = tokens[app_kind].lock();
    token_ptr_t user_token = tokens[user_kind].lock();
    if (app_token && user_token)
	return feature_set_ptr_t(new feature_set_t(shared_from_this(),
						   user_token, app_token));

    feature_set_ptr_t feature_set;


    // Otherwise, first get a user token if necessary and then the kind we want

    try {
	if (!user_token)
	{
	    // might throw a license_exception_t
	    user_token = flexnet_wrapper.checkout(tsuser, version, 1);
	    tokens[user_kind] = user_token;
	}
	
	char *feature;
	switch (app_kind) {
	case view_kind:
	    feature = tsview;
	    break;
	case tool_kind:
	    feature = tstool;
	    break;
	default:
	    throw std::runtime_error("Bad feature requested");
	}

	// Might throw, an that will be caught/re-thrown below
	app_token = flexnet_wrapper.checkout(feature, version, 1);

	tokens[app_kind] = app_token;

	feature_set.reset(new feature_set_t(shared_from_this(),
					    user_token, app_token));

    } catch (const std::runtime_error &) {
	release(user_kind);
	release(app_kind);
	throw;
    }

    if (remaining_days() < LIC_WARN_PERIOD)
	warning->info("License expires: " + user_token->get_date());

    return feature_set;
}

void
license_manager_t::release(license_kind_t license_kind)
{
    // First see if we had any, and explicitly release it in such case
    token_ptr_t app_token = tokens[license_kind].lock();
    if (app_token) app_token->release();

    if (license_kind != dry_run_kind) {
	token_ptr_t tool_token = tokens[tool_kind].lock();
	token_ptr_t view_token = tokens[view_kind].lock();

	if (license_kind == user_kind) {
	    // If we released a user token, we must release all other tokens
	    if (tool_token) tool_token->release();
	    if (view_token) view_token->release();

	    tokens.erase(tool_kind);
	    tokens.erase(view_kind);
	} else {
	    // license_kind is either tool_kind || view_kind
	    if (!tool_token && !view_token) {
		// remove the user token whenever we remove the last
		// of tool/view
		token_ptr_t user_token = tokens[user_kind].lock();
		if (user_token) user_token->release();
		tokens.erase(user_kind);
	    }
	}
    }

    // finally remove the weak pointer
    tokens.erase(license_kind);
}

bool
license_manager_t::check_license(int timeout_seconds)
{
    /// XXX How to use the timeout parameter?
    token_ptr_t user_token = tokens[user_kind].lock();

    return user_token && !flexnet_wrapper.is_reconnecting();
}

static bool
convert_date_string(const std::string &str, struct tm *tm)
{
    // dd-mon-yyyy, 1-jan-0 is reserved for the "permanent" license

    static const char *months[] = {"jan", "feb", "mar", "apr", "may", "jun",
				   "jul", "aug", "sep", "oct", "nov", "dec", 0};

    size_t dash = str.find('-');
    if (dash == string::npos)
	return false;

    size_t dash2 = str.find('-', dash+1);
    if (dash2 == string::npos)
	return false;

    tm->tm_mday = boost::lexical_cast<int>(str.substr(0, dash));

    string monthstr = str.substr(dash+1, dash2-dash-1);
    boost::to_lower(monthstr);
    const char **m = &months[0];

    while(true) {
	if (*m == 0)
	    return false;
	if (*m == monthstr)
	    break;
	m++;
    }

    tm->tm_mon = (int)(m - months);

    tm->tm_year = boost::lexical_cast<int>(str.substr(dash2+1))-1900;

    return true;
}

static bool
is_permanent(const std::string &date)
{
    // If it ends with [regexp] -0+$ or is "permanent", then we report
    // "permanent"

    if (date == "permanent")
	return true;

    std::string::const_reverse_iterator x = date.rbegin();
    bool seen0 = false;
    while (x != date.rend()) {
	if (*x == '-')
	    return seen0;
	if (*x == '0')
	    seen0 = true;
	else
	    return false;
	++x;
    }
    return false;
}

int
license_manager_t::remaining_days() const
{
    /* SSS */
	return 9999;
    /* SSS */
    token_ptr_t t = any_token();

    if (t.get() == 0)
	return -1;

    string d = t->get_date();
    if (is_permanent(d))
	return 9999;

    time_t last_day_time;
    time(&last_day_time);
    struct tm last_day = *localtime(&last_day_time);
    if (false == convert_date_string(d, &last_day))
	return -1;

    last_day.tm_hour = 23;
    last_day.tm_min = 59;
    last_day.tm_sec = 59;

    last_day_time = mktime(&last_day);
	if (last_day_time == (time_t) -1)
		return -1; // Invalid parameters, likely before 1970

    time_t now = time(0);

    double diff = difftime(last_day_time, now);

    return (int)(floor(diff/(24.0*60.0*60.0)));
}

token_ptr_t
license_manager_t::any_token() const
{
    token_ptr_t t;
    token_map_t::const_iterator i;

    i = tokens.find(user_kind);
    if (i != tokens.end())
	t = i->second.lock();

    if (t)
	return t;

    i = tokens.find(dry_run_kind);
    if (i != tokens.end())
	t = i->second.lock();

    return t;
}

string
license_manager_t::expiration_date() const
{
    token_ptr_t t = any_token();
    if (t.get() == 0)
	return "No license";

    string date = t->get_date();
    if (is_permanent(date))
	return "permanent";
    else
	return date;
}


void
license_manager_t::get_compile_time(char version[10])
{
    time_t vct = (time_t) version_compile_time;
    struct tm *compile_tm = gmtime(&vct);
    sprintf(version, "%4d.%02d%02d",
	    compile_tm->tm_year + 1900,
	    compile_tm->tm_mon + 1,
	    compile_tm->tm_mday);
}

END_NAMESPACE;
END_NAMESPACE;
