#include "library/manifest/manifest.hh"

#include <boost/filesystem.hpp>

#include <sstream>

#include <stdio.h>
#include <string.h>
#include <errno.h>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

namespace fs = boost::filesystem;

sha1_t
verify_manifest(const string &filename)
{
    try {
        string file_digest = sha1_as_string(calculate_sha1(filename));

        fs::path path(filename);
        fs::path manifest_path;
        while (path != path.root_path()) {
            path = path.parent_path();
            manifest_path = path / "manifest.rwm";
            if (exists(manifest_path))
                goto manifest_file_found;
        }

        throw manifest_exception_t(
            "No manifest file found for " + filename + ".");

    manifest_file_found:
        string manifest_filename = manifest_path.string();

        FILE *file;
        if ((file = fopen(manifest_filename.c_str(), "rb")) == NULL) {
            stringstream ss;
            ss << "Failed to open manifest file " << manifest_filename
               << ": Error " << errno << ", " << strerror(errno) << ".";
            throw manifest_exception_t(ss.str());
        }

        vector<string> digests;
        while (!feof(file)) {
            string digest;
            int c;
            while ((c = fgetc(file)) != EOF)
                if (c == '\n')
                    break;
                else
                    if (digest.size() < file_digest.size())
                        digest +=  (char)c;
            if (c == EOF && ferror(file))
                throw manifest_exception_t(
                    "Failed to read manifest file " + manifest_filename + ".");
            if (!digest.empty())
                digests.push_back(digest);
        }

        if (fclose(file) != 0) {
            stringstream ss;
            ss << "Failed to close manifest file " << manifest_filename
               << ": Error " << errno << ", " << strerror(errno) << ".";
            throw manifest_exception_t(ss.str());
        }

        for (vector<string>::const_iterator
                 ii = digests.begin(), ei = digests.end(); ii != ei; ii++)
            if (*ii == file_digest)
                return calculate_sha1(manifest_path.string());

	stringstream ss;
	ss << "Manifest file " << manifest_filename
	   << " does not authorize " << filename << " to be sampled.";
        throw manifest_exception_t(ss.str());
    }
    catch (sha1_exception_t &e) {
        throw manifest_exception_t(
            string("Failed to verify manifest: ") + e.what());
    }
    catch (fs::filesystem_error &e) {
        throw manifest_exception_t(
            "Error while searching for manifest file for " + filename + ": " +
            e.what() + ".");
    }
}

END_NAMESPACE;
END_NAMESPACE;
