/** @file library/random_lcg/random_lcg.c */

/* Copyright (c) 2007 Acumem AB.  All rights reserved. */

/* Linear congruential psuedo-random number generator. We use the hard-coded
   parameters A = 1664525, B = 1013904223 and M = 2^32. Since M = 2^32 and we
   do the calculations with unsigned 32-bit integers we get the modulo for
   free.

   See http://en.wikipedia.org/wiki/Linear_congruential_generator */

#include "library/am_mem/am_mem.h"

#include "random_lcg.h"

struct random_lcg {
    uint32_t v;
};

random_lcg_t *
random_lcg_new(uint32_t seed)
{
    random_lcg_t *lcg = am_malloc(sizeof(random_lcg_t));
    lcg->v = seed;
    return lcg;
}

void
random_lcg_delete(random_lcg_t *lcg)
{
    am_free(lcg);
}

uint32_t
random_lcg_next(random_lcg_t *lcg)
{
    lcg->v = (lcg->v * 1664525) + 1013904223;
    return lcg->v;
}
