/** @file library/random_lcg/random_lcg.h */

/* Copyright (c) 2007-2009 Acumem AB.  All rights reserved. */

/* Linear congruential generator, for repeatable psuedo-random numbers. */

#ifndef RANDOM_LCG_H
#define RANDOM_LCG_H

#include "library/lang/common_macros.h"
#include "library/lang/common_types.h"

BEGIN_EXTERN_C;

/** Maximum pseudo-random number returned by the linear congruential
    generator. */
#define RANDOM_LCG_MAX ((uint32_t)-1)

/** Linear congruential generator. */
typedef struct random_lcg random_lcg_t;

/** Create a linear congruential generator */
random_lcg_t *random_lcg_new(uint32_t seed);

/** Destroy the linear congruential generator. */
void random_lcg_delete(random_lcg_t *lcg);

/** Return the next pseudo-random number from the linear congruential
    generator. */
uint32_t random_lcg_next(random_lcg_t *lcg);

END_EXTERN_C;

#endif /* RANDOM_LCG_H */
