#include "library/sha1/sha1.hh"
#include "library/lang/am_assert.hh"

#include <openssl/sha.h>

#include <sstream>

#include <stdio.h>
#include <string.h>
#include <errno.h>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

#define CHUNK_SIZE 0x10000

sha1_t
calculate_sha1(const string &filename)
{
    sha1_t sha1;

    FILE *file;
    if ((file = fopen(filename.c_str(), "rb")) == NULL) {
        stringstream ss;
        ss << "Failed to open file " << filename << ": Error " << errno << ", "
           << strerror(errno) << ".";
        throw sha1_exception_t(ss.str());
    }

    SHA_CTX ctx;

    if (SHA1_Init(&ctx) != 1)
        throw sha1_exception_t("SHA1_Init() failed.");

    while (!feof(file)) {
        unsigned char buf[CHUNK_SIZE];
        size_t buf_read = fread(&buf, 1, sizeof(buf), file);
        if (SHA1_Update(&ctx, buf, buf_read) != 1)
            throw sha1_exception_t("SHA1_Update() failed.");
        if (buf_read < sizeof(buf) && ferror(file))
            throw sha1_exception_t("Failed to read file " + filename + ".");
    }

    AM_ASSERT(SHA_DIGEST_LENGTH == sizeof(sha1.bytes));
    if (SHA1_Final(sha1.bytes, &ctx) != 1)
        throw sha1_exception_t("SHA1_Final() failed.");

    if (fclose(file) != 0) {
        stringstream ss;
        ss << "Failed to close file " << filename << ": Error " << errno << ", "
           << strerror(errno) << ".";
        throw sha1_exception_t(ss.str());
    }

    return sha1;
}

string
sha1_as_string(const sha1_t &sha1)
{
    static const char *hex = "0123456789abcdef";

    string hexstring;
    for (size_t i = 0; i < sizeof(sha1.bytes); i++) {
	hexstring += hex[sha1.bytes[i] >> 4];
	hexstring += hex[sha1.bytes[i] & 0xf];
    }
    return hexstring;
}

END_NAMESPACE;
END_NAMESPACE;
