/// @file library/stream/stream_file.hh

// Copyright (c) 2008-2009 Acumem AB.  All rights reserved.

#ifndef FNR_STREAM_FILE_HH
#define FNR_STREAM_FILE_HH

#include "library/stream/stream.hh"
#include "library/lang/common_macros.h"

#include <boost/filesystem.hpp>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

/* This is an adaptation of file sinks and file sources as modules in the
 * stream concept.
 */

class stream_file_reader_t
{
public:
    /**
     * Creates a stream source by opening a file for reading. The created
     * node must be freed by calling stream_file_reader_delete.
     *
     * \param filename file to open, empty to open stdin.
     * \param stream_next next node in the stream
     */
    stream_file_reader_t(const boost::filesystem::path &filename,
			 stream_t *stream_next);


    /**
     * Creates a stream source on an open FILE *. The file must
     * previously have been opened for reading. It will not be freed
     * by this class. The created node must be freed by calling
     * stream_file_reader_delete.
     *
     * \param file file handle
     * \param stream_next next node in the stream
     */
    stream_file_reader_t(FILE *file, stream_t *stream_next);
    ~stream_file_reader_t();

    /**
     * Starts reading the file and pushes all data through the stream.
     *
     * \return a stream_status_t value.
     */
    stream_status_t dispatch();

private:
    bool shall_close;
    FILE *file;
    stream_t *stream_next;
};

class stream_file_writer_t
    : public stream_t
{
public:
    /**
     * Creates a new file writer.
     *
     * \param filename file to open, NULL for stdout.
     */
    stream_file_writer_t(const boost::filesystem::path &filename);

    /**
     * Creates a new file writer.
     *
     * \param f file to open, must be previously opened for writing -
     * will not be closed by this class.
     */
    stream_file_writer_t(FILE * f);
    ~stream_file_writer_t();

    stream_status_t process_data(const char *buffer, size_t len);
    stream_status_t flush();

private:
    bool shall_close;
    FILE *file;
};

END_NAMESPACE;
END_NAMESPACE;

#endif
