/// @file library/sym/debuglink.hh

// Copyright (c) 2009-2010 Acumem AB.  All rights reserved.

#ifndef DEBUGLINK_HH
#define DEBUGLINK_HH

#include "sym.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

using namespace std;

/// A debug file reference
struct debuglink_t {
    debuglink_t(string filename, bool valid)
	: filename(filename), valid(valid) {}
    /// Path to the debug information file
    string filename;
    /// True if the file was found and has correct checksum.
    bool valid;
};

typedef vector<debuglink_t> debuglink_vector_t;

/**
 * Class to look for debug file references in an elf file.
 */
class debuglink_finder_t {
public:
    template<class StringCont>
    debuglink_finder_t(const StringCont &debug_dirs)
	: debug_dirs(debug_dirs) {}

    debuglink_finder_t() {};

    /// Add a directory to search for debug symols in.
    void add_debug_dir(const string &dir);

    /// Look for debug links in an ELF file.
    debuglink_vector_t find_links(const string &filename,
				  elf_ptr_t part);

private:
    vector<string> debug_dirs;
};

END_NAMESPACE;
END_NAMESPACE;

#endif
