/// @file library/sym/function_cache.cc

// Copyright (c) 2010 Acumem AB.  All rights reserved.

#include "function_cache.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

function_ptr_t
function_cache_t::add_function(sym_paddr_t base,
                               sym_paddr_t end,
                               const string &name)
{
    string_cptr_t name_ptr = string_cache_lookup(sym.cppfilt.translate(name));

    function_ptr_t function(
        new function_t(sym_paddr_range_t(base, end), name_ptr));

    function_map.insert(make_pair(base, function));

    return function;
}

bool
function_cache_t::lookup_function(sym_paddr_t addr,
                                  function_ptr_t &ret) const
{
    function_map_t::const_iterator map_iter = function_map.upper_bound(addr);

    if (map_iter == function_map.begin())
        return false;

    function_ptr_t function = (--map_iter)->second;

    if (!function->range.contains(addr))
        return false;

    ret = function;
    return true;
}

END_NAMESPACE;
END_NAMESPACE;
