/// @file library/sym/function_cache.hh

// Copyright (c) 2010 Acumem AB.  All rights reserved.

#ifndef FUNCTION_CACHE_HH
#define FUNCTION_CACHE_HH

#include "sym.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

struct function_t {
    function_t(const sym_paddr_range_t &range,
               const string_cptr_t &name)
	: range(range),
	  name(name)
	{}

    const sym_paddr_range_t range;
    const string_cptr_t name;
};

typedef shared_ptr<function_t> function_ptr_t;

typedef map<sym_paddr_t, function_ptr_t> function_map_t;

class function_cache_t {
public:
    function_cache_t(sym_t &sym)
        : sym(sym)
        {}

    /// If base == end the upper bound of the function is treated as unknown
    /// and the function will stretch to the base of the next function.
    function_ptr_t add_function(sym_paddr_t base,
                                sym_paddr_t end,
                                const string &name);

    bool lookup_function(sym_paddr_t pc, function_ptr_t &ret) const;

private:
    sym_t &sym;
    function_map_t function_map;
};

END_NAMESPACE;
END_NAMESPACE;

#endif /* FUNCTION_CACHE_HH */
