/// @file library/sym/libdwarf_object.hh

// Copyright (c) 2008 Acumem AB.  All rights reserved.

#ifndef LIBDWARF_OBJECT_HH
#define LIBDWARF_OBJECT_HH

#include "library/lang/common_macros.h"

#include <libdwarf/libdwarf.h>
#include <boost/shared_ptr.hpp>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

class libdwarf_object_t {
public:
    virtual ~libdwarf_object_t() {}
    virtual int get_section_info(Dwarf_Half section_index,
				 Dwarf_Obj_Access_Section* return_section,
				 int* error) =0;
    virtual Dwarf_Endianness get_byte_order() =0;
    virtual Dwarf_Small get_length_size() =0;
    virtual Dwarf_Small get_pointer_size() =0;
    virtual Dwarf_Unsigned get_section_count() =0;
    virtual int load_section(Dwarf_Half section_index,
	Dwarf_Small** return_data, int* error) =0;

    bool init(Dwarf_Obj_Access_Interface_s *s);

private:

    virtual bool specific_init() { return true; }

    static Dwarf_Obj_Access_Methods_s methods;
    static int get_section_info_s(void* obj, Dwarf_Half section_index,
				  Dwarf_Obj_Access_Section* return_section,
				  int* error) {
	return ((libdwarf_object_t *)obj)->get_section_info(section_index, return_section, error);
    }
    static Dwarf_Endianness get_byte_order_s(void* obj) {
	return ((libdwarf_object_t *)obj)->get_byte_order();
    }
    static Dwarf_Small   get_length_size_s(void* obj) {
	return ((libdwarf_object_t *)obj)->get_length_size();
    }
    static Dwarf_Small   get_pointer_size_s(void* obj) {
	return ((libdwarf_object_t *)obj)->get_pointer_size();
    }
    static Dwarf_Unsigned  get_section_count_s(void* obj) {
	return ((libdwarf_object_t *)obj)->get_section_count();
    }
    static int load_section_s(void* obj, Dwarf_Half section_index,
			      Dwarf_Small** return_data, int* error) {
	return ((libdwarf_object_t *)obj)->load_section(section_index, return_data, error);
    }
};

typedef boost::shared_ptr<libdwarf_object_t> libdwarf_object_ptr_t;

END_NAMESPACE;
END_NAMESPACE;

#endif
