/// @file library/sym/statement_cache.cc

// Copyright (c) 2010 Acumem AB.  All rights reserved.

#include "statement_cache.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

statement_ptr_t
statement_cache_t::add_statement(sym_paddr_t base,
                                 sym_paddr_t end,
                                 const string &filename,
                                 int64_t row,
                                 int64_t col)
{
    string_cptr_t filename_ptr = sym.lookup_source_file(filename);

    statement_ptr_t statement(
        new statement_t(
            sym_paddr_range_t(base, end), filename_ptr, row, col));

    statement_map.insert(make_pair(base, statement));

    return statement;
}

bool
statement_cache_t::lookup_statement(sym_paddr_t addr,
                                    statement_ptr_t &ret) const
{
    statement_map_t::const_iterator map_iter = statement_map.upper_bound(addr);

    if (map_iter == statement_map.begin())
        return false;

    statement_ptr_t statement = (--map_iter)->second;

    if (!statement->range.contains(addr))
        return false;

    ret = statement;
    return true;
}

END_NAMESPACE;
END_NAMESPACE;
