/// @file library/sym/statement_cache.hh

// Copyright (c) 2010 Acumem AB.  All rights reserved.

#ifndef STATEMENT_CACHE_HH
#define STATEMENT_CACHE_HH

#include "sym.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

struct statement_t {
    statement_t(const sym_paddr_range_t &range,
                const string_cptr_t &filename,
                int64_t row,
                int64_t col)
	: range(range),
	  filename(filename),
	  row(row),
          col(col)
	{}

    const sym_paddr_range_t range;
    const string_cptr_t filename;
    const int64_t row;
    const int64_t col;
};

typedef shared_ptr<statement_t> statement_ptr_t;

typedef map<sym_paddr_t, statement_ptr_t> statement_map_t;

class statement_cache_t {
public:
    statement_cache_t(sym_t &sym)
        : sym(sym)
        {}

    /// If base == end the upper bound of the statement is treated as unknown
    /// and the statement will stretch to the base of the next statement.
    statement_ptr_t add_statement(sym_paddr_t base,
                                  sym_paddr_t end,
                                  const string &filename,
                                  int64_t row,
                                  int64_t col);

    bool lookup_statement(sym_paddr_t pc, statement_ptr_t &ret) const;

private:
    sym_t &sym;
    statement_map_t statement_map;
};

END_NAMESPACE;
END_NAMESPACE;

#endif /* STATEMENT_CACHE_HH */
