/// @file library/sym/string_cache.cc

// Copyright (c) 2010 Acumem AB.  All rights reserved.

#include "string_cache.hh"

#include <boost/weak_ptr.hpp>

#include <map>

using namespace std;
using namespace boost;

using boost::weak_ptr;

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

typedef weak_ptr<const string> string_wcptr_t;

struct string_cache_cmp {
    bool operator()(const string *a, const string *b) const {
        return *a < *b;
    }
};

typedef map<const string *, string_wcptr_t, string_cache_cmp> string_cache_t;

static string_cache_t cache;

static void
string_cache_deleter(const string *s)
{
    string_cache_t::iterator i = cache.find(s);
    assert(i != cache.end());
    cache.erase(i);
}

string_cptr_t
string_cache_lookup(const string &s)
{
    string_cache_t::iterator i = cache.find(&s);
    if (i != cache.end()) {
        assert(!i->second.expired());
        return i->second.lock();
    }
    else {
        string_cptr_t sp(new string(s), string_cache_deleter);
        cache.insert(make_pair(sp.get(), sp));
        return sp;
    }
}

END_NAMESPACE;
END_NAMESPACE;
