/// @file library/warning/warning.hh

// Copyright (c) 2007-2009 Acumem AB.  All rights reserved.

#ifndef WARNING_HH
#define WARNING_HH

#include <string>

#include <boost/shared_ptr.hpp>

#include "library/lang/common_macros.h"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(library);

using namespace std;

using boost::shared_ptr;

/**
 * Base class for all warnings. Inherit from this class and implement
 * the what() function in custom warnings. Warnings that don't require
 * their own class may use the generic_warning_t class.
 */
class warning_base_t
{
public:
    warning_base_t();
    virtual ~warning_base_t();

    /**
     * Returns a detailed descrtiption of the error.
     */
    virtual string what() const = 0;


    enum severity_t {
	Severity_None,
	Severity_Info,
	Severity_Warning,
	Severity_Error
    };

    virtual severity_t severity() const
	{
	    return Severity_Warning;
	}
};

typedef shared_ptr<const warning_base_t> warning_const_ptr_t;

/**
 * A generic warning class.
 */
class generic_warning_t
    : public warning_base_t
{
public:
    generic_warning_t(const string &what,
		      severity_t severity = Severity_Warning);

    virtual ~generic_warning_t();

    virtual string what() const;

    virtual severity_t severity() const;

private:
    string _what;
    severity_t _severity;
};

END_NAMESPACE;
END_NAMESPACE;

#endif
