/** @file sampler/src/atf_reader.h */

/* Copyright (c) 2006-2008, 2010-2011 Rogue Wave Software, Inc.  All rights reserved. */

/*
 * Author: Mikael Pettersson.
 */
#ifndef ATF_READER_H
#define ATF_READER_H

#include "atf.h"

/* The type of an ATF reader object. Clients must not access it directly. */
struct atf_reader {
    FILE *fin;
    uintptr_t prev_iaddr;
    unsigned int prev_tid;
    unsigned long long prev_local_macnt;
    unsigned long long prev_global_macnt;
    int macnts_are_uncompressed;
#ifdef __x86_64__
    int compat32;
#endif
};

/* Initialise an ATF reader object on top of the given FILE* object.
 * Decodes the ATF header and stores it in `*header'.
 * Returns < 0 on failure.
 */
int atf_read_init(FILE *fin, struct atf_reader *reader, struct atf_header *header);

/* Read the next ATF packet via the ATF reader object.
 * Returns 1 on success, 0 on end-of-file, and -1 on other errors.
 */
int atf_read_packet(struct atf_reader *reader, struct atf_packet *packet);

#endif /* ATF_READER_H */
