/// @file tools/acurate/acurate.cc

// Copyright (c) 2006-2010 Acumem AB.  All rights reserved.

#include "view_feedback.hh"
#include "report_runner.hh"
#include "time.h"

#include "analysis/feature_base/exceptions.hh"
#include "library/stream/stream.hh"
#include "library/feedback/licensed_feedback.hh"
#include "library/version/version.h"

#if defined(__linux__) || defined(__sun__)
#include <sys/resource.h>
#endif

#include "product.h"

using namespace acumem::library;
using namespace acumem::analysis;
using namespace acumem::acurate;
using namespace std;
using namespace boost;

BRING_IN_VERSIONINFO

class my_report_url_t : public report_url_t {
public:
    virtual void report_url(const std::string &url)
	{
	    cout << "Report server launched. Listening on url:" << endl
		 << url << endl;
	}
};

int
main(int argc, char *argv[])
{
    try {
        warning_consumer_ptr_t wc(new warning_consumer_stderr_t());
        license_manager_ptr_t license(new license_manager_t(wc));

	feature_set_ptr_t fs = license->acquire(license_manager_t::view_kind);

	if (!fs) {
	    cerr << "No license token available." << endl;
	    exit(1);
	}

	if (!fs->cmd_line()) {
	    cerr << "Your license does not include command line tools." << endl;
	    exit(1);
	}


#ifdef VIEW_STATIC_REPORT
	if (!fs->static_report()) {
	    cerr << "Your license does not allow generation of static reports." << endl;
	    exit(1);
	}
#endif

	warning_consumer_ptr_t warning_consumer(new warning_consumer_stderr_t);
	feedback_ptr_t feedback(new view_feedback_t(license));

        workspace_ptr_t workspace(new workspace_t());
	report_runner_t report_runner(workspace, warning_consumer, feedback);

	if (!report_runner.parse_args(true,
                                      fs->mt(),
                                      fs->multisocket(),
                                      fs->sw_prefetch(),
                                      fs->unlimited_callstack(),
                                      fs->blocking_expert(),
                                      argc,
                                      argv))
	    return 1;

	my_report_url_t report_url;
	if (!report_runner.run(license,
			       report_runner.no_browser() ? &report_url : NULL))
	    return 1;

    } catch (const feature_exception_t &t) {
        cerr << "Error: " << t.what() << endl;
	return 1;
    } catch (const stream_exception_t &se) {
        cerr << "Error: " << se.what() << endl;
	return 1;
    } catch (const bad_alloc &) {
        cerr << "Error: Out of memory." << endl;
#if defined(__linux__) || defined(__sun__)
        struct rlimit rlim;
        int res = getrlimit(RLIMIT_AS, &rlim);
        assert(res == 0);
        if (rlim.rlim_cur != (rlim_t)RLIM_INFINITY ||
            rlim.rlim_max != (rlim_t)RLIM_INFINITY)
            cerr << endl
                 << "A resource limit has been set on the available virtual "
                 << "address space. You may want to try to increase or remove "
                 << "it, e.g., \"ulimit -S -v unlimited\"." << endl;
#endif
        return 1;
    } catch (const runtime_error &re) {
	cerr << "Error: " << re.what() << endl;
	return 1;
    }

    return 0;
}
