/// @file tools/acurate/conf.hh

// Copyright (c) 2007-2010 Acumem AB.  All rights reserved.

#ifndef INCLUDED_ACURATE_CONF_HH
#define INCLUDED_ACURATE_CONF_HH

#include "library/feedback/feedback.hh"
#include "library/warning/warning_consumer.hh"

#include <string>
#include <vector>
#include <boost/filesystem.hpp>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);

extern const bool static_report;

struct conf_t {
    conf_t(acumem::library::warning_consumer_ptr_t &warning_consumer,
	   acumem::library::feedback_ptr_t feedback);
    bool init(bool interactive, int argc, char **argv);

    /// Create introduction HTML page if true.
    bool intro_page;

    /// Use a fixed scale for fr/mr/util diagrams.
    bool use_mr_scale;
    double max_mr_scale;

    boost::filesystem::path installation_dir;

    /// Intermediate format report file.
    std::string report_file;

    /// If non-zero, spawn a webserver on the specified port instead of
    /// preparing a static report.
    /// If zero, let the webserver find a port, and update this field.
    /// If -1, don't start a server.
    int server_port;

    /// Do not launch a browser if true.
    bool no_browser;

    /// Server inactivity timeout [seconds]
    int inactivity_timeout;

    /// Full path to report directory. Not necessarily an absolute path, but
    /// the full path specified by the user.
    std::string report_path;

    /// Name of the report directory itself.
    std::string report_dir;

    /// cookie - magic value associated with web session for
    /// authentication purposes.
    std::string cookie;

    /// Licensed features.
    bool feature_mt_advice;
    bool feature_sw_prefetch;
    bool feature_unlimited_callstack;
    bool feature_blocking_expert;
    bool feature_multisocket;

    /// Whether to allow several login sessions
    bool allow_many_logins;

    acumem::library::warning_consumer_ptr_t warning_consumer;
    acumem::library::feedback_ptr_t feedback;
};

typedef boost::shared_ptr<conf_t> conf_ptr_t;
typedef boost::shared_ptr<const conf_t> conf_cptr_t;

END_NAMESPACE;
END_NAMESPACE;

#endif
