/// @file tools/acurate/csv.hh

// Copyright (c) 2007-2009 Acumem AB.  All rights reserved.

#ifndef CSV_HH
#define CSV_HH

#include "analysis/report/report.hh"

#include <list>
#include <string>
#include <iostream>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);

using namespace acumem::analysis;

typedef vector<double> csv_data_vector_t;

class csv_t {
public:
    csv_t(const rpt_report_cptr_t &report,
          rpt_sweep_id_t sweep_id,
          const string &name,
          char quote,
          char delim)
	: report(report),
          sweep_id(sweep_id),
          name(name),
          quote(quote),
          delim(delim)
	{}

    void add_series(const string &name,
                    const csv_data_vector_t &data) {
	series_list.push_back(private_series_t(name, data));
    }
    void write_csv_file(const string &filename) const;

private:
    const rpt_report_cptr_t report;
    const rpt_sweep_id_t sweep_id;
    const string name;
    const char quote;
    const char delim;

    struct private_series_t {
	private_series_t(const string &name,
                         const csv_data_vector_t &data)
	    : name(name),
              data(data)
	    {
	    }
	const string name;
	const csv_data_vector_t data;
    };

    typedef list<private_series_t> series_list_t;

    series_list_t series_list;

    void q(ostream &f, const string &x) const {
	if (quote)
	    f << quote << x << quote;
	else
	    f << x;
    }
};

END_NAMESPACE;
END_NAMESPACE;

#endif
