/// @file tools/acurate/drill_down.hh

// Copyright (c) 2007-2009 Acumem AB.  All rights reserved.

#ifndef INCLUDED_DRILL_DOWN_HH
#define INCLUDED_DRILL_DOWN_HH

#include "tools/acurate/diagram.hh"
#include "analysis/report/report.hh"

#include <boost/shared_ptr.hpp>

#include <ostream>
#include <string>
#include <list>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);

using namespace acumem::analysis;

// ----- drill_down_t ---------------------------------------------------------

class drill_down_t {
public:
    drill_down_t(
        const rpt_report_cptr_t &report,
        rpt_stats_sweep_id_t stats_sweep_id);

    virtual ~drill_down_t();

    virtual void write_files() = 0;

    void prepare_fr(ostream &os);
    void prepare_fr(ostream &os, double max_scale);
    void prepare_wb(ostream &os);
    void prepare_wb(ostream &os, double max_scale);
    void prepare_su(ostream &os);

protected:
    void setup_diagram(diagram_t &d);
    void add_group(diagram_t &d, int group_id, const color_t &grid_color);
    void add_group(diagram_t &d, int group_id, const color_t &grid_color,
		   double max_mr_scale);

    void add_fetch_ratio(diagram_t &diagram, int group_id);
    void add_util_corr_fr(diagram_t &diagram, int group_id);
    void add_miss_ratio(diagram_t &diagram, int group_id);
    void add_write_back_ratio(diagram_t &diagram, int group_id);
    void add_util_corr_wbr(diagram_t &diagram, int group_id);
    void add_fetch_util(diagram_t &diagram, int group_id);
    void add_write_back_util(diagram_t &diagram, int group_id);
    void add_spatial_util(diagram_t &diagram, int group_id);

    virtual int diagram_width() = 0;
    virtual int diagram_height() = 0;

    const rpt_report_cptr_t report;
    const rpt_stats_sweep_id_t stats_sweep_id;
};

typedef boost::shared_ptr<drill_down_t> drill_down_ptr_t;
typedef list<drill_down_ptr_t> drill_down_list_t;

// ----- drill_down_global_t --------------------------------------------------

class drill_down_global_t
    : public drill_down_t
{
public:
    drill_down_global_t(
        const rpt_report_cptr_t &report,
        const string &file_base,
        double max_mr_scale);

    void write_files();

protected:
    int diagram_width() {
        return 300;
    }

    int diagram_height() {
        return 100;
    }

private:
    void report_csv();

    const string file_base;
    const double max_mr_scale;
};

// ----- drill_down_pc_set_t --------------------------------------------------

class drill_down_pc_set_t
    : public drill_down_t
{
public:
    drill_down_pc_set_t(
        const rpt_report_cptr_t &report,
        const string &id,
        rpt_location_set_id_t location_set_id);

    void write_files();

protected:
    int diagram_width() {
        return 300;
    }

    int diagram_height() {
        return 100;
    }

private:
    const string id;
};

END_NAMESPACE;
END_NAMESPACE;

#endif
