/// @file tools/acurate/html_about.cc

// Copyright (c) 2009-2010 Acumem AB.  All rights reserved.

#include "about.hh"
#include "macros.hh"
#include "help_links.hh"
#include "file_png.hh"

#include "../link_factory.hh"

#include "library/version/version.h"
#include "library/license/license.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);

using namespace std;
using namespace boost;

static void
legend_item(const html_context_t &html_context,
	    xml_file_t &xml,
	    const string &icon,
	    const string &name,
	    const string &helplink)
{
    image_factory_ptr_t image_factory = html_context.link_factory->image_factory();
    NL T_A A_TARGET("_blank")
	A_HREF(html_context.link_factory->get_help_link(helplink))
	T_IMG
	A_STYLE("vertical-align:text-bottom;")
	A_SRC(image_factory->static_image_path(icon)) A_BORDER("0");
    TEXT(name);
    T_BR;
}

void
html_about_t::emit(xml_file_t &xml, html_context_t &html_context)
{
    image_factory_ptr_t image_factory = html_context.link_factory->image_factory();

    T_IMG A_STYLE("float:right; padding: 1em;")
	A_SRC(image_factory->static_image_path(PNG_LOGO_SMALL));

    T_H3 emit_product_name_full(xml);
    char buf[20];
    version_get_svnversion(buf, sizeof(buf));
    T_P A_CLASS("smaller")
	TEXT(string("Version: " PACKAGE_VERSION ", Build: " ) + buf);

    T_P NL {
	TEXT("View the ");
	T_A A_TARGET("_blank")
	    A_HREF(html_context.link_factory->get_help_link(HELP_MANUAL))
	    TEXT("manual");
	TEXT(".");
    }

    T_H3 TEXT("Legend");
    T_TABLE A_WIDTH("100%") A_CLASS("summary") {
	T_TR {
	    T_TH TEXT("Slowspot issues");
	    T_TH TEXT("Opportunity issues");
	}
	T_TR {
	    T_TD {
		// red
		legend_item(html_context, xml, PNG_FETCH_UTILIZATION,
			    TEXT_FETCH_UTILIZATION, HELP_UTILIZATION);
		legend_item(html_context, xml, PNG_WRITE_BACK_UTILIZATION,
			    TEXT_WRITE_BACK_UTILIZATION, HELP_UTILIZATION);
		legend_item(html_context, xml, PNG_COMM_UTILIZATION,
			    TEXT_COMM_UTILIZATION, HELP_COMMUTIL);

		legend_item(html_context, xml, PNG_LOOPORDER,
			    TEXT_LOOPORDER, HELP_LOOP_NESTING);
		legend_item(html_context, xml, PNG_RANDOM,
			    TEXT_RANDOM, HELP_RANDOM);
		legend_item(html_context, xml, PNG_PREFETCH_CLOSE,
			    TEXT_PREFETCH_CLOSE, HELP_PREF_CLOSE);
		legend_item(html_context, xml, PNG_PREFETCH_FAR,
			    TEXT_PREFETCH_FAR, HELP_PREF_DISTANT);
		legend_item(html_context, xml, PNG_PREFETCH_HITS,
			    TEXT_PREFETCH_HITS, HELP_PREF_UNNEC);
		legend_item(html_context, xml, PNG_FALSE_SHARING,
			    TEXT_FALSE_SHARING, HELP_FALSESHARING);
	    }
	    T_TD {
		// Purple
		legend_item(html_context, xml, PNG_SPATIAL_BLOCK,
			    TEXT_SPATIAL_BLOCK, HELP_SPAT_BLOCK);
		legend_item(html_context, xml, PNG_TEMPORAL_BLOCK,
			    TEXT_TEMPORAL_BLOCK, HELP_TEMP_BLOCK);
		legend_item(html_context, xml, PNG_SPATIAL_TEMPORAL_BLOCK,
			    TEXT_SPATIAL_TEMPORAL_BLOCK, HELP_TEMP_SPAT_BLOCK);
		legend_item(html_context, xml, PNG_FUSION,
			    TEXT_FUSION, HELP_LOOP_FUSION);
		legend_item(html_context, xml, PNG_PREFETCH_NT,
			    TEXT_PREFETCH_NT, HELP_PREFETCHNTA);
		legend_item(html_context, xml, PNG_WRITE_NT,
			    TEXT_WRITE_NT, HELP_WNT_POSSIBLE);
		// Blue
		legend_item(html_context, xml, PNG_FETCH_HOTSPOT,
			    TEXT_FETCH_HOTSPOT, HELP_FETCH_HOTSPOT);
		legend_item(html_context, xml, PNG_WRITE_BACK_HOTSPOT,
			    TEXT_WRITE_BACK_HOTSPOT, HELP_WB_HOTSPOT);
		legend_item(html_context, xml, PNG_COMM_HOTSPOT,
			    TEXT_COMM_HOTSPOT, HELP_COMMSPOT);
	    }
	}
    }

    T_H3 TEXT("Contact information:");
    T_TABLE A_CLASS("summary") {
	T_TR T_TH A_COLSPAN("2") TEXT("ParaTools, Inc.");
	T_TR {
	    T_TH TEXT("Address");
	    T_TD {
		T_P TEXT("2836 Kincaid St.");
		T_P TEXT("Eugene, OR 97405 USA");
	    }
	}
	T_TR {
	    T_TH TEXT("Phone");
	    T_TD TEXT("+1 541-913-8797");
	}
	T_TR {
	    T_TH TEXT("Phone");
	    T_TD TEXT("+1 443-543-5475");
	}
	T_TR {
	    T_TH TEXT("Fax");
	    T_TD TEXT("+1 541-343-6086");
	}
	T_TR {
	    T_TH TEXT("Support Email");
	    T_TD T_A A_HREF("mailto:threadspotter@paratools.com")
		TEXT("threadspotter@paratools.com");
	}
	T_TR {
	    T_TH TEXT("Web");
	    T_TD T_A A_HREF("http://www.paratools.com") A_TARGET("_blank")
		TEXT("http://www.paratools.com");
	}
    }

#if 0
    // This was removed with the introduction of FlexNet, since these
    // fields are not recorded in the license file
    T_H3 TEXT("License");
    T_TABLE A_CLASS("summary") {
	T_TR {
	    T_TH TEXT("Licensee");
	    T_TD TEXT(html_context.license->get_name() + string(", ") +
                      html_context.license->get_company());
	}
	string expiration = html_context.license->get_expiration();
	if (expiration != "unlimited") T_TR {
            T_TH TEXT("License expires");
            T_TD TEXT(expiration);
        }
	string support = html_context.license->get_support_expiration();
	if (support != expiration && support != "unlimited") T_TR {
            T_TH TEXT("Support ends");
            T_TD TEXT(support);
        }
	string condition = html_context.license->get_conditions();
	if (condition.size()) T_TR {
            T_TH TEXT("Terms of use");
            T_TD TEXT(condition);
        }
    }
#endif

    emit_copyright(xml, false);
}

END_NAMESPACE;
END_NAMESPACE;
