/// @file tools/acurate/html_execution.cc

// Copyright (c) 2009-2010 Acumem AB.  All rights reserved.

#include "execution.hh"
#include "help_links.hh"

#ifdef _WIN32
#include <tchar.h>
#include <windows.h>
#endif

#undef TEXT
#include "macros.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);

using namespace std;
using namespace boost;

static void
emit_thread_binding(xml_file_t &xml,
		    html_context_t &html_context)
{
    const rpt_report_t &report(*html_context.report);

    T_TABLE A_CLASS("summary") {
	T_TR {
	    T_TH TEXT("Cache #");
	    T_TH TEXT("Thread id");
	    T_TH TEXT("Virtual Thread id");
	}

	bool alt = true;

	for (rpt_cache_id_t i = 0; i < report.num_caches; i++) {

	    alt = !alt;

	    const rpt_thread_id_set_t &threads = report.cache_threads.at(i);
	    size_t num_threads = threads.size();
	    BOOST_FOREACH(rpt_thread_id_t thread_id, threads) {
		const rpt_context_id_t context_id(
                    report.get_context_id(thread_id));
		T_TR {
		    if (alt)
			A_CLASS("alternate");

		    if (num_threads != 0) {
			T_TD A_ROWSPAN(lexical_cast<string>(num_threads))
			    A_VALIGN("top") TEXT(lexical_cast<string>(i));
			num_threads = 0;
		    }

		    T_TD TEXT(lexical_cast<string>(context_id));
		    T_TD TEXT(lexical_cast<string>(thread_id));
		}
	    }
	}
    }
}

void
html_execution_t::emit(xml_file_t &xml, html_context_t &html_context)
{
    const rpt_report_t &report(*html_context.report);

    T_TABLE A_CLASS("summary") {
	T_TR {
	    T_TH emit_glossary_phrase(html_context, xml,
				      "Command line", HELP_EXECUTION_CMD);
	    T_TD A_COLSPAN("2")
		BOOST_FOREACH(const std::string &s, report.argv)
		TEXT(s);
	}

	T_TR {
	    T_TH emit_glossary_phrase(html_context, xml, "Sampling start time",
				      HELP_EXECUTION_START);
	    T_TD A_COLSPAN("2") emit_time(xml, report.begin_time);
	}

	T_TR {
	    T_TH emit_glossary_phrase(html_context, xml, "Sampling end time",
				      HELP_EXECUTION_END);
	    T_TD A_COLSPAN("2") emit_time(xml, report.end_time);
	}

	T_TR {
	    T_TH emit_glossary_phrase(html_context, xml, "User", HELP_EXECUTION_USER);
	    T_TD A_COLSPAN("2") TEXT(report.user);
	}

	T_TR {
	    T_TH emit_glossary_phrase(html_context, xml, "Sample file",
				      HELP_EXECUTION_SAMPLEFILE);
	    T_TD A_COLSPAN("2") TEXT(report.sample_file);
        }

	T_TR {
	    T_TH emit_glossary_phrase(html_context, xml, "Number of samples",
				      HELP_EXECUTION_NUMSAMPLES);
	    T_TD A_COLSPAN("2") TEXT(lexical_cast<string>(report.sample_count));
	}

	T_TR {
	    T_TH emit_glossary_phrase(html_context, xml, "Analysis parameters",
				      HELP_EXECUTION_APARAMS);
	    T_TD A_COLSPAN("2") {
		BOOST_FOREACH(const std::string &s,
			      report.analysis_argv) {
		    TEXT(s);
		}
	    }
	}

	T_TR {
	    T_TH emit_glossary_phrase(html_context, xml, "Analysis time",
				      HELP_EXECUTION_ATIME);
	    T_TD A_COLSPAN("2") emit_time(xml, report.analysis_time);
	}

	if (report.mt_sampled) {
	    T_TR {
		T_TH emit_glossary_phrase(html_context, xml,
					  "Effective thread binding",
					  HELP_EXECUTION_THREADBIND);
		T_TD A_COLSPAN("2") emit_thread_binding(xml, html_context);
	    }
	}
    }

    emit_copyright(xml, true);
    emit_terms_of_use(html_context, xml);
}

END_NAMESPACE;
END_NAMESPACE;
