/// @file tools/acurate/html_files.cc

// Copyright (c) 2009-2010 Acumem AB.  All rights reserved.

#include "files.hh"

#include "../consumer.hh"
#include "help_links.hh"
#include "macros.hh"

#include <sstream>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);

using namespace std;
using namespace boost;

static const char *
describe_debug_info_status(
    rpt_debug_info_status_t debug_info_status)
{
    switch (debug_info_status) {
    case Rpt_Debug_File_Not_Found:
        return "File not found";
    case Rpt_Debug_Error_Reading_File:
        return "Error reading file";
    case Rpt_Debug_No_Info_Found:
        return "No debug info found";
    case Rpt_Debug_Symbols_Found:
        return "Only symbols found";
    case Rpt_Debug_Debug_Info_Found:
        return "Debug info found";
    default:
        assert(0);
        while (1);
    }
}

void
html_files_t::emit(xml_file_t &xml, html_context_t &html_context)
{
    const rpt_report_t &report(*html_context.report);

    if (html_context.result_unreliable())
	T_DIV A_CLASS("warning") T_P {
	    T_B TEXT("Warning!");
	    TEXT("These results are based on too few samples to be reliable.");
	}

    T_H3 TEXT("Source code files");

    if (html_context.file_repos->file_map.size()) {
	T_TABLE A_CLASS("summary") {
	    T_TR T_TH TEXT("File");
	    BOOST_FOREACH(const file_map_t::value_type &v,
			  html_context.file_repos->file_map)
	    {
		T_TR T_TD link_to_file(html_context, xml, v.first);
	    }
	}
    } else {
	T_P TEXT("No source files were found. Make sure the target program "
	       "has debug information.");
    }

    NL T_H3 TEXT("Shared libraries");

    if (!report.binaries.empty()) {
	T_TABLE A_CLASS("summary table-autosort:0 "
			"table-stripeclass:alternate") {
	    T_THEAD {
		T_TR {
		    T_TH A_CLASS("table-sortable:default") TEXT("Filename");
		    T_TH A_CLASS("table-sortable:default") TEXT("Symbols");
		    T_TH A_CLASS("table-sortable:numeric") TEXT("Low address");
		    T_TH A_CLASS("table-sortable:numeric") TEXT("High address");
		}
	    }

	    T_TBODY {
		BOOST_FOREACH(const rpt_binary_t &binary, report.binaries) {
		    T_TR {
			T_TD TEXT(binary.filename);
			T_TD TEXT(describe_debug_info_status(binary.debug_info_status));

			stringstream from;
			from << "0x" << hex << binary.base;
			stringstream to;
			to << "0x" << hex << binary.base + binary.size;

			T_TD TEXT(from.str());
			T_TD TEXT(to.str());
		    }
		}
	    }
	}
    } else {
	T_P TEXT("No shared libraries were recorded.");
    }

    emit_copyright(xml, true);
    emit_terms_of_use(html_context, xml);
}

END_NAMESPACE;
END_NAMESPACE;
