/// @file tools/acurate/html_front.hh

// Copyright (c) 2008-2009 Acumem AB.  All rights reserved.

#ifndef HTML_FRONT_HH
#define HTML_FRONT_HH

#include "issue.hh"
#include <fstream>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);

// ----------------------------------------------------------------------

class verdict_t;
typedef boost::shared_ptr<const verdict_t> verdict_cptr_t;

// ----------------------------------------------------------------------

class application_verdict_t {
public:
    application_verdict_t(html_context_t &html_context);

    void render_fr_gauge(std::ostream &os) const;
    void render_mr_gauge(std::ostream &os) const;
    void render_utilization_gauge(std::ostream &os) const;
    void render_mt_gauge(std::ostream &os) const;

    double bandwidth_problem_ratio;
    double latency_problem_ratio;
    double locality_problem_ratio;

    bool has_locality_advice;
    bool has_mt_advice;
    bool has_bandwidth_advice;
    bool has_latency_advice;
    bool has_issues;

    bool has_bandwidth_problem() const;
    bool has_latency_problem() const;
    bool has_locality_problem() const;

    verdict_cptr_t get_verdict() const;
};

// ----------------------------------------------------------------------

class html_front_t : public html_util_t {
public:
    html_front_t(html_context_t &html_context,
		 const application_verdict_t &app_verdict);
    void emit(xml_file_t &xml);

private:
    void emit_warning(xml_file_t &xml);
    void emit_header(xml_file_t &xml);
    void emit_logo(xml_file_t &xml);
    void emit_your_execution(xml_file_t &xml);
    void emit_what(xml_file_t &xml);
    void emit_resources(xml_file_t &xml);

    void emit_problem_gauge(xml_file_t &xml, std::string name);

    html_context_t &html_context;
    const application_verdict_t &app_verdict;
};



END_NAMESPACE;
END_NAMESPACE;

#endif
