/// @file tools/acurate/html_wnt_possible.cc

// Copyright (c) 2008-2010 Acumem AB.  All rights reserved.

#include "wnt_possible.hh"
#include "macros.hh"
#include "help_links.hh"

#include "../diagram.hh"
#include "../file_repos.hh"
#include "../link_factory.hh"
#include "../loop_info.hh"
#include "../xml_file.hh"

#include <sstream>
#include <iomanip>
#include <set>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);

using namespace std;
using namespace boost;
using namespace acumem::analysis;

html_wnt_possible_t::html_wnt_possible_t(html_context_t &html_context,
					 rpt_issue_id_t issue_id)
    : html_issue_t(html_context, issue_id)
{
}

summary_type_bitset_t
html_wnt_possible_t::summary_type() const
{
    summary_type_bitset_t result;

    result.set(SUMMARY_BANDWIDTH);

    return result;
}

string
html_wnt_possible_t::get_icon() const
{
    string icon = PNG_WRITE_NT;
    return html_context.link_factory->image_factory()->static_image_path(icon);
}

void
html_wnt_possible_t::annotate_source()
{
    annotate_generic(get_issue().depgroup_id, false);
}

void
html_wnt_possible_t::emit_issue_file(xml_file_t &issue_xml)
{
    const rpt_report_t &report(*html_context.report);
    const file_repos_t &file_repos(*html_context.file_repos);
    const rpt_wnt_possible_issue_t &issue(get_issue());

    fr_location_info_vector_t location_dep_info_vector =
	file_repos.get_depgroup_line_info(report, issue.depgroup_id);
    fr_location_info_vector_t location_loop_info_vector =
	file_repos.get_loop_incl_line_info(report, issue.loop_id);

    html_context.interesting_depgroups.insert(issue.depgroup_id);
    html_context.interesting_loops.insert(issue.loop_id);

    XML_OBJECT(issue_xml) {
	T_DIV A_ID("sections") A_STYLE("display: none") {
            emit_issue_header(issue_xml);

	    // Title
	    SECTION(h1, "d", "Issue #" + get_display_number() + ": " + get_name(),
                    get_icon(), HELP_WNT_POSSIBLE, false, false);
	    dump_inferior(xml);

	    // instruction group statistics
	    section_statistics(
                xml,
                html_context,
                "Statistics for instructions of this issue",
                HELP_ISSUE_STATISTICS,
                report.get_depgroup_stats_id(issue.depgroup_id),
                "d" + lexical_cast<string>(issue.depgroup_id),
                location_dep_info_vector,
                location_loop_info_vector,
                false, // initially visible
                false, // is_loop
                false, // show_avoidable
                "istat");

	    // instruction group instructions
	    section_instructions(xml, html_context,
				 "Instructions involved in this issue",
				 HELP_ISSUE_INSTRUCTIONS,
				 location_dep_info_vector,
				 location_loop_info_vector,
				 2,
				 false,
				 "iinstr");

	    fr_location_info_vector_t next_readers_info =
		file_repos.get_line_info(report, issue.next_readers);
	    section_instructions(xml, html_context,
				 "Instructions using data produced by this "
				 "instruction group",
				 HELP_WNT_POSSIBLE_CONSUMERS,
				 next_readers_info,
				 next_readers_info,
				 0,
				 false,
				 "inext");

	    // -----------------------------------------------------
	    // Common section for describing the Loop

	    all_loop_sections(xml, html_context,
			      issue.loop_id, location_loop_info_vector);

	    // -----------------------------------------------------
	    // done

            emit_issue_footer(html_context, issue_xml);


	}
    }
}

void
html_wnt_possible_t::emit_summary(xml_file_t &xml,
				  summary_context_t &summary_context,
				  summary_type_t summary_type)
{
    const rpt_report_t &report(*html_context.report);
    const file_repos_t &file_repos(*html_context.file_repos);
    const rpt_wnt_possible_issue_t &issue(get_issue());

    const fr_location_info_vector_t location_dep_info_vector =
	file_repos.get_depgroup_line_info(report, issue.depgroup_id);
    const fr_location_info_vector_t location_loop_info_vector =
	file_repos.get_loop_incl_line_info(report, issue.loop_id);

    add_summary_line(
        xml,
        summary_context,
        get_rel_file_name(),
        get_name(),
        get_icon(),
        location_dep_info_vector,
        location_loop_info_vector,
        report.get_depgroup_stats_id(issue.depgroup_id),
        lexical_cast<string>(issue.loop_id),
        summary_type);
}

END_NAMESPACE;
END_NAMESPACE;
