#include "link_factory_static.hh"

#include "html_context.hh"

#include "html/macros.hh"
#include "html/issue.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);

void
tab_factory_static_t::begin(xml_file_t &xml, const std::string &id)
{
    xml.start_tag("div");
    xml.add_attribute("class", "tabber");
}

void
tab_factory_static_t::end(xml_file_t &xml)
{
    xml.end_tag();
}

void
tab_factory_static_t::new_tab(xml_file_t &xml,
			      const std::string &title,
			      content_function_t content_function)
{
    T_DIV A_CLASS("tabbertab") A_TITLE(title) content_function(xml);
}

// ----------------------------------------------------------------------

std::string
image_factory_static_t::static_image_path(const std::string &image_name)
{
    return image_name;
}

std::string
image_factory_static_t::global_diagram_path(const std::string &image_name)
{
    return image_name;
}

std::string
image_factory_static_t::global_gauge_path(const std::string &image_name)
{
    return image_name;
}

// ----------------------------------------------------------------------


void
link_factory_static_t::add_source_link(xml_file_t &xml)
{
}

void
link_factory_static_t::add_section_toggle(xml_file_t &xml)
{
}

/// Get a subfactory for adding tabs
tab_factory_ptr_t
link_factory_static_t::tab_factory()
{
    return tab_factory_ptr_t(new tab_factory_static_t);
}

image_factory_ptr_t
link_factory_static_t::image_factory()
{
    return image_factory_ptr_t(new image_factory_static_t);
}

void
link_factory_static_t::emit_sliced_statistics_link(xml_file_t &xml,
						   html_context_t &html_context,
						   rpt_sliced_stats_id_t stats_id,
						   const tp_field_t &field)
{
    const rpt_report_t &report = *(html_context.report);
    const rpt_sliced_stats_t &stats = report.get_sliced_stats(stats_id);
    std::string unique_id = xml.unique_id();
    string action = "javascript:statistics('" + unique_id + "');";
    T_SPAN {
	if (html_context.feature_mt_advice)
	    A_ONCLICK(action) A_TITLE("Click for per-thread statistics")
		field.emit(xml, report.get_sliced_stats_global(stats));
	else
	    A_ONCLICK(action)
		field.emit(xml, report.get_sliced_stats_global(stats));

	T_DIV A_STYLE("display:none") A_ID(unique_id) {
	    html_util_t::emit_thread_popup_table(html_context, xml,
						 stats_id, field);
	}
    }
}

    /// Write a link (and other needed data) to represent a drill-down
    /// statistics table.
void
link_factory_static_t::emit_sliced_statistics_link_by_kind(
    xml_file_t &xml,
    html_context_t &html_context,
    rpt_sliced_stats_id_t stats_id,
    const tp_field_by_kind_t &field)
{
    const rpt_report_t &report = *(html_context.report);
    const rpt_sliced_stats_t &stats = report.get_sliced_stats(stats_id);

    const by_kind_value_t total_by_kind =
        field.by_kind(report.get_sliced_stats_global(stats));

    std::string unique_id = xml.unique_id();
    string action = "javascript:statistics('" + unique_id + "');";

    const char *title =
        html_context.feature_mt_advice ?
        "Click for per-thread statistics" :
        "Click for categorized breakdown";

    A_ONCLICK(action) A_TITLE(title)
        field.emit (xml, total_by_kind.total());

    T_SPAN {
	T_DIV A_STYLE("display:none") A_ID(unique_id) {
	    html_util_t::emit_thread_popup_table_by_kind(html_context, xml,
							 stats_id, field);
	}
    }
}

std::string
link_factory_static_t::get_diagram_link(const std::string &type,
					const std::string &kind,
					int id)
{
    return kind + lexical_cast<string>(id) + "-" + type + ".png";
}


std::string
link_factory_static_t::get_help_link(const std::string &section)
{
    return "manual_html/" + section;
}

END_NAMESPACE;
END_NAMESPACE;
