#ifndef ACURATE_LINK_FACTORY_STATIC_HH
#define ACURATE_LINK_FACTORY_STATIC_HH

#include "link_factory.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);


// ----------------------------------------------------------------------

/// A factory to create tabs using the old js based api.

class tab_factory_static_t : public tab_factory_t {
public:
    virtual void begin(xml_file_t &xml, const std::string &id);
    virtual void end(xml_file_t &xml);

    virtual void new_tab(xml_file_t &xml,
			 const std::string &title,
			 content_function_t content_function);
};

// ----------------------------------------------------------------------

class image_factory_static_t : public image_factory_t {
public:
    virtual std::string static_image_path(const std::string &image_name);
    virtual std::string global_diagram_path(const std::string &image_name);
    virtual std::string global_gauge_path(const std::string &image_name);
};

// ----------------------------------------------------------------------

/// An abstract factory that defines methods to add section contents,
/// links to an xml_t stream. Derived classes implement different
/// approaches - adding the contents inline, or adding a reference to
/// the contents for later AJAX-based loading.
class link_factory_static_t : public link_factory_t {
public:
    virtual ~link_factory_static_t() {}

    /// Add a link that populates the source code frame
    virtual void add_source_link(xml_file_t &xml);

    /// Add code to toggle a section.
    virtual void add_section_toggle(xml_file_t &xml);

    /// Get a subfactory for adding tabs
    virtual tab_factory_ptr_t tab_factory();

    /// Get a subfactory for adding tabs
    virtual image_factory_ptr_t image_factory();

    /// Write a link (and other needed data) to represent a drill-down
    /// statistics table.
    virtual void emit_sliced_statistics_link(xml_file_t &xml,
					     html_context_t &html_context,
					     rpt_sliced_stats_id_t stats_id,
					     const tp_field_t &field);

    /// Write a link (and other needed data) to represent a drill-down
    /// statistics table.
    virtual void emit_sliced_statistics_link_by_kind(
	xml_file_t &xml,
	html_context_t &html_context,
	rpt_sliced_stats_id_t stats_id,
	const tp_field_by_kind_t &field);

    /// Get a link to a diagram
    virtual std::string get_diagram_link(const std::string &type,
					 const std::string &kind,
					 int id);

    /// Get the url to a help section
    virtual std::string get_help_link(const std::string &section);

};

END_NAMESPACE;
END_NAMESPACE;

#endif
