/// @file tools/acurate/loop_info.cc

// Copyright (c) 2007-2009 Acumem AB.  All rights reserved.

#include "loop_info.hh"

#include "html/issues.hh"

#include <boost/foreach.hpp>
#include <iterator>
#include <iostream>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);

using namespace std;
using namespace boost;

// ----- depgroup_info_t ------------------------------------------------------

void
depgroup_info_t::add_issue(html_context_t &html_context, rpt_issue_id_t issue_id)
{
    rpt_issue_ptr_t rpt_issue(html_context.report->issues.at(issue_id));

    html_issue_ptr_t issue;
    switch (rpt_issue->get_issue_type()) {
    case Rpt_Issue_Hot_Spot:
        issue.reset(new html_hot_spot_t(html_context, issue_id));
        break;
    case Rpt_Issue_Prefetch:
        issue.reset(new html_prefetch_t(html_context, issue_id));
        break;
    case Rpt_Issue_Utilization:
        issue.reset(new html_utilization_t(html_context, issue_id));
        break;
    case Rpt_Issue_Loop_Order:
        issue.reset(new html_order_t(html_context, issue_id));
        break;
    case Rpt_Issue_Random:
        issue.reset(new html_random_t(html_context, issue_id));
        break;
    case Rpt_Issue_Loop_Fusion:
        issue.reset(new html_fusion_t(html_context, issue_id));
        break;
    case Rpt_Issue_Block:
        issue.reset(new html_block_t(html_context, issue_id));
        break;
    case Rpt_Issue_Comm_Spot:
        issue.reset(new html_comm_spot_t(html_context, issue_id));
        break;
    case Rpt_Issue_Comm_Utilization:
        issue.reset(new html_comm_utilization_t(html_context, issue_id));
        break;
    case Rpt_Issue_False_Sharing:
        issue.reset(new html_false_sharing_t(html_context, issue_id));
        break;
    case Rpt_Issue_Wnt_Possible:
        issue.reset(new html_wnt_possible_t(html_context, issue_id));
        break;
    case Rpt_Issue_Prefetchnta:
        issue.reset(new html_prefetchnta_t(html_context, issue_id));
        break;
    default:
        assert(0);
    }
    issues.push_back(issue);
}

// ----- loop_registry_t ------------------------------------------------------

loop_registry_t::loop_registry_t(const rpt_report_t &report)
{
    // Dummy loop for instructions without a loop, etc. Uses loop id 0.
    rpt_depgroup_vector_t extra_loop_depgroups;
    loop_infos[0] =
        loop_info_ptr_t(
            new loop_info_t(0, extra_loop_depgroups));

    // Handle loops.
    BOOST_FOREACH(const rpt_loop_map_t::value_type &v, report.loops) {
        rpt_loop_id_t loop_id = v.first;
        rpt_depgroup_vector_t depgroups;
        BOOST_FOREACH(rpt_depgroup_id_t depgroup_id, v.second.depgroups)
            depgroups.push_back(depgroup_id);
	loop_infos[loop_id] =
            loop_info_ptr_t(
                new loop_info_t(loop_id, depgroups));
    }

    // Handle dependence groups.
    BOOST_FOREACH(const rpt_depgroup_map_t::value_type &v,
                  report.depgroups)
        depgroup_infos.push_back(
            depgroup_info_ptr_t(
                new depgroup_info_t(v.first)));
}

void
loop_registry_t::add_issue(html_context_t &html_context, rpt_issue_id_t issue_id)
{
    const rpt_issue_ptr_t issue(html_context.report->issues.at(issue_id));
    // If there are issues of types not supported by this version of the
    // product the issue pointer may be NULL, therefore this check.
    if (issue)
        depgroup_infos[issue->get_depgroup_id()]->add_issue(html_context, issue_id);
}

END_NAMESPACE;
END_NAMESPACE;
