/// @file tools/acurate/loop_info.hh

// Copyright (c) 2007-2009 Acumem AB.  All rights reserved.

#ifndef INCLUDED_LOOP_INFO
#define INCLUDED_LOOP_INFO

#include "html/issue.hh"

#include <boost/shared_ptr.hpp>

#include <map>
#include <list>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);

using namespace acumem::analysis;
using namespace acumem::library;

// ----- depgroup_info_t ------------------------------------------------------

struct depgroup_info_t {
    depgroup_info_t(rpt_depgroup_id_t id)
        : id(id)
        {}

    void add_issue(html_context_t &context, rpt_issue_id_t issue_id);

    rpt_depgroup_id_t id;

    html_issue_list_t issues;
};

typedef boost::shared_ptr<depgroup_info_t> depgroup_info_ptr_t;
typedef std::vector<depgroup_info_ptr_t> depgroup_info_vector_t;

// ----- loop_info_t ----------------------------------------------------------

struct loop_info_t {
    loop_info_t(rpt_loop_id_t loop_id,
                const rpt_depgroup_vector_t &depgroup_ids)
        : loop_id(loop_id),
          depgroup_ids(depgroup_ids)
        {}

    rpt_loop_id_t loop_id;
    rpt_depgroup_vector_t depgroup_ids;
};

typedef boost::shared_ptr<loop_info_t> loop_info_ptr_t;
typedef std::map<rpt_loop_id_t, loop_info_ptr_t> loop_info_map_t;

// ----- loop_registry_t ------------------------------------------------------

class loop_registry_t {
public:
    loop_registry_t(const rpt_report_t &report);

    void add_issue(html_context_t &html_context, rpt_issue_id_t issue_id);

    /// Dependence group info, indexed by dependence group id.
    depgroup_info_vector_t depgroup_infos;

    /// Loop info, indexed by loop id.
    loop_info_map_t loop_infos;
};

typedef boost::shared_ptr<loop_registry_t> loop_registry_ptr_t;

END_NAMESPACE;
END_NAMESPACE;

#endif
