/// @file tools/acurate/report_runner.hh

// Copyright (c) 2009-2010 Acumem AB.  All rights reserved.

#ifndef INCLUDED_REPORT_RUNNER_HH
#define INCLUDED_REPORT_RUNNER_HH

#include "tools/acurate/html_context.hh"
#include "analysis/feature_base/feature_base.hh"
#include "library/license/license.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);

using namespace acumem::analysis;
using namespace acumem::library;
using namespace acumem::license;
using namespace std;

class report_url_t {
public:
    virtual ~report_url_t() {}
    virtual void report_url(const std::string &url) =0;
};

extern void run_report(html_context_t &html_context, report_url_t *ru);

// This is a wrapper around a few distinct phases in running the
// reporter.  While everything could be done linearly in one sweep,
// this division allows some freedom in letting different threads
// parse options and run report generation respectively.
class report_runner_t
{
public:
    report_runner_t(workspace_ptr_t workspace,
                    warning_consumer_ptr_t warning_consumer,
                    feedback_ptr_t feedback);
    ~report_runner_t();

    bool parse_args(bool interactive,
                    bool feature_mt_advice,
                    bool feature_multisocket,
                    bool feature_sw_prefetch,
                    bool feature_unlimited_callstack,
                    bool feature_blocking_expert,
                    int argc, char **argv);
    bool parse_args(bool interactive,
                    bool feature_mt_advice,
                    bool feature_multisocket,
                    bool feature_sw_prefetch,
                    bool feature_unlimited_callstack,
                    bool feature_blocking_expert,
                    const vector<string> &args);

    /// Returns true on success, false on error. Probably throws all kinds of
    /// exceptions too.
    bool run(const license_manager_ptr_t license, report_url_t *ru = 0);


    /// Check whether the configuration specifies that no browser
    /// shall be launched.
    bool no_browser() const;

private:
    bool check_container(const string &path) const;

    conf_ptr_t arg_conf;
    feature_context_ptr_t context;
};

END_NAMESPACE;
END_NAMESPACE;

#endif
