#include "dyn_content.hh"
#include "session.hh"

#include "../html_context.hh"

#include <iostream>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);
BEGIN_NAMESPACE(server);

using namespace pion::net;
using namespace std;

dyn_content_t::dyn_content_t(session_collection_t &session_collection,
			     session_ptr_t session)
    : session_collection(session_collection),
      session(session)
{
}

void
dyn_content_t::send_not_found_response(HTTPRequestPtr& http_request,
				       TCPConnectionPtr& tcp_conn)
{
    static const std::string NOT_FOUND_HTML_START =
	"<html><head>\n"
	"<title>404 Not Found</title>\n"
	"</head><body>\n"
	"<h1>Not Found</h1>\n"
	"<p>The requested URL ";
    static const std::string NOT_FOUND_HTML_FINISH =
	" was not found on this server.</p>\n"
	"</body></html>\n";
    HTTPResponseWriterPtr writer(
	HTTPResponseWriter::create(tcp_conn, *http_request,
				   boost::bind(&TCPConnection::finish,
					       tcp_conn)));
    writer->getResponse().setStatusCode(HTTPTypes::RESPONSE_CODE_NOT_FOUND);
    writer->getResponse().setStatusMessage(HTTPTypes::RESPONSE_MESSAGE_NOT_FOUND);
    if (http_request->getMethod() != HTTPTypes::REQUEST_METHOD_HEAD) {
	writer->writeNoCopy(NOT_FOUND_HTML_START);
	writer << http_request->getResource();
	writer->writeNoCopy(NOT_FOUND_HTML_FINISH);
    }
    writer->send();
}

void
dyn_content_t::send_license_error_response(HTTPRequestPtr& http_request,
					   TCPConnectionPtr& tcp_conn)
{
    static const std::string FORBIDDEN_HTML_START =
	"<html><head>\n"
	"<title>403 Forbidden</title>\n"
	"</head><body>\n"
	"<h1>No License</h1>\n"
	"<p>The requested URL ";
    static const std::string FORBIDDEN_HTML_FINISH =
	" could not be served due to lack of a current lease.</p>\n"
	"</body></html>\n";
    HTTPResponseWriterPtr writer(
	HTTPResponseWriter::create(tcp_conn, *http_request,
				   boost::bind(&TCPConnection::finish,
					       tcp_conn)));
    writer->getResponse().setStatusCode(HTTPTypes::RESPONSE_CODE_FORBIDDEN);
    writer->getResponse().setStatusMessage(HTTPTypes::RESPONSE_MESSAGE_FORBIDDEN);
    if (http_request->getMethod() != HTTPTypes::REQUEST_METHOD_HEAD) {
	writer->writeNoCopy(FORBIDDEN_HTML_START);
	writer << http_request->getResource();
	writer->writeNoCopy(FORBIDDEN_HTML_FINISH);
    }
    writer->send();
}

void
dyn_content_t::operator() (HTTPRequestPtr &request, TCPConnectionPtr &tcp_conn)
{
    // get the relative resource path for the request
    const std::string relative_path(getRelativeResource(request->getResource()));
    url_repository_t *url = &session_collection.global_url_repository;
    if (session)
	url = &session->url_repository;


    ostringstream stream;
    string mime_type;

    try {
	if (url->dispatch(session_collection, session, relative_path, stream, mime_type)) {
	    HTTPResponseWriterPtr writer(
		HTTPResponseWriter::create(tcp_conn, *request,
					   boost::bind(&TCPConnection::finish,
						       tcp_conn)));
	    writer->getResponse().setContentType(mime_type);
	    //writer->getResponse().addHeader(HTTPTypes::HEADER_LAST_MODIFIED, now);
	    writer->getResponse().setStatusCode(HTTPTypes::RESPONSE_CODE_OK);
	    writer->getResponse().setStatusMessage(HTTPTypes::RESPONSE_MESSAGE_OK);

	    std::string contents = stream.str();
	    if (request->getMethod() != HTTPTypes::REQUEST_METHOD_HEAD) {
		writer->write(contents);
		//writer->writeNoCopy(contents); // XXX doesn't seem to work. Partial buffers written.
	    }
	    writer->send();

	} else {
	    throw 404;
	}

    } catch (int error_message) {
	switch (error_message) {
	case 403: // 403 forbidden == No valid license
	    send_license_error_response(request, tcp_conn);
	    break;

	case 404: // 404 not found
	default:
	    send_not_found_response(request, tcp_conn);
	    break;
	}
    }
}

END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;
