#include "link_factory_ajax.hh"
#include "url_repository.hh"

#include "../html/macros.hh"
#include "../html/issue.hh"

#include "analysis/report/report.hh"

#include <boost/foreach.hpp>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);
BEGIN_NAMESPACE(server);

// ----------------------------------------------------------------------

void
tab_factory_jquery_t::begin(xml_file_t &xml, const std::string &id)
{
    this->id = id;
}

void
tab_factory_jquery_t::end(xml_file_t &xml)
{
    T_DIV A_ID(id) {
	T_UL {
	    BOOST_FOREACH(tab_t &tab, tab_vector) {
		tab.tag = xml.unique_id();
		T_LI T_A A_HREF("#" + tab.tag) T_SPAN TEXT(tab.title);
	    }
	}
	BOOST_FOREACH(tab_t &tab, tab_vector) {
	    T_DIV A_ID(tab.tag) tab.content_function(xml);
	}
    }
}

void
tab_factory_jquery_t::new_tab(xml_file_t &xml,
			      const std::string &title,
			      content_function_t content_function)
{
    tab_vector.push_back(tab_t(title, content_function));
}

// ----------------------------------------------------------------------

std::string
image_factory_dynamic_t::static_image_path(const std::string &image_name)
{
    return "/images/" + image_name;
}

std::string
image_factory_dynamic_t::global_diagram_path(const std::string &image_name)
{
    return "d/diagram/" + image_name + "/global/0";
}

std::string
image_factory_dynamic_t::global_gauge_path(const std::string &image_name)
{
    return "d/gauge/" + image_name;
}

// ----------------------------------------------------------------------

link_factory_ajax_t::link_factory_ajax_t(url_repository_t &url_repository)
    : url_repository(url_repository)
{
}

void
link_factory_ajax_t::add_source_link(xml_file_t &xml)
{
}

void
link_factory_ajax_t::add_section_toggle(xml_file_t &xml)
{
}

tab_factory_ptr_t
link_factory_ajax_t::tab_factory()
{
    return tab_factory_ptr_t(new tab_factory_jquery_t);
}

image_factory_ptr_t
link_factory_ajax_t::image_factory()
{
    return image_factory_ptr_t(new image_factory_dynamic_t);
}

void
link_factory_ajax_t::emit_sliced_statistics_link(xml_file_t &xml,
						 html_context_t &html_context,
						 rpt_sliced_stats_id_t stats_id,
						 const tp_field_t &field)
{
    const rpt_report_t &report = *(html_context.report);
    const rpt_sliced_stats_t &stats = report.get_sliced_stats(stats_id);
    string stats_id_str = lexical_cast<string>(stats_id);
    string action =
	"javascript:stat(" + stats_id_str + "," + field.code() + ")";
    T_SPAN {
	if (html_context.feature_mt_advice)
	    A_ONCLICK(action) A_TITLE("Click for per-thread statistics")
		field.emit(xml, report.get_sliced_stats_global(stats));
	else
	    A_ONCLICK(action)
		field.emit(xml, report.get_sliced_stats_global(stats));
    }
}

void
link_factory_ajax_t::emit_sliced_statistics_link_by_kind(
    xml_file_t &xml,
    html_context_t &html_context,
    rpt_sliced_stats_id_t stats_id,
    const tp_field_by_kind_t &field)
{
    const rpt_report_t &report = *(html_context.report);
    const rpt_sliced_stats_t &stats = report.get_sliced_stats(stats_id);
    string stats_id_str = lexical_cast<string>(stats_id);
    string action =
	"javascript:stat_by_kind(" + stats_id_str + "," + field.code() + ")";
    const by_kind_value_t total_by_kind =
        field.by_kind(report.get_sliced_stats_global(stats));
    const char *title =
        html_context.feature_mt_advice ?
        "Click for per-thread statistics" :
        "Click for categorized breakdown";

    A_ONCLICK(action) A_TITLE(title)
        field.emit (xml, total_by_kind.total());
}

std::string
link_factory_ajax_t::get_diagram_link(const std::string &type,
				      const std::string &kind,
				      int id)
{
    return "d/diagram/" + type + "/" + kind + "/" + lexical_cast<string>(id);
}

std::string
link_factory_ajax_t::get_help_link(const std::string &section)
{
    return "/doc/manual_html/" + section;
}

END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;
