#ifndef ACURATE_LINK_FACTORY_AJAX_HH
#define ACURATE_LINK_FACTORY_AJAX_HH

#include "url_repository.hh"
#include "../link_factory.hh"
#include "analysis/report/report.hh"

#include <vector>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);
BEGIN_NAMESPACE(server);

// ----------------------------------------------------------------------

/// An abstract factory for creating tabs for jquery.ui
class tab_factory_jquery_t : public tab_factory_t {
public:
    virtual void begin(xml_file_t &xml, const std::string &id);
    virtual void end(xml_file_t &xml);

    virtual void new_tab(xml_file_t &xml,
			 const std::string &title,
			 content_function_t content_function);

private:
    std::string id;

    struct tab_t {
	tab_t(std::string title,
	      content_function_t content_function)
	    : title(title),
	      content_function(content_function)
	    {}

	std::string tag;
	std::string title;
	content_function_t content_function;
    };

    typedef std::vector<tab_t> tab_vector_t;
    tab_vector_t tab_vector;
};

// ----------------------------------------------------------------------

class image_factory_dynamic_t : public image_factory_t {
public:
    std::string static_image_path(const std::string &image_name);
    std::string global_diagram_path(const std::string &image_name);
    std::string global_gauge_path(const std::string &image_name);
};

// ----------------------------------------------------------------------

/// A specific factory to defer all links to request loading the stuff
/// using AJAX when the user clicks on the link.
class link_factory_ajax_t : public acumem::acurate::link_factory_t {
public:
    /// Construct this factory with a reference to the repository for
    /// storing (URL -> content provider) mappings.
    link_factory_ajax_t(url_repository_t &url_repository);

    virtual ~link_factory_ajax_t() {}

    /// Add a link that populates the source code frame
    virtual void add_source_link(xml_file_t &xml);

    /// Add code to toggle a section.
    virtual void add_section_toggle(xml_file_t &xml);

    /// Get a subfactory for adding tabs
    virtual tab_factory_ptr_t tab_factory();

    /// Get a subfactory for emitting images
    virtual image_factory_ptr_t image_factory();

    virtual void emit_sliced_statistics_link(xml_file_t &xml,
					     html_context_t &html_context,
					     rpt_sliced_stats_id_t stats_id,
					     const tp_field_t &field);

    /// Write a link (and other needed data) to represent a drill-down
    /// statistics table.
    virtual void emit_sliced_statistics_link_by_kind(
	xml_file_t &xml,
	html_context_t &html_context,
	rpt_sliced_stats_id_t stats_id,
	const tp_field_by_kind_t &field);

    virtual std::string get_diagram_link(const std::string &type,
					 const std::string &kind,
					 int id);

    /// Get the url to a help section
    virtual std::string get_help_link(const std::string &section);
private:
    url_repository_t &url_repository;
};


END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;

#endif
