#ifndef ACURATE_SERVER_SESSION_HH
#define ACURATE_SERVER_SESSION_HH

#include "library/lang/common_macros.h"

#include "link_factory_ajax.hh"
#include "url_repository.hh"
#include "../file_repos.hh"
#include "../loop_info.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);
BEGIN_NAMESPACE(server);

class shutdown_manager_t;

class session_t {
public:

    session_t(html_context_t &html_context,
	      url_repository_t &url_repository,
	      const std::string &id)
	: url_repository(url_repository),
	  html_context(html_context),
	  id(id)
	{}

    url_repository_t &url_repository;
    html_context_t &html_context;
    string id;
};

typedef boost::shared_ptr<session_t> session_ptr_t;

class session_collection_t {
public:
    session_collection_t(license_manager_ptr_t license,
			 shutdown_manager_t &shutdown_manager)
	: license(license),
	  shutdown_manager(shutdown_manager)
	{
	    server_port = -1;
	    allow_many_logins = false;
	}

    void shutdown();

    /// List of sessions handled by this server
    typedef std::vector<session_ptr_t> session_vector_t;
    session_vector_t session_vector;

    /// Url repository for global objects
    url_repository_t global_url_repository;

    /// This is the port where the server is listening.
    int server_port;

    /// This is the license object.
    license_manager_ptr_t license;

    /// Whether to allow more logins than one
    bool allow_many_logins;

private:
    /// To signal that the server should be shutdown
    shutdown_manager_t &shutdown_manager;
};

END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;



#endif
