// adapted from:

// ------------------------------------------------------------------
// pion-net: a C++ framework for building lightweight HTTP interfaces
// ------------------------------------------------------------------
// Copyright (C) 2007-2008 Atomic Labs, Inc.  (http://www.atomiclabs.com)
//
// Distributed under the Boost Software License, Version 1.0.
// See http://www.boost.org/LICENSE_1_0.txt
//

#ifndef INCLUDE_SHUTDOWNMANAGER_HH
#define INCLUDE_SHUTDOWNMANAGER_HH

#include "library/lang/common_macros.h"

#include <boost/thread/mutex.hpp>
#include <boost/thread/condition.hpp>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);
BEGIN_NAMESPACE(server);

///
/// ShutdownManager: used to manage shutdown for the main thread
///
class shutdown_manager_t {
public:
	// default constructor & destructor
	shutdown_manager_t(void) : m_shutdown_now(false) {}
	~shutdown_manager_t() {}

	/// signals the shutdown condition
	inline void shutdown(void) {
		boost::mutex::scoped_lock shutdown_lock(m_shutdown_mutex);
		m_shutdown_now = true;
		m_shutdown_cond.notify_all();
	}

	/// blocks until the shutdown condition has been signaled
	inline void wait(void) {
		boost::mutex::scoped_lock shutdown_lock(m_shutdown_mutex);
		while (! m_shutdown_now)
			m_shutdown_cond.wait(shutdown_lock);
	}

private:
	/// true if we should shutdown now
	bool					m_shutdown_now;

	/// used to protect the shutdown condition
	boost::mutex			m_shutdown_mutex;

	/// triggered when it is time to shutdown
	boost::condition		m_shutdown_cond;
};

END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;

#endif
