#ifndef ACURATE_URL_REPOSITORY_HH
#define ACURATE_URL_REPOSITORY_HH

#include "library/lang/common_macros.h"

#include <map>
#include <string>
#include <boost/function.hpp>
#include <ostream>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);
BEGIN_NAMESPACE(server);

// External and forward declarations
class session_t;
typedef boost::shared_ptr<session_t> session_ptr_t;
class session_collection_t;

/// Definition of the signature of a function that an url can be dispatched to
typedef boost::function<void (session_collection_t &session_collection,
			      session_ptr_t session,
			      const std::string &relative_url,
			      std::ostream &os,
			      std::string &mime_type)> dispatch_function_t;

/// Repository of mappings from an url to a dispatch function.
class url_repository_t {
public:
    /// Add a dispatcher mapping
    void add_dispatcher(const std::string &relative_url,
			dispatch_function_t f);

    /// Dispatch an url to the corresponding dispatch function.
    /// Returns false if not found.  The dispatch function updates
    /// 'os' and 'mime_type'.
    bool dispatch(session_collection_t &session_collection,
		  session_ptr_t session,
		  const std::string &relative_url,
		  std::ostream &os,
		  std::string &mime_type);

private:
    // XXX Add mutex here for concurrent access and protection of the
    // dispatch map.

    typedef std::map<std::string, dispatch_function_t> dispatch_map_t;
    dispatch_map_t dispatch_map;
};

END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;

#endif
