#ifndef ACURATE_SERVER_WEBSERVER_HH
#define ACURATE_SERVER_WEBSERVER_HH

#include "library/lang/common_macros.h"
#include "session.hh"

#include <boost/asio/deadline_timer.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/system/error_code.hpp>

namespace pion {
    namespace net {
	class WebServer;
	class HTTPAuth;
    }
}

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);
BEGIN_NAMESPACE(server);

typedef boost::shared_ptr<pion::net::WebServer> WebServer_ptr_t;
typedef boost::shared_ptr<pion::net::HTTPAuth> HTTPAuthPtr;

class webserver_t
{
public:
    webserver_t(int port,
		session_collection_t &session_collection,
		int inactivity_timeout);
    ~webserver_t();

    void add_session(session_ptr_t session);

    void start();
    void stop();

private:

    session_collection_t &session_collection;
    bool started;
    WebServer_ptr_t webserver;
    int inactivity_timeout;
    HTTPAuthPtr auth;

    boost::shared_ptr<boost::asio::deadline_timer> inactivity_timer;

    void recharge_timer(bool initial_boost = false);

    void keep_alive(session_collection_t &session_collection,
		    session_ptr_t session,
		    const std::string &relative_url,
		    std::ostream &os,
		    std::string &mime_type);

    void title(session_collection_t &session_collection,
	       session_ptr_t session,
	       const std::string &relative_url,
	       std::ostream &os,
	       std::string &mime_type);

    void topleft(session_collection_t &session_collection,
		 session_ptr_t session,
		 const std::string &relative_url,
		 std::ostream &os,
		 std::string &mime_type);

    void drill_down_table(session_collection_t &session_collection,
			  session_ptr_t session,
			  const std::string &relative_url,
			  std::ostream &os,
			  std::string &mime_type);

    void drill_down_table_by_kind(session_collection_t &session_collection,
				  session_ptr_t session,
				  const std::string &relative_url,
				  std::ostream &os,
				  std::string &mime_type);


    void display_issue(session_collection_t &session_collection,
		       session_ptr_t session,
		       const std::string &relative_url,
		       std::ostream &os,
		       std::string &mime_type);

    void display_loop(session_collection_t &session_collection,
		      session_ptr_t session,
		      const std::string &relative_url,
		      std::ostream &os,
		      std::string &mime_type);

    void display_source(session_collection_t &session_collection,
			session_ptr_t session,
			const std::string &relative_url,
			std::ostream &os,
			std::string &mime_type);

    void display_diagram(session_collection_t &session_collection,
			 session_ptr_t session,
			 const std::string &relative_url,
			 std::ostream &os,
			 std::string &mime_type);

    void display_gauge(session_collection_t &session_collection,
		       session_ptr_t session,
		       const std::string &relative_url,
		       std::ostream &os,
		       std::string &mime_type);

    void front(session_collection_t &session_collection,
	       session_ptr_t session,
	       const std::string &relative_url,
	       std::ostream &os,
	       std::string &mime_type);

    void sessions(session_collection_t &session_collection,
		  session_ptr_t session,
		  const std::string &relative_url,
		  std::ostream &os,
		  std::string &mime_type);

    void timeout(const boost::system::error_code &ec);
};

typedef shared_ptr<webserver_t> webserver_ptr_t;

END_NAMESPACE;
END_NAMESPACE;
END_NAMESPACE;

#endif
