/// @file tools/acurate/view_feedback.cc

// Copyright (c) 2011 Acumem AB.  All rights reserved.

#include "tools/acurate/view_feedback.hh"

#include <iostream>

view_feedback_t::view_feedback_t(license_manager_ptr_t license)
    : licensed_feedback_t(license),
      last_was_minor(false)
{
}

void
view_feedback_t::conditional_new_line()
{
    if (last_was_minor)
        cerr << endl;
    last_was_minor = false;
}

int
view_feedback_t::add_major_step(const std::string &message)
{
    phases.push_back(message);
    return (int)(phases.size() - 1);
}

void
view_feedback_t::select_major_step(int step)
{
    assert(step >= 0);
    assert(step < (int) phases.size());

    test_cancel();
    conditional_new_line();
    cerr << "Step " << step + 1 << ": " << phases[step] << endl;
    steps = 0;
}

void
view_feedback_t::set_minor_steps(int s)
{
    steps = s;
}

void
view_feedback_t::update(int completed_minor_steps, const std::string &message)
{
    test_cancel();
    if (completed_minor_steps != steps) {
        cerr << "\r"
             << "(" << completed_minor_steps << "/" << steps << ")   "
             << message;
        last_was_minor = true;
    } else {
        cerr << "\r(done) " << message
             << "                               " << endl;
        last_was_minor = false;
    }

    cerr.flush();
}

void
view_feedback_t::log(log_type_t type, const string &message)
{
    test_cancel();
    conditional_new_line();
    switch (type) {
    case LOG_WARNING:
        cerr << "Warning: " << message << endl;
        break;

    case LOG_ERROR:
        cerr << "Error: " << message << endl;
        break;

    case LOG_INFO:
        cerr << "\t" << message << endl;
        break;

    default:
        break;
    }
}
