/// @file tools/acurate/xml_file.cc

// Copyright (c) 2007-2009 Acumem AB.  All rights reserved.

#include "xml_file.hh"

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);

using namespace std;

static string blanks(int l)
{
    string x;
    while(l-- > 0) {
	x += ' ';
    }
    return x;
}


xml_file_t::xml_file_t(ostream &o, doctype_t doctype)
    : os(o)
{
    indent = 0;
    num_tags = 0;

    switch (doctype) {
    case Doctype_Strict:
	os << "<!DOCTYPE html " << endl
	   << "     PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"" << endl
	   << "     \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">"
	   << endl;
	break;

    case Doctype_Transitional:
	os << "<!DOCTYPE html " << endl
	   << "     PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\"" << endl
	   << "     \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">"
	   << endl;
	break;

    case Doctype_Frameset:
	os << "<!DOCTYPE html " << endl
	   << "     PUBLIC \"-//W3C//DTD XHTML 1.0 Frameset//EN\"" << endl
	   << "     \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-frameset.dtd\">"
	   << endl;
	break;

    default:
	break;
    }

    saved_nl = true;
}

bool
xml_file_t::emit_start_tag(bool close, bool nl)
{
    if (last_tag.size() == 0)
	return false;

    if (allow_nl())
	os << blanks(indent);

    num_tags ++;
    os << "<" << last_tag;
    BOOST_FOREACH(attributes_t::value_type &v, last_attributes) {
	os << " " << v.first << "=\"" << v.second << "\"";
    }
    if (close)
	os << "/>";
    else {
	os << ">";
	indent ++;
    }
    if (nl && allow_nl())
	os << endl;

    last_tag = "";

    return true;
}

xml_file_t &
xml_file_t::start_tag(const string &tag_name)
{
    tags.push_back(tag_name);
    emit_start_tag(false, true);
    last_tag = tag_name;
    last_attributes.clear();
    return *this;
}

xml_file_t &
xml_file_t::end_tag()
{
    if (!emit_start_tag(true, saved_nl)) {
	indent--;
	if (!allow_nl())
	     os << "</" << tags.back() << ">";
	else {
	    if (saved_nl)
		os << blanks(indent) << "</" << tags.back() << ">" << endl;
	    else
		os << "</" << tags.back() << ">" << endl;
	}
	saved_nl = true;
	num_tags++;
    }

    tags.pop_back();
    return *this;
}

xml_file_t &
xml_file_t::add_text(const string &s, bool nl)
{
    string r;

    BOOST_FOREACH(char ch, s) {
	if (ch == '>')
	    r += "&gt;";
	else if (ch == '<')
	    r += "&lt;";
	else if (ch == '&')
	    r += "&amp;";
	else if (ch == '"')
	    r += "&quot;";
	else r += ch;
    }

    emit_start_tag(false, nl);
    saved_nl = nl;
    if (nl && allow_nl())
	os << blanks(indent) << r << endl;
    else
	os << r;
    return *this;
}

xml_file_t &
xml_file_t::add_text_t(const string &s, bool nl)
{
    emit_start_tag(false, nl);
    saved_nl = nl;
    if (nl && allow_nl())
	os << blanks(indent) << s << endl;
    else
	os << s;
    return *this;
}


int
xml_file_t::add_attribute(const string &k, const string &v)
{
    if (v.size())
	last_attributes[k] = v;
    return 0;
}

void
xml_file_t::push_nl(bool n)
{
    nl_stack.push_back(n);
}

void
xml_file_t::pop_nl()
{
    nl_stack.pop_back();
    os << endl;
}

bool
xml_file_t::allow_nl()
{
    if (nl_stack.size() == 0)
	return true;
    return nl_stack.back();
}

END_NAMESPACE;
END_NAMESPACE;
