/// @file tools/acurate/xml_file.hh

// Copyright (c) 2007, 2009 Acumem AB.  All rights reserved.

#ifndef INCLUDED_XML_FILE_HH
#define INCLUDED_XML_FILE_HH

#include "library/lang/common_macros.h"

#include <iosfwd>
#include <string>
#include <map>
#include <vector>
#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(acurate);

enum doctype_t {
    Doctype_None,
    Doctype_Strict,
    Doctype_Transitional,
    Doctype_Frameset,
};

class xml_file_t {
public:
    xml_file_t(std::ostream &o, doctype_t doctype);

    xml_file_t &start_tag(const std::string &tag_name);
    xml_file_t &end_tag();
    xml_file_t &add_text(const std::string &text, bool nl = true);
    xml_file_t &add_text_t(const std::string &text, bool nl = true);

    int add_attribute(const std::string &k, const std::string &v);
    void push_nl(bool n);
    void pop_nl();

    std::string unique_id() const {
	return "i" + boost::lexical_cast<std::string>(num_tags);
    }

private:
    bool allow_nl();
    bool emit_start_tag(bool close, bool nl);

    std::ostream &os;
    std::vector<std::string> tags;
    std::string last_tag;

    typedef std::map<std::string, std::string> attributes_t;
    attributes_t last_attributes;

    int indent;
    bool saved_nl;

    std::vector<bool> nl_stack;
    int num_tags;
};

class xml_file_feature_t {
public:
    xml_file_feature_t(xml_file_t &xml)
	: xml(xml)
	{
	    done_flag = false;
	}

    void done()
	{
	    done_flag = true;
	}

    bool get()
	{
	    return !done_flag;
	}

protected:
    xml_file_t &xml;

private:
    bool done_flag;
};

class element : public xml_file_feature_t {
public:
    element(xml_file_t &xml, const std::string &tag)
	: xml_file_feature_t(xml)
	{
	    xml.start_tag(tag);
	}

    ~element()
	{
	    xml.end_tag();
	}
};

class attribute : public xml_file_feature_t {
public:
    attribute(xml_file_t &xml, const std::string &key, const std::string &value)
	: xml_file_feature_t(xml)
	{
	    xml.add_attribute(key, value);
	}
};

class nl : public xml_file_feature_t {
public:
    nl(xml_file_t &xml, bool n)
	: xml_file_feature_t(xml)
	{
	    xml.push_nl(n);
	}

    ~nl()
	{
	    xml.pop_nl();
	}
};

END_NAMESPACE;
END_NAMESPACE;

#endif
