/// @file tools/report/report_feedback.hh

// Copyright (c) 2011 Acumem AB.  All rights reserved.

#ifndef REPORT_FEEDBACK_HH
#define REPORT_FEEDBACK_HH

#include "library/feedback/licensed_feedback.hh"

#include <iostream>

using namespace acumem::library;
using namespace acumem::license;

BEGIN_NAMESPACE(acumem);
BEGIN_NAMESPACE(report);

class report_feedback_t : public licensed_feedback_t {
public:
    report_feedback_t(license_manager_ptr_t license)
        : licensed_feedback_t(license),
          last_was_minor(false)
        {}

    void conditional_new_line() {
	if (last_was_minor)
	    cout << endl;
	last_was_minor = false;
    }

    int add_major_step(const std::string &message) {
	phases.push_back(message);
	return (int)(phases.size() - 1);
    }

    void select_major_step(int step) {
	assert(step >= 0);
	assert(step < (int) phases.size());

	test_cancel();
	conditional_new_line();
	cout << "Step " << step + 1 << ": " << phases[step] << endl;
	steps = 0;
    }

    void set_minor_steps(int s) {
	steps = s;
    }

    void update(int completed_minor_steps, const std::string &message) {
	test_cancel();
	if (completed_minor_steps != steps) {
	    cout << "\r"
		 << "(" << completed_minor_steps << "/" << steps << ")   "
		 << message;
	    last_was_minor = true;
	} else {
	    cout << "\r(done) " << message
		 << "                               " << endl;
	    last_was_minor = false;
	}

	cout.flush();

    }

    virtual void log(log_type_t type, const string &message) {
	test_cancel();
	conditional_new_line();
	switch (type) {
	case LOG_WARNING:
	    cerr << "Warning: " << message << endl;
	    break;

	case LOG_ERROR:
	    cerr << "Error: " << message << endl;
	    break;

	case LOG_INFO:
	    cout << "\t" << message << endl;
	    break;

	default:
	    break;
	}
    }
    bool last_was_minor;
    int steps;
    vector<string> phases;
};

END_NAMESPACE;
END_NAMESPACE;

#endif /* REPORT_FEEDBACK_HH */
