/// @file tools/report_printer/report_printer.cc

// Copyright (c) 2011 Acumem AB.  All rights reserved.

#if !defined(PRODUCT)
#define PRODUCT PRODUCT_INTERNAL
#endif

#include "analysis/report_codec/report_codec.hh"
#include "analysis/report/report_printer.hh"
#include "library/stream/stream_container.hh"
#include "library/stream/stream_file.hh"
#include "product.h"

#include <boost/program_options.hpp>

#include <iostream>

#include <stdio.h>

namespace po = boost::program_options;

using namespace acumem::analysis;

struct conf_t {
    conf_t(int argc, char **argv);

    string in_file;
};

conf_t::conf_t(int argc, char **argv)
{
    po::options_description opt_desc_generic("Generic options");
    opt_desc_generic.add_options()
        ("help", "Print this help message.")
        ("report,i", po::value<string>(&in_file), "Report file to read.")
        ;

    po::options_description opt_desc_invisible("Options");
    opt_desc_invisible.add_options()
        ("positional-args", "")
        ;

    po::options_description opt_desc_visible("Options");
    opt_desc_visible.add(opt_desc_generic);

    po::options_description opt_desc_all("Options");
    opt_desc_all.add(opt_desc_visible)
        .add(opt_desc_invisible);

    po::positional_options_description pos_desc;
    pos_desc.add("positional-args", -1);

    po::variables_map vm;
    try {
        po::store(po::command_line_parser(argc, argv)
                  .options(opt_desc_all).positional(pos_desc).run(), vm);
        po::notify(vm);
    }
    catch (po::error &e) {
        cerr << "Error: " << e.what() << endl;
        exit(1);
    }

    if (vm.count("positional-args") > 0) {
        cerr << "Error: no positional arguments allowed" << endl;
        exit(1);
    }

    if (vm.count("help") > 0) {
        cout << opt_desc_visible << endl;
        exit(0);
    }

    if (vm.count("report") == 0) {
        cerr << "Error: report file not specified" << endl;
        exit(1);
    }
}

int
main(int argc, char **argv)
{
    conf_t conf(argc, argv);

    warning_consumer_ptr_t wc(new warning_consumer_stderr_t());
    report_decoder_t in_decoder(wc);
    stream_container_reader_t in_container(&in_decoder, wc);
    stream_file_reader_t in_file(conf.in_file.c_str(), &in_container);
    in_file.dispatch();
    rpt_report_cptr_t report = in_decoder.get_report();

    print_report(stdout, *report);

    return 0;
}
