/// @file tools/sample_gui/cpu_selector.hh

// Copyright (c) 2008-2009 Acumem AB.  All rights reserved.

#ifndef CPU_SELECTOR_HH
#define CPU_SELECTOR_HH

#include <gtk/gtk.h>
#include <string>

#include "library/cpudb/cpudb.hh"

// Sampling line size selection dialog box.
class cpu_selector_t {
public:
    cpu_selector_t(const acumem::library::cpudb_cptr_t &cpudb,
		   bool feature_mt_advice,
		   bool feature_multisocket,
		   GtkWidget *parent_win);
    ~cpu_selector_t();

    // Show the dialog box.
    bool show();

    /// @return id on the form mfgr/cpu or auto.
    std::string get_selected_id();
    const acumem::library::cpudb_cpu_t *get_selected_cpu();
    std::string get_selected_display_name();

    unsigned int get_num_cpus() const;

    void update_cpu_list();

private:
    enum {
	COLUMN_CPU_DATA,
	COLUMN_CPU_ID,
	COLUMN_CPU_NAME,
	N_CPU_COLUMNS
    };

    enum {
	COLUMN_MFGR_DATA,
	COLUMN_MFGR_NAME,
	N_MFGR_COLUMNS
    };

    static void on_mfgr_changed(GtkComboBox *widget, gpointer user_data);
    static void on_cpu_changed(GtkTreeSelection *selection, gpointer user_data);

    void append_mfgr(const acumem::library::cpudb_manufacturer_cptr_t &mfgr);
    void append_mfgr_auto();
    void append_cpu(const acumem::library::cpudb_cpu_cptr_t &cpu);

    GtkDialog *dialog;
    GtkTreeView *cpu_list;
    GtkComboBox *mfgr_list;

    GtkListStore *list_store_mfgr;
    GtkListStore *list_store_cpu;

    GtkSpinButton *ncpu_sbutton;

    const acumem::library::cpudb_cptr_t cpudb;
    acumem::library::cpudb_cpu_cptr_t autodetected_cpu;

    bool feature_mt_advice;
    bool feature_multisocket;

    bool autodetect_failed;
};

#endif /* CPU_SELECTOR_HH */
