/// @file tools/sample_gui/line_size_selector.cc

// Copyright (c) 2007-2008 Acumem AB.  All rights reserved.

#include "line_size_selector.hh"
#include "library/version/product_version.h"

#include <sstream>

#include <math.h>
#include <assert.h>

using namespace std;

#define NUM_LINE_SIZES 13

line_size_selector_t::line_size_selector_t(GtkWidget *parent_win)
    : analysis_line_size2(0)
{
    dialog = gtk_dialog_new_with_buttons(
        "Select cache line sizes to sample - " PRODUCT_VERSION,
        GTK_WINDOW(parent_win),
        GTK_DIALOG_MODAL,
        GTK_STOCK_OK, GTK_RESPONSE_OK,
        NULL);

    GtkWidget *label = gtk_label_new("Select cache line sizes to sample");
    gtk_widget_show(label);
    gtk_box_pack_start_defaults(
        GTK_BOX(GTK_DIALOG(dialog)->vbox), label);

    for (int i = 0; i < NUM_LINE_SIZES; i++) {
        stringstream ss;
        ss << (1 << i);
        string label = ss.str() + (i == 0 ? " byte" : " bytes");
        GtkWidget *ls = gtk_check_button_new_with_label(label.c_str());
        line_size_cbutton[i] = ls;
        switch (i) {
        case 0:
        case 12:
            /* Can not allow sampling of 1-byte or 4k-byte lines. See ticket
               #694. */
            break;
        case 6:
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ls), TRUE);
	    /* FALL THROUGH */
        default:
            gtk_widget_show(ls);
            break;
        }
        gtk_box_pack_start_defaults(
            GTK_BOX(GTK_DIALOG(dialog)->vbox), ls);
    }
}

void
line_size_selector_t::show() {
    gtk_widget_show(dialog);
    gint response = gtk_dialog_run(GTK_DIALOG(dialog));
    switch (response) {
    case GTK_RESPONSE_OK:
        break;
    case GTK_RESPONSE_DELETE_EVENT:
        break;
    default:
        assert(0);
    }
    gtk_widget_hide(dialog);
}



string
line_size_selector_t::line_sizes() {
    string s;
    for (int i = 0; i < 13; i++)
        if (gtk_toggle_button_get_active(
                GTK_TOGGLE_BUTTON(line_size_cbutton[i]))) {
            if (s.size() != 0)
                s += ",";
            stringstream ss;
            ss << (1 << i);
            s += ss.str();
        }
    return s;
}

int
line_size_selector_t::num_line_sizes() {
    int num = 0;
    for (int i = 0; i < 13; i++)
        if (gtk_toggle_button_get_active(
                GTK_TOGGLE_BUTTON(line_size_cbutton[i])))
            num++;
    return num;
}

void
line_size_selector_t::set_analysis_line_size(unsigned int line_size)
{
    if (analysis_line_size2 > 0)
	gtk_widget_set_sensitive(
	    line_size_cbutton[analysis_line_size2], TRUE);

    if (line_size > 0) {
	analysis_line_size2 = (unsigned int)log2(line_size);
	assert(line_size == (1UL << analysis_line_size2));
	assert(analysis_line_size2 < NUM_LINE_SIZES);

	gtk_widget_set_sensitive(
	    line_size_cbutton[analysis_line_size2], FALSE);

	gtk_toggle_button_set_active(
	    GTK_TOGGLE_BUTTON(line_size_cbutton[analysis_line_size2]), TRUE);

    } else
	analysis_line_size2 = 0;
}


line_size_selector_t::~line_size_selector_t()
{
}
