/// @file tools/sample_gui/pid_selector_linux.cc

// Copyright (c) 2007-2008 Acumem AB.  All rights reserved.

#include <unistd.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <stdlib.h>

#include <sstream>
#include <fstream>

#include "pid_selector_linux.hh"

using namespace std;

bool
linux_process_stat_t::load(pid_t pid)
{
    ostringstream fn;
    fn << "/proc/" << pid << "/stat";

    return load(fn.str().c_str());
}

bool
linux_process_stat_t::load(const char* stat_file)
{
    FILE *f = fopen(stat_file, "r");
    char c_state;
    int ret;
    comm[0] = '\0';

    if(!f)
	return false;

    if(fscanf(f, "%d (", &pid) != 1) {
	fclose(f);
	return false;
    }

    for(int i = 0; i < MAX_COMM_LEN; i++) {
	char c = fgetc(f);
	if(c == EOF) {
	    fclose(f);
	    return false;
	} else if(c == ')') {
	    comm[i] = '\0';
	    break;
	} else {
	    comm[i] = c;
	}
    }

    ret = fscanf(f, " %c %d %d %d %d %d %u %lu %lu %lu "
		 "%lu %lu %lu %ld %ld %ld %ld %d 0 %llu %lu %ld %lu %lu %lu "
		 "%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %d %d %u %u",
		 &c_state, &ppid, &pgrp, &session, &ttyp_nr, &tpgid,
		 &flags, &minflt, &cminflt, &majflt, &cmajflt, &utime, &stime,
		 &cutime, &cstime, &priority, &nice, &num_threads,
		 &starttime, &vsize, &rss, &rlim, &startcode, &endcode,
		 &startstack, &kstkesp, &kstkeip, &signal, &blocked, &sigignore,
		 &sigcatch, &wchan, &nswap, &cnswap, &exit_signal, &processor,
		 &rt_priority, &policy);

    fclose(f);

    state = (linux_process_state_t)c_state;

    return ret == 38;
}


uid_t pid_selector_linux_t::get_pid_uid(int pid)
{
    ostringstream fn;
    struct stat buf;
    fn << "/proc/" << pid;

    if(stat(fn.str().c_str(), &buf))
       return 0;

    return buf.st_uid;
}

pid_selector_linux_t::pid_selector_linux_t(GtkWidget *parent_win)
  : pid_selector_base_t(parent_win)
{
}

pid_selector_linux_t::~pid_selector_linux_t()
{
}

void
pid_selector_linux_t::get_cmd_line(int pid, char *buf, int max_len)
{
    ostringstream fn;
    fn << "/proc/" << pid << "/cmdline";
    ifstream f_cmd(fn.str().c_str());

    buf[0] = '\0';
    f_cmd.read(buf, max_len);
}

bool
pid_selector_linux_t::update_list() {
    struct dirent *dp;
    DIR *d = opendir("/proc/");
    uid_t me = getuid();
    char cmd[1024];

    linux_process_stat_t ps;

    gtk_list_store_clear(list_store);

    if(!d) {
	display_error(errno, "Failed to open /proc/");
	return false;
    }

    do {
	errno = 0;
	if((dp = readdir(d)) != NULL) {
	    if(is_str_int(dp->d_name)) {
		int pid = atoi(dp->d_name);

		if(me != 0 && get_pid_uid(pid) != me)
		    continue;

		if(ps.load(pid)) {
		    if(ps.state == PROCESS_STATE_ZOMBIE ||
		       ps.state == PROCESS_STATE_STOPPED)
			continue;

		    get_cmd_line(pid, cmd, sizeof(cmd));
		    append_process(pid, cmd);
		}
	    }
	}
    } while(dp);
    if(errno != 0) {
	display_error(errno, "Failed to read directory entry in /proc/.");
	closedir(d);
	return false;
    }

    closedir(d);
    return true;
}
