/// @file tools/sample_gui/pid_selector_linux.hh

// Copyright (c) 2007 Acumem AB.  All rights reserved.

#ifndef PID_SELECTOR_LINUX_HH
#define PID_SELECTOR_LINUX_HH

#include "pid_selector.hh"

#define MAX_COMM_LEN 512

enum linux_process_state_t {
    PROCESS_STATE_UNKNOWN = 0,
    PROCESS_STATE_RUNNING = 'R',
    PROCESS_STATE_SLEEPING = 'S',
    PROCESS_STATE_WAITING = 'D',
    PROCESS_STATE_ZOMBIE = 'Z',
    PROCESS_STATE_STOPPED = 'T',
    PROCESS_STATE_PAGING = 'W'
};

struct linux_process_stat_t {
    bool load(pid_t pid);
    bool load(const char* stat_file);

    pid_t pid;
    char comm[MAX_COMM_LEN];
    linux_process_state_t state;
    pid_t ppid;
    int pgrp;
    int session;
    int ttyp_nr;
    int tpgid;
    unsigned int flags;
    unsigned long int minflt;
    unsigned long int cminflt;
    unsigned long int majflt;
    unsigned long int cmajflt;
    unsigned long int utime;
    unsigned long int stime;
    long int cutime;
    long int cstime;
    long int priority;
    long int nice;
    int num_threads;
    unsigned long long int starttime;
    unsigned long int vsize;
    long int rss;
    unsigned long int rlim;
    unsigned long int startcode;
    unsigned long int endcode;
    unsigned long int startstack;
    unsigned long int kstkesp;
    unsigned long int kstkeip;
    unsigned long int signal;
    unsigned long int blocked;
    unsigned long int sigignore;
    unsigned long int sigcatch;
    unsigned long int wchan;
    unsigned long int nswap;
    unsigned long int cnswap;
    int exit_signal;
    int processor;
    unsigned int rt_priority;
    unsigned int policy;
};

class pid_selector_linux_t
    : public pid_selector_base_t
{
public:
    pid_selector_linux_t(GtkWidget *parent_win);
    ~pid_selector_linux_t();

protected:
    uid_t get_pid_uid(int pid);
    void get_cmd_line(int pid, char *buf, int max_len);
    bool update_list();
};

#endif
