/// @file tools/sample_gui/web_browser.hh

// Copyright (c) 2008 Acumem AB.  All rights reserved.

#ifndef WEB_BROWSER_HH
#define WEB_BROWSER_HH

#include <string>
#include <vector>

struct web_browser_exception_t
{
    web_browser_exception_t(const std::string &what)
	: _what(what) {};

    virtual ~web_browser_exception_t() {
    };

    const virtual std::string &what() {
	return _what;
    }

private:
    std::string _what;
};

struct web_browser_not_found_exception_t
    : public web_browser_exception_t
{
    web_browser_not_found_exception_t()
	: web_browser_exception_t("Could not find a suitable web browser.") {}
};

struct web_browser_launch_failed_exception_t
    : public web_browser_exception_t
{
    web_browser_launch_failed_exception_t(const std::string &what)
	: web_browser_exception_t(what) {}
};

/**
 * Class autodetection and launching of a web browser.
 */
class web_browser_t
{
public:
    web_browser_t();

    /**
     * Open a URL in the web browser. Throws an exception on error.
     *
     * \param url URL to open.
     * \throw litmus_exception_t only exception thrown on error.
     */
    void open_url(const std::string &url) const;

private:
    /// See http://www.catb.org/~esr/BROWSER/
    bool lookup_in_environ();
    /// Uses a well known browser from the list browser_commands in web_browser.cc.
    bool lookup_known();

    /**
     * Performs BROWSER argument substitutions. The following
     * substitutions are defined:
     *  %% - %
     *  %s - URL
     *
     * Undefined substitutions are purged from the string.
     *
     * \param arg argument to perform substitution in.
     * \param url URL to use.
     * \param url_substituted is set to true if the URL was substituted.
     * \return the processed argument.
     */
    static std::string setup_argument(const std::string &arg,
				      const std::string &url,
				      bool &url_substituted);

    std::string binary;
    std::vector<std::string> arguments;
};

#endif
