/// @file tools/trace_printer/trace_printer.cc

// Copyright (c) 2008 Acumem AB.  All rights reserved.

#include "library/version/version.h"
#include "library/fenris/flow_file_reader.hh"
#include "library/fenris/flow_printer.hh"

/* Bring in acumem svn version information */
BRING_IN_VERSIONINFO

#include <fstream>
#include <iostream>

#include <boost/program_options.hpp>

using namespace acumem::library;
using namespace std;
using namespace boost;

namespace po = boost::program_options;

struct conf_t {
    conf_t()
	: in_file(""), out_file(""), container_info(false) {}
    string                  in_file;
    string                  out_file;
    bool                    container_info;
};

static int
parse_args(conf_t &conf, int argc, char **argv)
{
    int ret = 0;

    po::options_description desc("Allowed options");
    desc.add_options()
	("help,h", "produce help message")
	("in-file,i", po::value<string>(&conf.in_file), "specifies the input file. stdin is used if no file is specified")
	("out-file,o", po::value<string>(&conf.out_file), "specifies the output file. stdout is used if no file is specified")
	("container-info,c", "show container information")
	;

    po::variables_map vm;
    po::store(po::parse_command_line(argc, argv, desc), vm);
    po::notify(vm);

    if (vm.count("help") != 0)
	ret += 1;


    conf.container_info = vm.count("container-info") > 0;

    if (ret)
	cout << desc << endl;

    return ret;
}

int
main(int argc, char *argv[])
{
    using namespace acumem::library;
    conf_t conf;

    if (parse_args(conf, argc, argv))
	return 1;

    try {
	ostream *os =
	    (conf.out_file == "" ? &cout : new ofstream(conf.out_file.c_str()));

        warning_consumer_ptr_t warning_consumer(
            new warning_consumer_stderr_t());
	flow_printer_t printer(*os);
	flow_file_reader_t in(conf.in_file == "" ? NULL : conf.in_file.c_str(),
			      &printer,
                              warning_consumer);
	in.dispatch();

        if (conf.container_info)
            *os << in.get_header();

	if (os != &cout)
	    delete os;
    } catch (flow_exception_t fe) {
	cerr << "Caught flow exception:" << endl
	     << fe.what() << endl;
	exit(1);
    } catch (stream_exception_t se) {
	cerr << "Caught stream exception:" << endl
	     << se.what() << endl;
	exit(1);
    }

    return 0;
}
